/* dungeon.c: the main command interpreter, updating player status

   Copyright (c) 1989 James E. Wilson, Robert A. Koeneke

   This software may be copied and distributed for educational, research, and
   not for profit purposes provided that this copyright and statement are
   included in all such copies. */



#include "constant.h"
#include "config.h"
#include "types.h"
#include "externs.h"
#include "monster.h"

#ifdef USG
#ifndef ATARIST_MWC
#include <string.h>
#endif
#else
#include <strings.h>
#endif

static void count_cmd(void);
static char original_commands();
static void do_command();
static int valid_countcommand();
static void regenhp();
static void regenmana();
static int enchanted();
int special_check();
static void examine_book();
static void activate();
static void go_up();
static void go_down();
static void jamdoor();
static void refill_lamp();
static int regen_monsters();

extern int peek;
extern int rating;

/* ANGBAND game module					-RAK-	*/
/* The code in this section has gone through many revisions, and */
/* some of it could stand some more hard work.	-RAK-	       */

/* It has had a bit more hard work.			-CJS- */

int good_item_flag=FALSE;
int create_up_stair=FALSE;
int create_down_stair=FALSE;

void dungeon()
{
  int find_count, i;
  int regen_amount;	    /* Regenerate hp and mana*/
  char command;		/* Last command		 */
  static long old_turn;
  register struct misc *p_ptr;
  register inven_type *i_ptr;
  register struct flags *f_ptr;

  /* Main procedure for dungeon.			-RAK-	*/
  /* Note: There is a lot of preliminary magic going on here at first*/

  /* init pointers. */
  f_ptr = &py.flags;
  p_ptr = &py.misc;

  /* Check light status for setup	   */
  i_ptr = &inventory[INVEN_LIGHT];
  if (i_ptr->p1 > 0 || f_ptr->light)
    player_light = TRUE;
  else
    player_light = FALSE;
  /* Check for a maximum level		   */
  if (dun_level > p_ptr->max_dlv)
    p_ptr->max_dlv = dun_level;

  /* Reset flags and initialize variables  */
  command_count = 0;
  find_count = 0;
  new_level_flag    = FALSE;
  find_flag	= FALSE;
  teleport_flag = FALSE;
  mon_tot_mult	= 0;
  cave[char_row][char_col].cptr = 1;
  if (create_up_stair || create_down_stair) {
    register cave_type *c_ptr;
    register int cur_pos;

    c_ptr = &cave[char_row][char_col];
      if ((c_ptr->tptr == 0) ||
            (t_list[c_ptr->tptr].tval < TV_MIN_WEAR) ||
                  (t_list[c_ptr->tptr].tval > TV_MIN_WEAR) ||
                  !(t_list[c_ptr->tptr].flags2 & TR_ARTIFACT)) { /* if no artifact here -CFT */
        if (c_ptr->tptr != 0)
          (void) delete_object(char_row, char_col);
        cur_pos = popt();
        c_ptr->tptr = cur_pos;
        if (create_up_stair)
          invcopy(&t_list[cur_pos], OBJ_UP_STAIR);
        else if (create_down_stair && !is_quest(dun_level))
          invcopy(&t_list[cur_pos], OBJ_DOWN_STAIR);
        }
        else
          msg_print("The object resists your attempt to transform it into a stairway.");
    create_down_stair = FALSE;
    create_up_stair = FALSE;
  }
  /* Ensure we display the panel. Used to do this with a global var. -CJS- */
  panel_row = panel_col = -1;
  /* Light up the area around character	   */
  check_view ();
  /* must do this after panel_row/col set to -1, because search_off() will call
     check_view(), and so the panel_* variables must be valid before
     search_off() is called */
  if (search_flag)
    search_off();
  /* Light,  but do not move critters	    */
  creatures(FALSE);
  /* Print the depth			   */
  prt_depth();

  if ((good_item_flag || rating) &&
      ((turn-old_turn)>randint(50)) && (old_turn>randint(50)) && dun_level) {

    if (good_item_flag)
      msg_print("You feel there is something special about this level.");
    else if (rating>100)
      msg_print("You have a superb feeling about this level.");
    else if (rating>70)
      msg_print("You have an excellent feeling that your luck is turning...");
    else if (rating>40)
      msg_print("You have a very good feeling.");
    else if (rating>30)
      msg_print("You have a good feeling.");
    else if (rating>22)
      msg_print("You feel strangely lucky.");
    else if (rating>15)
      msg_print("You feel your luck is turning...");
    else if (rating>10)
      msg_print("You like the look of this place.");
  }
  old_turn = turn;

  /* Loop until dead,  or new level		*/
  do
    {
      /* Increment turn counter			*/
      turn++;
#ifndef MAC
      /* The Mac ignores the game hours file		*/
      /* Check for game hours			       */
      if (((turn % 100) == 1) && !check_time())
	if (closing_flag > 2)
	  {
	    msg_print("The gates to ANGBAND are now closed.");
	    (void) strcpy (died_from, "(closing gate: saved)");
	    if (!save_char())
	      {
		(void) strcpy (died_from, "a slammed gate");
		death = TRUE;
	      }
	    exit_game();
	  }
	else
	  {
	    disturb (0, 0);
	    closing_flag++;
	    msg_print("The gates to ANGBAND are closing due to high load.");
	    msg_print("Please finish up or save your game.");
	  }
#endif

      /* turn over the store contents every, say, 1000 turns */
      if ((dun_level != 0) && ((turn % 1000) == 0)) {
	if (peek) msg_print("Store update: ");
	store_maint();
	if (peek) msg_print("Complete ");
      }

      /* Check for creature generation		*/
      if (randint(MAX_MALLOC_CHANCE) == 1)
	alloc_monster(1, MAX_SIGHT, FALSE);
        if (!(turn % 20))
	regen_monsters();
      /* Check light status			       */
      i_ptr = &inventory[INVEN_LIGHT];
      if (player_light)
	if (i_ptr->p1 > 0)
	  {
            if (!(i_ptr->flags2 & TR_LIGHT)) i_ptr->p1--; /* don't dec if perm light -CFT */
	    if (i_ptr->p1 == 0)
	      {
		player_light = FALSE;
		disturb (0, 1);
		/* unlight creatures */
		creatures(FALSE);
		msg_print("Your light has gone out!");
	      }
            else if ((i_ptr->p1 < 40) && (randint(5) == 1) &&
                     (py.flags.blind < 1) &&
                     !(i_ptr->flags2 & TR_LIGHT)) /* perm light doesn't dim -CFT */
	      {
		disturb (0, 0);
		msg_print("Your light is growing faint.");
	      }
	  }
	else
	  {
	    if (!f_ptr->light) {
	      player_light = FALSE;
	      disturb (0, 1);
	      /* unlight creatures */
	      creatures(FALSE);
	    }
	  }
      else if (i_ptr->p1 > 0 || f_ptr->light)
	{
          if (!(i_ptr->flags2 & TR_LIGHT)) i_ptr->p1--; /* don't dec if perm light -CFT */
	  player_light = TRUE;
	  disturb (0, 1);
	  /* light creatures */
	  creatures(FALSE);
	}

      /* Update counters and messages			*/
      /* Check food status	       */
      regen_amount = PLAYER_REGEN_NORMAL;
      if (f_ptr->food < PLAYER_FOOD_ALERT)
	{
	  if (f_ptr->food < PLAYER_FOOD_WEAK)
	    {
	      if (f_ptr->food < 0)
		regen_amount = 0;
	      else if (f_ptr->food < PLAYER_FOOD_FAINT)
		regen_amount = PLAYER_REGEN_FAINT;
	      else if (f_ptr->food < PLAYER_FOOD_WEAK)
		regen_amount = PLAYER_REGEN_WEAK;
	      if ((PY_WEAK & f_ptr->status) == 0)
		{
		  f_ptr->status |= PY_WEAK;
		  msg_print("You are getting weak from hunger.");
		  disturb (0, 0);
		  prt_hunger();
		}
	      if ((f_ptr->food < PLAYER_FOOD_FAINT) && (randint(8) == 1))
		{
		  f_ptr->paralysis += randint(5);
		  msg_print("You faint from the lack of food.");
		  disturb (1, 0);
		}
	    }
	  else if ((PY_HUNGRY & f_ptr->status) == 0)
	    {
	      f_ptr->status |= PY_HUNGRY;
	      msg_print("You are getting hungry.");
	      disturb (0, 0);
	      prt_hunger();
	    }
	}
      /* Food consumption	*/
      /* Note: Speeded up characters really burn up the food!  */
      if (f_ptr->speed < 0)
	f_ptr->food -=	f_ptr->speed*f_ptr->speed;
      f_ptr->food -= f_ptr->food_digested;
      if (f_ptr->food < 0)
	{
	  take_hit (-f_ptr->food/16, "starvation");   /* -CJS- */
	  disturb(1, 0);
	}
      /* Regenerate	       */
      if (f_ptr->regenerate)  regen_amount = regen_amount * 3 / 2;
      if (search_flag || f_ptr->rest > 0 || f_ptr->rest==-1)
	regen_amount = regen_amount * 2;
      if ((py.flags.poisoned < 1) && (py.flags.cut < 1) &&
	  (p_ptr->chp < p_ptr->mhp))
	regenhp(regen_amount);
      if (p_ptr->cmana < p_ptr->mana)
	regenmana(regen_amount);
      /* Blindness	       */
      if (f_ptr->blind > 0)
	{
	  if ((PY_BLIND & f_ptr->status) == 0)
	    {
	      f_ptr->status |= PY_BLIND;
	      prt_map();
	      prt_blind();
	      disturb (0, 1);
	      /* unlight creatures */
	      creatures (FALSE);
	    }
	  f_ptr->blind--;
	  if (f_ptr->blind == 0)
	    {
	      f_ptr->status &= ~PY_BLIND;
	      prt_blind();
	      prt_map();
	      /* light creatures */
	      disturb (0, 1);
	      creatures(FALSE);
	      msg_print("The veil of darkness lifts.");
	    }
	}
      /* Confusion	       */
      if (f_ptr->confused > 0)
	{
	  if ((PY_CONFUSED & f_ptr->status) == 0)
	    {
	      f_ptr->status |= PY_CONFUSED;
	      prt_confused();
	    }
	  f_ptr->confused--;
	  if (f_ptr->confused == 0)
	    {
	      f_ptr->status &= ~PY_CONFUSED;
	      prt_confused();
	      msg_print("You feel less confused now.");
	      if (py.flags.rest > 0 || py.flags.rest == -1)
		rest_off ();
	    }
	}
      /* Afraid		       */
      if (f_ptr->afraid > 0)
	{
	  if ((PY_FEAR & f_ptr->status) == 0)
	    {
	      if ((f_ptr->shero+f_ptr->hero) > 0)
		f_ptr->afraid = 0;
	      else
		{
		  f_ptr->status |= PY_FEAR;
		  prt_afraid();
		}
	    }
	  else if ((f_ptr->shero+f_ptr->hero) > 0)
	    f_ptr->afraid = 1;
	  f_ptr->afraid--;
	  if (f_ptr->afraid == 0)
	    {
	      f_ptr->status &= ~PY_FEAR;
	      prt_afraid();
	      msg_print("You feel bolder now.");
	      disturb (0, 0);
	    }
	}

      /* Cut */
      if (f_ptr->cut > 0) {
	if (f_ptr->cut>1000) {
	  take_hit(3 , "a fatal wound");
	  disturb(1,0);
	} else if (f_ptr->cut>200) {
	  take_hit(3, "a fatal wound");
	  f_ptr->cut-=con_adj()+1;
	  disturb(1,0);
	} else if (f_ptr->cut>100) {
	  take_hit(2, "a fatal wound");
	  f_ptr->cut-=con_adj()+1;
	  disturb(1,0);
	} else {
	  take_hit(1, "a fatal wound");
	  f_ptr->cut-=con_adj()+1;
	  disturb(1,0);
	}
	prt_cut();
	if (f_ptr->cut<=0) {
	  f_ptr->cut=0;
	  if (py.misc.chp>=0)
	    msg_print("Your wound heals.");
	}
      }
      prt_cut();

      /* Stun */
      if (f_ptr->stun > 0) {

          int oldstun = f_ptr->stun;
  
        f_ptr->stun -= (con_adj()<=0 ? 1 : (con_adj()/2+1)); /* fixes endless 
                                                       stun if bad con. -CFT */
          if ((oldstun > 50) && (f_ptr->stun <= 50)){ /* if crossed 50 mark... */
          p_ptr->ptohit +=15;
          p_ptr->ptodam +=15;
          p_ptr->dis_th +=15;
          p_ptr->dis_td +=15;
          }             
        if (f_ptr->stun<=0) {
          f_ptr->stun=0;
          msg_print("Your head stops stinging.");
          p_ptr->ptohit +=5;
          p_ptr->ptodam +=5;
          p_ptr->dis_th +=5;
          p_ptr->dis_td +=5;
        }
      }
      prt_stun();

      /* Poisoned	       */
      if (f_ptr->poisoned > 0)
	{
	  if ((PY_POISONED & f_ptr->status) == 0)
	    {
	      f_ptr->status |= PY_POISONED;
	      prt_poisoned();
	    }
	  f_ptr->poisoned--;
	  if (f_ptr->poisoned == 0 || f_ptr->poison_im ||
	      f_ptr->poison_resist || f_ptr->resist_poison)
	    {
	      f_ptr->poisoned = 0;
	      f_ptr->status &= ~PY_POISONED;
	      prt_poisoned();
	      msg_print("You feel better.");
	      disturb (0, 0);
	    }
	  else
	    {
	      switch(con_adj())
		{
		case -4:  i = 4;  break;
		case -3:
		case -2:  i = 3;  break;
		case -1:  i = 2;  break;
		case 0:	  i = 1;  break;
		case 1: case 2: case 3:
		  i = ((turn % 2) == 0);
		  break;
		case 4: case 5:
		  i = ((turn % 3) == 0);
		  break;
		case 6:
		  i = ((turn % 4) == 0);
		  break;
		case 7:
		  i = ((turn % 5) == 0);
		  break;
		default:
		  i = ((turn % 6) == 0);
		  break;
		}
	      take_hit (i, "poison");
	      disturb (1, 0);
	    }
	}
      /* Fast		       */
      if (f_ptr->fast > 0)
	{
	  if ((PY_FAST & f_ptr->status) == 0)
	    {
	      f_ptr->status |= PY_FAST;
	      change_speed(-1);
	      msg_print("You feel yourself moving faster.");
	      disturb (0, 0);
	    }
	  f_ptr->fast--;
	  if (f_ptr->fast == 0)
	    {
	      f_ptr->status &= ~PY_FAST;
	      change_speed(1);
	      msg_print("You feel yourself slow down.");
	      disturb (0, 0);
	    }
	}
      /* Slow		       */
      if (f_ptr->slow > 0)
	{
	  if ((PY_SLOW & f_ptr->status) == 0)
	    {
	      f_ptr->status |= PY_SLOW;
	      change_speed(1);
	      msg_print("You feel yourself moving slower.");
	      disturb (0, 0);
	    }
	  f_ptr->slow--;
	  if (f_ptr->slow == 0)
	    {
	      f_ptr->status &= ~PY_SLOW;
	      change_speed(-1);
	      msg_print("You feel yourself speed up.");
	      disturb (0, 0);
	    }
	}
      /* Resting is over?      */
      if (f_ptr->rest>0 || f_ptr->rest==-1) {
	if (f_ptr->rest>0) {
	  f_ptr->rest--;
	  if (f_ptr->rest == 0)  /* Resting over */
	    rest_off();
	} else {
	  if (py.misc.chp==py.misc.mhp && py.misc.cmana==py.misc.mana) {
	    f_ptr->rest=0;
	    rest_off();
	  }
	}
      }

      /* Check for interrupts to find or rest. */
      if ((command_count > 0 || find_flag || f_ptr->rest>0 || f_ptr->rest==-1)
	  && (check_input (find_flag ? 0 : 10000))
	  )
	{
	  disturb (0, 0);
	}

      /* Hallucinating?	 (Random characters appear!)*/
      if (f_ptr->image > 0)
	{
	  end_find ();
	  f_ptr->image--;
	  if (f_ptr->image == 0)
	    prt_map ();	 /* Used to draw entire screen! -CJS- */
	}
      /* Paralysis	       */
      if (f_ptr->paralysis > 0)
	{
	  /* when paralysis true, you can not see any movement that occurs */
	  f_ptr->paralysis--;
	  disturb (1, 0);
	}
      /* Protection from evil counter*/
      if (f_ptr->protevil > 0)
	{
	  f_ptr->protevil--;
	  if (f_ptr->protevil == 0)
	    msg_print ("You no longer feel safe from evil.");
	}
      /* Invulnerability	*/
      if (f_ptr->invuln > 0)
	{
	  if ((PY_INVULN & f_ptr->status) == 0)
	    {
	      f_ptr->status |= PY_INVULN;
	      disturb (0, 0);
	      py.misc.ptoac += 100; /* changed to ptoac -CFT */
	      py.misc.dis_tac += 100;
	      msg_print("Your skin turns into steel!");
              f_ptr->status |= PY_ARMOR; /* have to update ac display */
	    }
	  f_ptr->invuln--;
	  if (f_ptr->invuln == 0)
	    {
	      f_ptr->status &= ~PY_INVULN;
	      disturb (0, 0);
	      py.misc.ptoac -= 100; /* changed to ptoac -CFT */
	      py.misc.dis_tac -= 100;
	      msg_print("Your skin returns to normal.");
              f_ptr->status |= PY_ARMOR; /* have to update ac display */
	   }
	}
      /* Heroism       */
      if (f_ptr->hero > 0)
	{
	  if ((PY_HERO & f_ptr->status) == 0)
	    {
	      f_ptr->status |= PY_HERO;
	      disturb (0, 0);
	      p_ptr->mhp += 10;
	      p_ptr->chp += 10;
	      p_ptr->ptohit += 12;
	      p_ptr->dis_th += 12;
	      msg_print("You feel like a HERO!");
	      prt_mhp();
	      prt_chp();
	    }
	  f_ptr->hero--;
	  if (f_ptr->hero == 0)
	    {
	      f_ptr->status &= ~PY_HERO;
	      disturb (0, 0);
	      p_ptr->mhp -= 10;
	      if (p_ptr->chp > p_ptr->mhp)
		{
		  p_ptr->chp = p_ptr->mhp;
		  p_ptr->chp_frac = 0;
		  prt_chp();
		}
	      p_ptr->ptohit -= 12;
	      p_ptr->dis_th -= 12;
	      msg_print("The heroism wears off.");
	      prt_mhp();
	    }
	}
      /* Super Heroism */
      if (f_ptr->shero > 0)
	{
	  if ((PY_SHERO & f_ptr->status) == 0)
	    {
	      f_ptr->status |= PY_SHERO;
	      disturb (0, 0);
	      p_ptr->mhp += 30;
	      p_ptr->chp += 30;
	      p_ptr->ptohit += 24;
	      p_ptr->dis_th += 24;
	      p_ptr->ptoac -= 10;
	      p_ptr->dis_tac -= 10;
	      msg_print("You feel like a killing machine!");
	      prt_mhp();
	      prt_chp();
	    }
	  f_ptr->shero--;
	  if (f_ptr->shero == 0)
	    {
	      f_ptr->status &= ~PY_SHERO;
	      disturb (0, 0);
	      p_ptr->mhp -= 30;
	      p_ptr->ptoac += 10;
	      p_ptr->dis_tac += 10;
	      if (p_ptr->chp > p_ptr->mhp)
		{
		  p_ptr->chp = p_ptr->mhp;
		  p_ptr->chp_frac = 0;
		  prt_chp();
		}
	      p_ptr->ptohit -= 24;
	      p_ptr->dis_th -= 24;
	      msg_print("You feel less Berserk.");
	      prt_mhp();
	    }
	}
      /* Blessed       */
      if (f_ptr->blessed > 0)
	{
	  if ((PY_BLESSED & f_ptr->status) == 0)
	    {
	      f_ptr->status |= PY_BLESSED;
	      disturb (0, 0);
	      p_ptr->ptohit += 10;
	      p_ptr->dis_th += 10;
	      p_ptr->ptoac += 5; /* changed to ptoac -CFT */
	      p_ptr->dis_tac+= 5;
	      msg_print("You feel righteous!");
              f_ptr->status |= PY_ARMOR; /* have to update ac display */
	    }
	  f_ptr->blessed--;
	  if (f_ptr->blessed == 0)
	    {
	      f_ptr->status &= ~PY_BLESSED;
	      disturb (0, 0);
	      p_ptr->ptohit -= 10;
	      p_ptr->dis_th -= 10;
	      p_ptr->ptoac -= 5; /* changed to ptoac -CFT */
	      p_ptr->dis_tac -= 5;
	      msg_print("The prayer has expired.");
              f_ptr->status |= PY_ARMOR; /* have to update ac display */
	    }
	}
      /* Shield       */
      if (f_ptr->shield > 0)
	{
	  f_ptr->shield--;
	  if (f_ptr->shield == 0)
	    {
	      disturb (0, 0);
	      msg_print("Your mystic shield crumbles away.");
              py.misc.ptoac -= 50; /* changed to ptoac -CFT */
              py.misc.dis_tac -= 50;
              f_ptr->status |= PY_ARMOR; /* have to update ac display */
	    }
	}
      /* Resist Heat   */
      if (f_ptr->resist_heat > 0)
	{
	  f_ptr->resist_heat--;
	  if (f_ptr->resist_heat == 0)
	    msg_print ("You no longer feel safe from flame.");
	}
      /* Resist Cold   */
      if (f_ptr->resist_cold > 0)
	{
	  f_ptr->resist_cold--;
	  if (f_ptr->resist_cold == 0)
	    msg_print ("You no longer feel safe from cold.");
	}
      /* Resist Acid   */
      if (f_ptr->resist_acid > 0)
	{
	  f_ptr->resist_acid--;
	  if (f_ptr->resist_acid == 0)
	    msg_print ("You no longer feel safe from acid.");
	}
      /* Resist Lightning   */
      if (f_ptr->resist_light > 0)
	{
	  f_ptr->resist_light--;
	  if (f_ptr->resist_light == 0)
	    msg_print ("You no longer feel safe from lightning.");
	}
      /* Resist Poison   */
      if (f_ptr->resist_poison > 0)
	{
	  f_ptr->resist_poison--;
	  if (f_ptr->resist_poison == 0)
	    msg_print ("You no longer feel safe from poison.");
	}

      /* Timeout Implants  */
      for (i = INVEN_IMP1; i < (INVEN_ARRAY_SIZE); i++) {
		i_ptr = &inventory[i];
		if (i_ptr->tval == TV_IMPLANT ) {
		  if (i_ptr->shots>0)
		    i_ptr->shots--;
			}
      }


      /* Detect Invisible      */
      if (f_ptr->detect_inv > 0)
	{
	  if ((PY_DET_INV & f_ptr->status) == 0)
	    {
	      f_ptr->status |= PY_DET_INV;
	      f_ptr->see_inv = TRUE;
	      /* light but don't move creatures */
	      creatures (FALSE);
	    }
	  f_ptr->detect_inv--;
	  if (f_ptr->detect_inv == 0)
	    {
	      f_ptr->status &= ~PY_DET_INV;
	      /* may still be able to see_inv if wearing magic item */
              if (py.misc.prace == 9)
                f_ptr->see_inv = TRUE;
              else {
                f_ptr->see_inv = FALSE; /* unless item grants it */
                for (i = INVEN_WIELD; i <= INVEN_LIGHT; i++)
                  if (TR_SEE_INVIS & inventory[i].flags)
                    f_ptr->see_inv = TRUE;
              }
	      /* unlight but don't move creatures */
	      creatures (FALSE);
	    }
	}
      /* Timed infra-vision    */
      if (f_ptr->tim_infra > 0)
	{
	  if ((PY_TIM_INFRA & f_ptr->status) == 0)
	    {
	      f_ptr->status |= PY_TIM_INFRA;
	      f_ptr->see_infra++;
	      /* light but don't move creatures */
	      creatures (FALSE);
	    }
	  f_ptr->tim_infra--;
	  if (f_ptr->tim_infra == 0)
	    {
	      f_ptr->status &= ~PY_TIM_INFRA;
	      f_ptr->see_infra--;
	      /* unlight but don't move creatures */
	      creatures (FALSE);
	    }
	}
	/* Word-of-Recall  Note: Word-of-Recall is a delayed action	 */
      if (f_ptr->word_recall > 0)
	if (f_ptr->word_recall == 1)
	  {
	    new_level_flag = TRUE;
	    f_ptr->paralysis++;
	    f_ptr->word_recall = 0;
	    if (dun_level > 0)
	      {
		dun_level = 0;
		msg_print("You feel yourself yanked upwards!");
	      }
	    else if (py.misc.max_dlv != 0)
	      {
		dun_level = py.misc.max_dlv;
		msg_print("You feel yourself yanked downwards!");
	      }
	  }
	else
	  f_ptr->word_recall--;

      /* Random teleportation  */
      if ((py.flags.teleport) && (randint(100) == 1))
	{
	  disturb (0, 0);
	  teleport(40);
	}

      /* See if we are too weak to handle the weapon or pack.  -CJS- */
      if (py.flags.status & PY_STR_WGT)
	check_strength();
      if (py.flags.status & PY_STUDY)
	prt_study();
      if (py.flags.status & PY_SPEED)
	{
	  py.flags.status &= ~PY_SPEED;
	  prt_speed();
	}
      if ((py.flags.status & PY_PARALYSED) && (py.flags.paralysis < 1))
	{
	  prt_state();
	  py.flags.status &= ~PY_PARALYSED;
	}
      else if (py.flags.paralysis > 0)
	{
	  prt_state();
	  py.flags.status |= PY_PARALYSED;
	}
      else if (py.flags.rest > 0 || py.flags.rest==-1)
	prt_state();

      if ((py.flags.status & PY_ARMOR) != 0)
	{
          py.misc.dis_ac = py.misc.pac + py.misc.dis_tac; /* use updated ac */
	  prt_pac();
	  py.flags.status &= ~PY_ARMOR;
	}
      if ((py.flags.status & PY_STATS) != 0)
	{
	  for (i = 0; i < 6; i++)
	    if ((PY_STR << i) & py.flags.status)
	      prt_stat(i);
	  py.flags.status &= ~PY_STATS;
	}
      if (py.flags.status & PY_HP)
	{
	  prt_mhp();
	  prt_chp();
	  py.flags.status &= ~PY_HP;
	}
      if (py.flags.status & PY_MANA)
	{
	  prt_cmana();
	  py.flags.status &= ~PY_MANA;
	}

      /* Allow for a slim chance of detect enchantment -CJS- */
      /* for 1st level char, check once every 2160 turns
	 for 40th level char, check once every 416 turns */
      if (py.misc.pclass==2?
	  ((f_ptr->confused == 0) && (py.misc.pclass!=0) &&
	   (randint((int)(10000/(py.misc.lev*py.misc.lev+40)) + 1) == 1))
	  :
	  (((turn & 0xF) == 0) && (f_ptr->confused == 0)
	  && (randint((int)(10 + 750 / (5 + py.misc.lev))) == 1))
	  )
	{
	  vtype tmp_str;

	  for (i = 0; i < INVEN_ARRAY_SIZE; i++) {
	    if (i == inven_ctr)
	      i = 22;
	    i_ptr = &inventory[i];
	    /* if in inventory, succeed 1 out of 50 times,
	    if in equipment list, success 1 out of 10 times,
	    unless your a priest or rogue... */
	    if ((i_ptr->tval != TV_NOTHING) && special_check(i_ptr) &&
		((py.misc.pclass==2 || py.misc.pclass==3)?
		 (randint(i < 22 ? 5 : 1) == 1)
		 :
		 (randint(i < 22 ? 50 : 10) == 1)))
	      {
		extern char *describe_use();

		if (py.misc.pclass==0||py.misc.pclass==3||py.misc.pclass==5)
		  if ((i_ptr->tval==TV_SWORD) ||
		      (i_ptr->tval==TV_HANDGUN) ||
		      (i_ptr->tval==TV_HEAVYGUN) ||
		      (i_ptr->tval==TV_BLOWGUN) ||
		      (i_ptr->tval==TV_DIGGING) ||
		      (i_ptr->tval==TV_AMMO) ||
		      (i_ptr->tval==TV_SOFT_ARMOR) ||
		      (i_ptr->tval==TV_HARD_ARMOR) ||
		      (i_ptr->tval==TV_HELM) ||
		      (i_ptr->tval==TV_BOOTS) ||
		      (i_ptr->tval==TV_CLOAK) ||
		      (i_ptr->tval==TV_GLOVES) ||
		      (i_ptr->tval==TV_SHIELD))
		    continue;
		(void) sprintf(tmp_str,
			       "There's something %s about what you are %s...",
			       special_check(i_ptr)>0?"good":"bad",
			       describe_use(i));
		disturb(0, 0);
		msg_print(tmp_str);
		add_inscribe(i_ptr, (special_check(i_ptr)>0)?
			   ID_MAGIK:ID_DAMD);
	      }
	  }
	}

      /* Warriors, Rogues and paladins inbuilt ident */
      if (((py.misc.pclass==0) && (f_ptr->confused == 0) &&
	  (randint((int)(9000/(py.misc.lev*py.misc.lev+40)) + 1) == 1))
	  ||
	  ((py.misc.pclass==3) && (f_ptr->confused == 0) &&
	  (randint((int)(20000/(py.misc.lev*py.misc.lev+40)) + 1) == 1))
	  ||
	  ((py.misc.pclass==5) && (f_ptr->confused == 0) &&
	  (randint((int)(80000/(py.misc.lev*py.misc.lev+40)) + 1) == 1))) {
	  vtype tmp_str;
	  char *value_check();

	  for (i = 0; i < INVEN_ARRAY_SIZE; i++) {
	    if (i == inven_ctr)
	      i = 22;
	    i_ptr = &inventory[i];
	    /* if in inventory, succeed 1 out of 5 times,
	    if in equipment list, always succeed! */
	    if (((i_ptr->tval==TV_SOFT_ARMOR) ||
		 (i_ptr->tval==TV_HARD_ARMOR) ||
		 (i_ptr->tval==TV_SWORD) ||
		 (i_ptr->tval==TV_HANDGUN) ||
		 (i_ptr->tval==TV_HEAVYGUN) ||
		 (i_ptr->tval==TV_SHIELD) ||
		 (i_ptr->tval==TV_HELM) ||
		 (i_ptr->tval==TV_BOOTS) ||
		 (i_ptr->tval==TV_GLOVES) ||
		 (i_ptr->tval==TV_DIGGING) ||
		 (i_ptr->tval==TV_AMMO) ||
		 (i_ptr->tval==TV_BLOWGUN) ||
		 (i_ptr->tval==TV_CLOAK))
		 && value_check(i_ptr) &&
		(randint(i < 22 ? 5 : 1) == 1))
	      {
		extern char *describe_use();
		char out_val[100], tmp[100], *ptr;
		int i;

		(void) strcpy(tmp, object_list[i_ptr->index].name);

		ptr = tmp;
		i=0;
		while (tmp[i]==' ' || tmp[i]=='&')
		  ptr=&tmp[++i];

		(void) strcpy(out_val, ptr);

		ptr=out_val;

		while (*ptr) {
		  if (*ptr == '~') *ptr = 's';
		  ptr++;
		}

		(void) sprintf(tmp_str,
			       "You feel the %s you are %s %s %s...",
			       out_val,
			       describe_use(i),
			       ((i_ptr->tval==TV_AMMO) ||
				(i_ptr->tval==TV_BOOTS) ||
				(i_ptr->tval==TV_GLOVES))?"are":"is",
			       value_check(i_ptr));
		disturb(0, 0);
		msg_print(tmp_str);
		if (!strcmp(value_check(i_ptr), "terrible"))
		  add_inscribe(i_ptr, ID_DAMD);
		else if (!strcmp(value_check(i_ptr), "worthless"))
		  add_inscribe(i_ptr, ID_DAMD);
		else
		  inscribe(i_ptr, value_check(i_ptr));
	      }
	  }
	}

      if ((py.flags.paralysis < 1) &&	     /* Accept a command?     */
	  (py.flags.rest == 0) &&
	  (py.flags.stun < 100) &&
	  (!death))
	/* Accept a command and execute it				 */
	{
	  do
	    {
	      if (py.flags.status & PY_REPEAT)
		prt_state ();
	      default_dir = FALSE;
	      free_turn_flag = FALSE;

	      if (find_flag)
		{
		  find_run();
		  find_count--;
		  if (find_count == 0)
		    end_find();
		  put_qio();
		}
	      else if (doing_inven)
		inven_command (doing_inven);
	      else
		{
		  /* move the cursor to the players character */
		  move_cursor_relative (char_row, char_col);
		  if (command_count > 0)
		    {
		      command_count--;
		      msg_flag = FALSE;
		      default_dir = TRUE;
		    }
		  else
		    {
		      msg_flag = FALSE;
		      command = inkey();
		      i = 0;
			
		      /* Another way of typing control codes -CJS- */
		      if (command == '^')
			{
			  if (command_count > 0)
			    prt_state();
			  if (get_com("Control-", &command))
			    {
			      if (command >= 'A' && command <= 'Z')
				command -= 'A' - 1;
			      else if (command >= 'a' && command <= 'z')
				command -= 'a' - 1;
			      else
				{
			       msg_print("Type ^ <letter> for a control char");
				  command = ' ';
				}
			    }
			  else
			    command = ' ';
			}
		      /* move cursor to player char again, in case it moved */
		      move_cursor_relative (char_row, char_col);
		      /* Commands are always converted to rogue form. -CJS- */
			/* Hahaha!!  Not any more!! -- JLM  */
			
		    }
		  /* Flash the message line. */
		  erase_line(MSG_LINE, 0);
		  move_cursor_relative(char_row, char_col);
		  put_qio();

		  do_command (command);
		  /* Find is counted differently, as the command changes. */
		  if (find_flag)
		    {
		      find_count = command_count;
		      command_count = 0;
		    }
		  if (free_turn_flag)
		    command_count = 0;
		}
	      /* End of commands				     */
	    }
	  while (free_turn_flag && !new_level_flag && !eof_flag);
	}
      else
	{
	  /* if paralyzed, resting, or dead, flush output */
	  /* but first move the cursor onto the player, for aesthetics */
	  move_cursor_relative (char_row, char_col);
	  put_qio ();
	}

      /* Teleport?		       */
      if (teleport_flag)  teleport(100);
      /* Move the creatures	       */
      if (!new_level_flag)  creatures(TRUE);
      /* Exit when new_level_flag is set   */
    }
  while (!new_level_flag && !eof_flag);
}



/* Is an item an enchanted weapon or armor and we don't know?  -CJS- */
/* returns positive if it is a good enchantment */
/* returns negative if a bad enchantment... */
int special_check(t_ptr)
register inven_type *t_ptr;
{
  if (t_ptr->tval==TV_NOTHING) return 0;
  if (known2_p(t_ptr))
    return 0;
  if (store_bought_p(t_ptr))
    return 0;
  if (t_ptr->ident & ID_MAGIK)
    return 0;
  if (t_ptr->ident & ID_DAMD)
    return 0;
  if (t_ptr->flags & TR_CURSED) return -1;
  if (t_ptr->tval!=TV_HARD_ARMOR && t_ptr->tval!=TV_SWORD &&
      t_ptr->tval!=TV_SOFT_ARMOR && t_ptr->tval!=TV_SHIELD &&
      t_ptr->tval!=TV_CLOAK && t_ptr->tval!=TV_GLOVES &&
      t_ptr->tval!=TV_BOOTS && t_ptr->tval!=TV_HELM &&
      t_ptr->tval!=TV_DIGGING && t_ptr->tval!=TV_SPIKE &&
      t_ptr->tval!=TV_AMMO && 
       t_ptr->tval!=TV_BLOWGUN &&
      t_ptr->tval!=TV_HEAVYGUN && t_ptr->tval!=TV_HANDGUN)
      return 0;
  if (t_ptr->tohit > 0 || t_ptr->todam > 0 || t_ptr->toac > 0)
    return 1;
  return 0;
}


/* Is an item an enchanted weapon or armor and we don't know?  -CJS- */
/* returns a description */
char *value_check(t_ptr)
register inven_type *t_ptr;
{
  if (t_ptr->tval==TV_NOTHING) return 0;
  if (known2_p(t_ptr))
    return 0;
  if (store_bought_p(t_ptr))
    return 0;
  if (t_ptr->ident & ID_MAGIK)
    return 0;
  if (t_ptr->ident & ID_DAMD)
    return 0;
  if (t_ptr->inscrip[0] != '\0')
    return 0;
  if (t_ptr->flags&TR_CURSED && t_ptr->name2==SN_NULL) return "worthless";
  if (t_ptr->flags&TR_CURSED && t_ptr->name2!=SN_NULL) return "terrible";
  if ((t_ptr->tohit<=0 && t_ptr->todam<=0 && t_ptr->toac<=0) &&
      t_ptr->name2==SN_NULL)
    return "average";
  if (t_ptr->name2==SN_NULL)
    return "good";
  if ((t_ptr->name2==SN_R) || (t_ptr->name2==SN_RA) ||
      (t_ptr->name2==SN_RF) || (t_ptr->name2==SN_RC) ||
      (t_ptr->name2==SN_RL) || (t_ptr->name2==SN_FED) ||
      (t_ptr->name2==SN_SYND) || (t_ptr->name2==SN_FT) ||
      (t_ptr->name2==SN_AP) || (t_ptr->name2==SN_FB) ||
      (t_ptr->name2==SN_ADAM) || (t_ptr->name2==SN_FREE_ACTION) ||
      (t_ptr->name2==SN_ACC) || (t_ptr->name2==SN_SLAYING) ||
      (t_ptr->name2==SN_WOUND) || (t_ptr->name2==SN_SLOW_DESCENT) ||
      (t_ptr->name2==SN_SPEED) || 
      (t_ptr->name2==SN_INTELLIGENCE) || (t_ptr->name2==SN_MECH) ||
      (t_ptr->name2==SN_INFRAVISION) || (t_ptr->name2==SN_MIGHT) ||
      (t_ptr->name2==SN_LORDLINESS) || (t_ptr->name2==SN_BETEL) ||
      (t_ptr->name2==SN_BEAUTY) || (t_ptr->name2==SN_SEEING) ||
      (t_ptr->name2==SN_REGENERATION) || (t_ptr->name2==SN_PROTECTION) ||
      (t_ptr->name2==SN_FIRE) || (t_ptr->name2==SN_BLACK_TALON) ||
      (t_ptr->name2==SN_U_TIP) || (t_ptr->name2==SN_DIAMOND_TIP) ||
      (t_ptr->name2==SN_HIGH_DENSITY) || (t_ptr->name2==SN_MONO) ||
      (t_ptr->name2==SN_VULCAN) 
		)
    return "excellent";
  return "special";
}

void edit_commands()
{
	char tmp[25]={0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
	char command; int index=0;
	
	prt("Extended command:", 0,0);
	while (index!=25 &&(command=inkey()) != '\n' && command != ESCAPE)
	{	if (index!=0 && (command==DELETE || command == CTRL('H')))
			tmp[--index]='\0';
		else
			tmp[index++]=command;
		prt(tmp, 0, 18);
	}
	if (strcmp(tmp, "edit"))
	{		/* tmp is not "edit" */
		for (index=0; index<NUM_COMMANDS; index++)
			if (!strcmp(tmp, command_array[index].name))
			{ free_turn_flag=FALSE;
					(*(command_array[index].function))(); return;	}
		msg_print("Type \":help\" for help.");
		return;
	}
	else
	{	
		prt("Edit which command:", 0,0);
		for (index=24; index; index--)
			tmp[index]='\0';
		while(index!=25 && (command=inkey())!='\n' && command!=ESCAPE)
		{	if (index && (command==DELETE || command==CTRL('H')))
				tmp[--index]='\0';
			else	tmp[index++]=command;
			prt(tmp, 0, 21);
		}
		for (index=0; index<NUM_COMMANDS; index++)
			if (!strcmp(tmp, command_array[index].name))
				break;
		if (index==NUM_COMMANDS)
			msg_print("Invalid command.");
		else
		{	/* We have a command to edit. */
			prt("Type new key:", 0,0);
			command=inkey();
			if (command==ESCAPE) return;
			if (command==' ') command='\0';
			if (command==':') msg_print("Sorry, ':' is reserved.");
			else {
				int i;
				for (i=0; i<NUM_COMMANDS; i++)
					if (command_array[i].key==command)
						command_array[i].key='\0';
				command_array[index].key=command;
				}
		}
}}


static int do_pickup = TRUE;

static void do_command(com_val)
char com_val;
{
	register int index;
	
	if (!com_val || com_val==' ' || com_val==ESCAPE)
		{ free_turn_flag=TRUE; return; }		/* Do nothing. */
	if (com_val == ':')
		{ free_turn_flag=TRUE;	edit_commands();return;	}
	for (index=0; index<NUM_COMMANDS; index++)
		if (command_array[index].key==com_val)
		{	(*(command_array[index].function))();	return;	}
	if ('0'<=com_val && com_val<='9')
		{	command_count=(com_val-'0');	count_cmd();	return;}
	msg_print("Type \":help\" for help.");
}

void count_cmd(void)
{
	char tmp[8], command;
	int i;

	prt("Repeat count:", 0,0);
	i=command_count;	command=inkey();
	while(TRUE)
	{
		if (command==DELETE || command == CTRL('H'))
		{
			i=i/10;
			(void) sprintf(tmp, "%d", i);
			prt(tmp, 0, 14);
		}
		else if (command >= '0' && command <= '9')
		{
			if (i>99)
				bell();
			else
			{	i=i*10 + command-'0';
				(void) sprintf(tmp, "%d", i);
				prt(tmp, 0, 14);
			}
		}
		else break;
		command = inkey();
	}
	if (i==0)
	{
		i=99;
		(void) sprintf(tmp, "%d", i);
		prt(tmp, 0, 14);
	}
	/* Allow numbers as commands. */
	if (command == ' ')
	{
		prt("Command:", 0,20);
		command=inkey();
	}
	if (command == '^')
	{
		if (get_com("Control-", &command))
		{	if (command >= 'A' && command <= 'Z')
			command -= 'A' -1;
			else if (command >= 'a' && command <= 'z')
				command -= 'a' - 1;
			else
			{	msg_print("Type ^ <letter> for a control character");
				command = ' ';
			}
		}
		else command = ' ';
	}
	move_cursor_relative(char_row, char_col);
	if (i>0)
	{	if (!valid_countcommand(command))
		{	free_turn_flag=TRUE;
			msg_print("Invalid command with a count.");
			command = ' ';
		}
	  else
		{	command_count=i;
			prt_state();
			command_count--;
		}
	}
	erase_line(MSG_LINE, 0);
	move_cursor_relative(char_row, char_col);
	put_qio();
	do_command(command);	
}
	
/*  int dir_val, do_pickup;
  int y, x, i, j;
  vtype out_val, tmp_str;
  register struct flags *f_ptr; */

void nopickup(void)
{
	int i, dir_val;


  /* hack for move without pickup.  Map '-' to a movement command. */
      do_pickup = FALSE;
      i = command_count;
      if (get_dir(NULL, &dir_val))
	  command_count = i;
      else
	dir_val = ' ';
	
	do_command(dir_val);
	do_pickup=TRUE;
}

void Kill(void)
{
      flush();
      if ((!total_winner)?get_check("Do you really want to quit?")
	                 :get_check("Do you want to retire?"))
	{
	  new_level_flag = TRUE;
	  death = TRUE;
	  (void) strcpy(died_from, "Quitting");
	}
      free_turn_flag = TRUE;
}

void prev_msg(void)
{
int i, j, x;

      if (command_count > 0)
	{
	  i = command_count;
	  if (i > MAX_SAVE_MSG)
	    i = MAX_SAVE_MSG;
	  command_count = 0;
	}
      else if (last_command != 16)
	i = 1;
      else
	i = MAX_SAVE_MSG;
      j = last_msg;
      if (i > 1) {
	save_screen();
	x = i;
	while (i > 0) {
	  i--;
	  prt(old_msg[j], i, 0);
	  if (j == 0)
	    j = MAX_SAVE_MSG-1;
	  else
	    j--;
	}
	erase_line (x, 0);
	pause_line(x);
	restore_screen();
      }
      else {
	/* Distinguish real and recovered messages with a '>'. -CJS- */
	put_buffer(">", 0, 0);
	prt(old_msg[j], 0, 1);
      }
      free_turn_flag = TRUE;
}

void enter_wiz(void)
{
      if (wizard)
	{
	  wizard = FALSE;
	  msg_print("Wizard mode off.");
	}
      else if (enter_wiz_mode())
	msg_print("Wizard mode on.");
	else msg_print("Type \":help\" for help.");
      prt_winner();
      free_turn_flag = TRUE;
}

void exit_save(void)
{
      if (total_winner)
	{
       msg_print("You are a Total Winner,  your character must be retired.");
	    msg_print ("Type \":kill\" when you are ready to retire.");
	}
      else
	{
	  (void) strcpy (died_from, "(saved)");
	  msg_print ("Saving game...");
	  if (save_char ())
	    exit_game();
	  msg_print("Save failed...");
	  (void) strcpy (died_from, "(alive and well)");
	}
      free_turn_flag = TRUE;
}

void set_opt(void)
{
      save_screen();
      set_options();
      restore_screen();
      free_turn_flag = TRUE;
}

void enscribe(void)
{
      scribe_object ();
      free_turn_flag = TRUE;
}

void rerate_hp(void)
{      if (wizard)
	rerate();
	else msg_print("Type \":help\" for help.");
      free_turn_flag = TRUE;
}

void southw(void)	{
      move_char(1, do_pickup);
}
void south(void){
      move_char(2, do_pickup);
}
void southe(void){
      move_char(3, do_pickup);
}
void west(void){
      move_char(4, do_pickup);
}
void east(void){
      move_char(6, do_pickup);
}
void northw(void){
      move_char(7, do_pickup);
}
void north(void){
      move_char(8, do_pickup);
}
void northe(void){
      move_char(9, do_pickup);
}
/* running command code here. */
void run_dir(void){
int dir_val;
		if (get_dir(NULL,&dir_val) && dir_val != 5)
			find_init(dir_val);
		else free_turn_flag = TRUE;
}
void runsw(void){
	find_init(1);
}
void runs(void){
	find_init(2);
}
void runse(void){
	find_init(3);
}
void runw(void)
{	find_init(4);
}
void rune(void)
{	find_init(6);
}
void runnw(void)
{	find_init(7);
}
void runn(void)
{	find_init(8);
}
void runne(void)
{	find_init(9);
}
/* Back to your normal commands. */
void identi(void){
      ident_char();
      free_turn_flag = TRUE;
}
void stay(void){
      move_char (5, do_pickup);
      if (command_count > 1)
	{
	  command_count--;
	  rest();
	}
}
void help(void){
	int i;
	vtype tmp_str;
	command_type command;

	save_screen();
	clear_screen();
	put_buffer("Type ^R to redraw screen.  Type ESC to abort command.", 0,0);
	put_buffer("Type \":edit\" to edit commands.",1,0);
	for (i=0; i<((float)NUM_REG_COMMANDS)/4; i++)
	{
		command = command_array[4*i];
		sprintf(tmp_str, "%c%c  %s", (command.key && command.key<=26)?'^':' ',
			(command.key<=26)?(command.key?command.key+'A'-1:' '):(command.key),
			command.name);
		put_buffer(tmp_str, i+2, 0);
		command = command_array[4*i+1];
		sprintf(tmp_str, "%c%c  %s", (command.key && command.key<=26)?'^':' ',
			(command.key<=26)?(command.key?command.key+'A'-1:' '):(command.key),
			command.name);
		put_buffer(tmp_str, i+2, 20);
		command = command_array[4*i+2];
		sprintf(tmp_str, "%c%c  %s", (command.key && command.key<=26)?'^':' ',
			(command.key<=26)?(command.key?command.key+'A'-1:' '):(command.key),
			command.name);
		put_buffer(tmp_str, i+2, 40);
		command = command_array[4*i+3];
		sprintf(tmp_str, "%c%c  %s", (command.key && command.key<=26)?'^':' ',
			(command.key<=26)?(command.key?command.key+'A'-1:' '):(command.key),
			command.name);
		put_buffer(tmp_str, i+2, 60);
/*		command = command_array[5*i+4];
		sprintf(tmp_str, "%c%c  %s", (command.key && command.key<=26)?'^':' ',
			(command.key<=26)?(command.key?command.key+'A'-1:' '):(command.key),
			command.name);
		put_buffer(tmp_str, i+2, 64); 	*/

	}
	prt("Press any key to continue.", 23, 23);
	(void) inkey();
	restore_screen();
      free_turn_flag = TRUE;
}
void charsheet(void){
      save_screen();
      change_name();
      restore_screen();
      free_turn_flag = TRUE;
}
void loading_gun(void){
		inven_command('l');
}
void unload(void)
{	inven_command('u');
}

void locate(void){
	int x,y;
	vtype tmp_str, out_val;
	int dir_val;
      if ((py.flags.blind > 0) || no_light())
	msg_print("You can't see your map.");
      else
	{
	  int cy, cx, p_y, p_x;

	  y = char_row;
	  x = char_col;
	  if (get_panel(y, x, TRUE))
	    prt_map();
	  cy = panel_row;
	  cx = panel_col;
	  for(;;)
	    {
	      p_y = panel_row;
	      p_x = panel_col;
	      if (p_y == cy && p_x == cx)
		tmp_str[0] = '\0';
	      else
		(void) sprintf(tmp_str, "%s%s of",
			       p_y < cy ? " North" : p_y > cy ? " South" : "",
			       p_x < cx ? " West" : p_x > cx ? " East" : "");
	      (void) sprintf(out_val,
	   "Map sector [%d,%d], which is%s your sector. Look which direction?",
			     p_y, p_x, tmp_str);
	      if (!get_dir(out_val, &dir_val))
		break;
/*								      -CJS-
// Should really use the move function, but what the hell. This
// is nicer, as it moves exactly to the same place in another
// section. The direction calculation is not intuitive. Sorry.
*/
	      for(;;){
		x += ((dir_val-1)%3 - 1) * SCREEN_WIDTH/2;
		y -= ((dir_val-1)/3 - 1) * SCREEN_HEIGHT/2;
		if (x < 0 || y < 0 || x >= cur_width || y >= cur_width)
		  {
		    msg_print("You've gone past the end of your map.");
		    x -= ((dir_val-1)%3 - 1) * SCREEN_WIDTH/2;
		    y += ((dir_val-1)/3 - 1) * SCREEN_HEIGHT/2;
		    break;
		  }
		if (get_panel(y, x, TRUE))
		  {
		    prt_map();
		    break;
		  }
	      }
	    }
	  /* Move to a new panel - but only if really necessary. */
	  if (get_panel(char_row, char_col, FALSE))
	    prt_map();
	}
      free_turn_flag = TRUE;
}
void search_tog(void){
      if (search_flag)
	search_off();
      else
	search_on();
      free_turn_flag = TRUE;
}
/* Insert tunnelling command code here. */
void tunnelling(void){
	int dir_val;
		if (get_dir(NULL, &dir_val) && dir_val != 5)
			tunnel(dir_val);
		else free_turn_flag=TRUE;
}
void tunsw(void){
	tunnel(1);}
void tuns(void){
	tunnel(2);}
void tunse(void){
	tunnel(3);}
void tune(void){
	tunnel(6);}
void tunw(void){
	tunnel(4);}
void tunnw(void){
	tunnel(7);}
void tunn(void){
	tunnel(8);}
void tunne(void){
	tunnel(9);}
/* No more tunnelling. */

void map(void){
      screen_map();
      free_turn_flag = TRUE;
}
void brouse(void){
      examine_book();
      free_turn_flag = TRUE;
}
void drop_obj(void){
      inven_command('d');
}
void equip(void){
      inven_command('e');
}
void invent(void){
      inven_command('i');
}
void examine(void){
      look();
      free_turn_flag = TRUE;
}

void shoot_gun(void)
{	int x, dir_val,i,y ;
		x = char_col; y=char_row;
		if (get_dir("Which direction?", &dir_val))
		for (i=attack_blows(inventory[INVEN_WIELD].weight, &i);	
			i; i--)
			{ fire_gun(&inventory[INVEN_WIELD], dir_val);
			if (!locate_monster(&x,&y,dir_val)) break; }
		else free_turn_flag=TRUE;
}
void search_turn(void){
      search(char_row, char_col, py.misc.srh);
}
void Remove(void){
      inven_command('t');
}
void version(void){
      helpfile(ANGBAND_VER);
      free_turn_flag = TRUE;
}
void wield(void){
      inven_command('w');
}
void exchg(void){
      inven_command('x');
}
void cureall(void){
	register struct flags * f_ptr;
	free_turn_flag=TRUE;
	if (wizard){
	      (void) remove_curse();
	      (void) cure_blindness();
	      (void) cure_confusion();
	      (void) cure_poison();
	      (void) remove_fear();
	      (void) res_stat(A_STR);
	      (void) res_stat(A_APT);
	      (void) res_stat(A_MCP);
	      (void) res_stat(A_CON);
	      (void) res_stat(A_DEX);
	      (void) res_stat(A_CHR);
	      (void) restore_level();
	      (void) hp_player(2000);
	      f_ptr = &py.flags;
	      if (f_ptr->slow > 1)
		f_ptr->slow = 1;
	      if (f_ptr->image > 1)
		f_ptr->image = 1;
	      if (f_ptr->cut > 1)
		f_ptr->cut = 1;
	      if (f_ptr->stun > 1)
		f_ptr->stun = 1;
 }
	else msg_print("Type \":help\" for help.");
}
void charedit(void){
	free_turn_flag=TRUE;
	if (wizard)
	      change_character();
	else msg_print("Type \":help\" for help.");
}
void geno(void){
	free_turn_flag=TRUE;
	if (wizard)
	      (void) mass_genocide(FALSE);
	else msg_print("Type \":help\" for help.");
}
void make_treasure(void)
{	int i;
	free_turn_flag=TRUE;
	if (wizard){
	      if (command_count > 0)
		{
		  i = command_count;
		  command_count = 0;
		}
	      else
		i = 1;
	      random_object(char_row, char_col, i);
	      prt_map();
	} else msg_print("Type \":help\" for help.");
}
void make_special(void)
{	int i;
	free_turn_flag=TRUE;
	if (wizard){
	      if (command_count > 0)
		{
		  i = command_count;
		  command_count = 0;
		}
	      else
		i = 1;
	      special_random_object(char_row, char_col, i);
	      prt_map();
	} else msg_print("Type \":help\" for help.");
}
void change_level(void){
	int i;
	vtype tmp_str;
	free_turn_flag=TRUE;
	if (wizard){
	      if (command_count > 0)
		{
		  if (command_count > 99)
		    i = 0;
		  else
		    i = command_count;
		  command_count = 0;
		}
	      else
		{
		  prt("Go to which level (0-10000) ? ", 0, 0);
		  i = -1;
		  if (get_string(tmp_str, 0, 27, 10))
		    i = atoi(tmp_str);
		    if (i>10000) i=10000;
		}
	      if (i > -1)
		{
		  dun_level = i;
		  if (dun_level > 10000)
		    dun_level = 10000;
		  new_level_flag = TRUE;
		}
	      else
		erase_line(MSG_LINE, 0);
	} else msg_print("Type \":help\" for help.");
}

void disp_objects(void){
	free_turn_flag=TRUE;
	if  (wizard)
	      print_objects();
	else msg_print("Type \":help\" for help.");
}
void wizhelp(void){
	int i, j=0;
	vtype tmp_str;
	command_type command;

	free_turn_flag=TRUE;
	if (wizard)	{
		save_screen();
		clear_screen();
		for (i=NUM_REG_COMMANDS+3; i<NUM_COMMANDS; i+=2)
		{
			command = command_array[i];
			sprintf(tmp_str, "%c%c  %s",
				 (command.key && command.key<=26)?'^':' ',
				(command.key<=26)?(command.key?command.key+'A'-1:' '):
						(command.key),		command.name);
			put_buffer(tmp_str, j+2, 0);
			command = command_array[i+1];
			sprintf(tmp_str, "%c%c  %s",
				(command.key && command.key<=26) ? '^' : ' ',
				(command.key<=26)?(command.key?command.key+'A'-1:' ')
								: (command.key),		command.name);
			put_buffer(tmp_str, (j++)+2, 40);
		}
	prt("Press any key to continue.", 23, 23);
	(void) inkey();
	restore_screen();
	}
	else msg_print("Type \":help\" for help.");
}

void wizid(void){
	free_turn_flag=TRUE;
	if (wizard)
	      (void) ident_spell();
	else msg_print("Type \":help\" for help.");
}
void idall(void){
	int i;
	vtype tmp_str;
	free_turn_flag=TRUE;
	if (wizard){
	      prt("Identify objects upto which level (0-200) ? ", 0, 0);
	      i = -1;
	      if (get_string(tmp_str, 0, 47, 10))
		i = atoi(tmp_str);
	      if (i>200) i=200;
	      if (i>-1) {
		int j;
		inven_type inv;

		for (j=0; j<MAX_DUNGEON_OBJ; j++) {
		  if (object_list[j].level <= i) {
		    invcopy(&inv, j);
		    known1(&inv);
		  }
		}
	      }
	      erase_line(MSG_LINE, 0);
	} else msg_print("Type \":help\" for help.");
}
void wizlight(void){
	free_turn_flag=TRUE;
	if (wizard)
	      wizard_light(FALSE);
	else msg_print("Type \":help\" for help.");
}
void wizmap(void){
	free_turn_flag=TRUE;
	if (wizard)
	      map_area();
	else msg_print("Type \":help\" for help.");
}
void artif(void){
	free_turn_flag=TRUE;
	if (wizard)
	      artifact_check();
	else msg_print("Type \":help\" for help.");
}
void selfknow(void){
	free_turn_flag=TRUE;
	if (wizard)
	      self_knowledge();
	else msg_print("Type \":help\" for help.");
}
void wiztel(void){
	free_turn_flag=TRUE;
	if (wizard)
	      teleport(100);
	else msg_print("Type \":help\" for help.");
	}
void wizexp(void){
	free_turn_flag=TRUE;
	if (wizard){
	      if (command_count > 0)
		{
		  py.misc.exp = command_count;
		  command_count = 0;
		}
	      else if (py.misc.exp == 0)
		py.misc.exp = 1;
	      else
		py.misc.exp = py.misc.exp * 2;
	      prt_experience();
	} else msg_print("Type \":help\" for help.");
}
void summon(void){
	int x,y;
	free_turn_flag=TRUE;
	if (wizard){
	      y = char_row;
	      x = char_col;
	      (void) summon_monster(&y, &x, TRUE);
	      creatures(FALSE);
	 } else msg_print("Type \":help\" for help.");
}
void create(void){
	free_turn_flag=TRUE;
	if (wizard)
	      wizard_create();
	else msg_print("Type \":help\" for help.");
}

/* Check whether this command will accept a count.     -CJS-  */
static int valid_countcommand(c)
char c;
{
	int i;
	if (c==' ' || c==ESCAPE)
		return TRUE;
	for (i=0; i<NUM_COMMANDS; i++)
		if (c == (command_array[i].key))
		break;
  switch(i)
    {
    case 0:
	case 2:
    case 8:
    case 10:
    case 11:
    case 12:
    case 13: case 14: case 15: case 16:
	case 26: case 27: case 28: case 29:
	case 30: case 31: case 32: case 33:
	case 35:
	case 39:
	case 50:
	case 42:
	case 48:
    case 58:
    case 60:
    case 64:
    case 73:
    case 85:
      return TRUE;
    default:
      return FALSE;
    }
}


/* Regenerate hit points				-RAK-	*/
static void regenhp(percent)
int percent;
{
  register struct misc *p_ptr;
  register int32 new_chp, new_chp_frac;
  int old_chp;

  p_ptr = &py.misc;
  old_chp = p_ptr->chp;
  new_chp = ((long)p_ptr->mhp) * percent + PLAYER_REGEN_HPBASE;
  p_ptr->chp += new_chp >> 16;	/* div 65536 */
  /* check for overflow */
  if (p_ptr->chp < 0 && old_chp > 0)
    p_ptr->chp = MAX_SHORT;
  new_chp_frac = (new_chp & 0xFFFF) + p_ptr->chp_frac; /* mod 65536 */
  if (new_chp_frac >= 0x10000L)
    {
      p_ptr->chp_frac = new_chp_frac - 0x10000L;
      p_ptr->chp++;
    }
  else
    p_ptr->chp_frac = new_chp_frac;

  /* must set frac to zero even if equal */
  if (p_ptr->chp >= p_ptr->mhp)
    {
      p_ptr->chp = p_ptr->mhp;
      p_ptr->chp_frac = 0;
    }
  if (old_chp != p_ptr->chp)
    prt_chp();
}


/* Regenerate mana points				-RAK-	*/
static void regenmana(percent)
int percent;
{
  register struct misc *p_ptr;
  register int32 new_mana, new_mana_frac;
  int old_cmana;

  p_ptr = &py.misc;
  old_cmana = p_ptr->cmana;
  new_mana = ((long)p_ptr->mana) * percent + PLAYER_REGEN_MNBASE;
  p_ptr->cmana += new_mana >> 16;  /* div 65536 */
  /* check for overflow */
  if (p_ptr->cmana < 0 && old_cmana > 0)
    p_ptr->cmana = MAX_SHORT;
  new_mana_frac = (new_mana & 0xFFFF) + p_ptr->cmana_frac; /* mod 65536 */
  if (new_mana_frac >= 0x10000L)
    {
      p_ptr->cmana_frac = new_mana_frac - 0x10000L;
      p_ptr->cmana++;
    }
  else
    p_ptr->cmana_frac = new_mana_frac;

  /* must set frac to zero even if equal */
  if (p_ptr->cmana >= p_ptr->mana)
    {
      p_ptr->cmana = p_ptr->mana;
      p_ptr->cmana_frac = 0;
    }
  if (old_cmana != p_ptr->cmana)
    prt_cmana();
}


/* Is an item an enchanted weapon or armor and we don't know?  -CJS- */
/* only returns true if it is a good enchantment */
static int enchanted (t_ptr)
register inven_type *t_ptr;
{
  if (t_ptr->tval < TV_MIN_ENCHANT || t_ptr->tval > TV_MAX_ENCHANT
      || t_ptr->flags & TR_CURSED)
    return FALSE;
  if (known2_p(t_ptr))
    return FALSE;
  if (t_ptr->ident & ID_MAGIK)
    return FALSE;
  if (t_ptr->tohit > 0 || t_ptr->todam > 0 || t_ptr->toac > 0)
    return TRUE;
  if ((0x4000107fL & t_ptr->flags) && t_ptr->p1 > 0)
    return TRUE;
  if (0x07ffe980L & t_ptr->flags)
    return TRUE;

  return FALSE;
}

int ruin_stat(stat)
register int stat;
{
  register int tmp_stat, loss;

  tmp_stat = py.stats.cur_stat[stat];
  if (tmp_stat > 3) {
    if (tmp_stat > 6) {
      if (tmp_stat < 19) {
	tmp_stat-=3;
      } else {
	tmp_stat /= 2;
	if (tmp_stat < 18)
	  tmp_stat = 18;
      }
    } else tmp_stat--;

    py.stats.cur_stat[stat] = tmp_stat;
    py.stats.max_stat[stat] = tmp_stat;
    set_use_stat (stat);
    prt_stat (stat);
    return TRUE;
  } else return FALSE;
}

static void activate () {}
/* Commented out for now.		-- JLM --	
static void activate() {
  int i, a, flag, first, num, j, redraw, dir;
  char out_str[200], tmp[200], tmp2[200], tmp3[200], choice;
  inven_type *i_ptr;
  treasure_type *t_ptr;
  struct misc *m_ptr;

  flag=FALSE;
  redraw=FALSE;
  num=0;
  first=0;
  for (i=22; i<(INVEN_ARRAY_SIZE-1); i++) {
    if ((inventory[i].flags2 & TR_ACTIVATE) && (known2_p(&(inventory[i])))) {
      num++;
      if (!flag) first=i;
      flag=TRUE;
    }
  }
  if (!flag) {
    msg_print("You are not wearing/wielding anything that can be activated.");
    return;
  }
  sprintf(out_str, "Activate which item? (%c-%c, * to list, ESC to exit) ?",
	  'a', 'a'+(num-1));
  flag=FALSE;
  while (!flag && get_com(out_str, &choice)) {
    if (choice=='*') {
      save_screen();
      j=0;
      if (!redraw) {
	for (i=first; i<(INVEN_ARRAY_SIZE-1); i++) {
	  if ((inventory[i].flags2 & TR_ACTIVATE) && known2_p(&(inventory[i]))) {
	    objdes(tmp2, &inventory[i], TRUE);
	    sprintf(tmp, "%c) %-40s", 'a'+j, tmp2);
	    prt(tmp, 1+j, 36);
	    j++;
	  }
	}
	redraw=TRUE;
	continue;
      }
    } else {
      if (choice>='A' && choice<=('A'+(num-1)))
	choice-='A';
      else if (choice>='a' && choice<=('a'+(num-1)))
	choice-='a';
      else if (choice=='\033') {
	if (redraw)
	  restore_screen();
	free_turn_flag = TRUE;
	break;
      }
      if (redraw)
	restore_screen();

      if (choice>num) continue;
      flag=TRUE;
      j=0;
      for (i=first; i<(INVEN_ARRAY_SIZE-1); i++) {
	  if ((inventory[i].flags2 & TR_ACTIVATE) && known2_p(&(inventory[i]))) {
	    if (j==choice) break;
	    j++;
	  }
	}
      if (inventory[i].timeout>0) {
	msg_print("It whines, glows and fades...");
	break;
      }
      if (py.stats.use_stat[A_APT]<randint(18) &&
	  randint(object_list[inventory[i].index].level)>py.misc.lev) {
	msg_print("You fail to activate it properly.");
	break;
      }
      msg_print("You activate it...");
      switch (inventory[i].index) {
      case (29):
      case (395):
      case (396): /* The dreaded daggers:-> *\/
      case (397):
	if (inventory[i].name2 == SN_NARTHANC) {
	  msg_print("Your Dagger is covered in fire...");
	  if (get_dir(NULL, &dir)) {
	    if (py.flags.confused > 0) {
	      msg_print("You are confused.");
	      do {dir = randint(9);} while (dir == 5);
	    }
	    fire_bolt(GF_FIRE, dir, char_row, char_col, damroll(9,8),
		      "Fire Bolt");
	    inventory[i].timeout=5+randint(10);
	  }
	}
	else if (inventory[i].name2 == SN_NIMTHANC) {
	  msg_print("Your Dagger is covered in frost...");
	  if (get_dir(NULL, &dir)) {
	    if (py.flags.confused > 0) {
	      msg_print("You are confused.");
	      do {dir = randint(9);} while (dir == 5);
	    }
	    fire_bolt(GF_FROST, dir, char_row, char_col, damroll(6,8),
		      "Frost Bolt");
	    inventory[i].timeout=4+randint(8);
	  }
	}
	else if (inventory[i].name2 == SN_DETHANC) {
	  msg_print("Your Dagger is covered in sparks...");
	  if (get_dir(NULL, &dir)) {
	    if (py.flags.confused > 0) {
	      msg_print("You are confused.");
	      do {dir = randint(9);} while (dir == 5);
	    }
	    fire_bolt(GF_LIGHTNING, dir, char_row, char_col, damroll(4,8),
		      "Lightning Bolt");
	    inventory[i].timeout=3+randint(7);
	  }
	}
	else if (inventory[i].name2 == SN_RILIA) {
	  msg_print("Your Dagger throbs deep green...");
	  if (get_dir(NULL, &dir)) {
	    if (py.flags.confused > 0) {
	      msg_print("You are confused.");
	      do {dir = randint(9);} while (dir == 5);
	    }
	    fire_ball(GF_POISON_GAS, dir, char_row, char_col, 12,
		      "Stinking Cloud");
	    inventory[i].timeout=3+randint(3);
	  }
	}
	else if (inventory[i].name2 == SN_BELANGIL) {
	  msg_print("Your Dagger is covered in frost...");
	  if (get_dir(NULL, &dir)) {
	    if (py.flags.confused > 0) {
	      msg_print("You are confused.");
	      do {dir = randint(9);} while (dir == 5);
	    }
	    fire_ball(GF_FROST, dir, char_row, char_col, 48,
		      "Frost Ball");
	    inventory[i].timeout=3+randint(7);
	  }
	}
	break;
      case (91):
	if (inventory[i].name2 == SN_DAL) {
	  msg_print("You feel energy flow through your feet...");
	  remove_fear();
	  cure_poison();
	  inventory[i].timeout = 5;
	}
	break;
      case (42):
      case (43):
	if (inventory[i].name2 == SN_RINGIL) {
	  msg_print("Your sword glows an intense blue...");
	  if (get_dir(NULL, &dir)) {
	    if (py.flags.confused > 0) {
	      msg_print("You are confused.");
	      do {dir = randint(9);} while (dir == 5);
	    }
	    fire_ball(GF_FROST, dir, char_row, char_col, 100,
		      "storm of Ice");
	    inventory[i].timeout=300;
	  }
	} else if (inventory[i].name2 == SN_ANDURIL) {
	  msg_print("Your sword glows an intense red...");
	  if (get_dir(NULL, &dir)) {
	    if (py.flags.confused > 0) {
	      msg_print("You are confused.");
	      do {dir = randint(9);} while (dir == 5);
	    }
	    fire_ball(GF_FIRE, dir, char_row, char_col, 72,
		      "huge ball of Fire");
	    inventory[i].timeout=400;
	  }
	}
	break;
      case (52):
	if (inventory[i].name2 == SN_FIRESTAR) {
	  msg_print("Your Morningstar rages in fire...");
	  if (get_dir(NULL, &dir)) {
	    if (py.flags.confused > 0) {
	      msg_print("You are confused.");
	      do {dir = randint(9);} while (dir == 5);
	    }
	    fire_ball(GF_FIRE, dir, char_row, char_col, 72,
		      "huge ball of Fire");
	    inventory[i].timeout=100;
	  }
	}
	break;
      case (92):
	if (inventory[i].name2 == SN_FEANOR) {
	  py.flags.fast += randint(25) + 15;
	  inventory[i].timeout=200;
	}
	break;
      case (59):
	if (inventory[i].name2 == SN_THEODEN) {
	  msg_print("The blade of your axe glows black...");
	  if (get_dir(NULL, &dir)) {
	    if (py.flags.confused > 0) {
	      msg_print("You are confused.");
 	      do {dir = randint(9);} while (dir == 5);
	    }
	    drain_life(dir, char_row, char_col, 120);
	    inventory[i].timeout=400;
	  }
	}
	break;
      case (62):
	if (inventory[i].name2 == SN_TURMIL) {
          msg_print("The head of your hammer glows white...");
 	  if (get_dir(NULL, &dir)) {
	    if (py.flags.confused > 0) {
	      msg_print("You are confused.");
	      do {dir = randint(9);} while (dir == 5);
	    }
	    drain_life(dir, char_row, char_col, 90);
	    inventory[i].timeout = 70;
	  }
	}
	break;
      case (1111):
	if (inventory[i].name2 == SN_CASPANION) {
	  msg_print("Your mail magically disarms traps...");
	  td_destroy();
	  inventory[i].timeout=10;
	}
	break;
      case (71):
	if (inventory[i].name2 == SN_AVAVIR) {
	  if (py.flags.word_recall == 0)
	    py.flags.word_recall = 15 + randint(20);
	  msg_print("The air about you becomes charged...");
	  inventory[i].timeout=200;
	}
	break;
      case (53):
	if (inventory[i].name2 == SN_TARATOL) {
	  if (py.flags.fast == 0)
	  py.flags.fast += randint(30) + 15;
	  inventory[i].timeout=166;
	}
	break;
      case (54):
	if (inventory[i].name2 == SN_ERIRIL) {
	  ident_spell();
	  inventory[i].timeout=10;
	}
        else if (inventory[i].name2 == SN_OLORIN) {
          probing();
          inventory[i].timeout=20;
        }
	break;
      case (67):
	if (inventory[i].name2 == SN_EONWE) {
	  msg_print("Your axe lets out a long, shrill note...");
	  mass_genocide(TRUE);
          inventory[i].timeout=1000;
        }
	break;
      case (68):
	if (inventory[i].name2 == SN_LOTHARANG) {
	  msg_print("Your Battle Axe radiates deep purple...");
	  hp_player(damroll(4, 7));
	  if (py.flags.cut>0) {
	    py.flags.cut=(py.flags.cut/2)-50;
	    if (py.flags.cut<0) py.flags.cut=0;
	    msg_print("You wounds heal.");
	  }
	  inventory[i].timeout=2+randint(2);
	}
	break;
      case (75):
	break;
      case (34):
      case (35):
	if (inventory[i].name2 == SN_ARUNRUTH) {
	  msg_print("Your sword glows a pale blue...");
	  if (get_dir(NULL, &dir)) {
	    if (py.flags.confused > 0) {
	      msg_print("You are confused.");
	      do {dir = randint(9);} while (dir == 5);
	    }
	    fire_bolt(GF_FROST, dir, char_row, char_col, damroll(12,8),
		      "bolt of Frost");
	    inventory[i].timeout=500;
	  }
	}
	break;
      case (64):
	if (inventory[i].name2 == SN_AEGLOS) {
	  msg_print("Your spear glows a bright white...");
	  if (get_dir(NULL, &dir)) {
	    if (py.flags.confused > 0) {
	      msg_print("You are confused.");
	      do {dir = randint(9);} while (dir == 5);
	    }
	    fire_ball(GF_FROST, dir, char_row, char_col, 100,
		      "huge ball of Frost");
	    inventory[i].timeout=500;
	  }
	} else if (inventory[i].name2 == SN_OROME) {
	    msg_print("Your spear pulsates...");
	    if (get_dir(NULL, &dir)) {
	      if (py.flags.confused > 0) {
		msg_print("You are confused.");
		do {dir = randint(9);} while (dir == 5);
	      }
	    wall_to_mud(dir, char_row, char_col);
	    inventory[i].timeout=5;
	  }
	}
	break;
      case (118):
	if (inventory[i].name2 == SN_SOULKEEPER) {
	  msg_print("Your armour glows a bright white...");
	  msg_print("You feel much better...");
	  hp_player(1000);
	  inventory[i].timeout=888;
	}
	break;
      case (120):
	if (inventory[i].name2 == SN_BELEGENNON) {
	  teleport(10);
	  inventory[i].timeout=2;
      }
      break;
      case (119):
        if (inventory[i].name2 == SN_CELEBORN) {
          genocide(TRUE);
          inventory[i].timeout=500;
        }
        break;
      case (124):
	if (inventory[i].name2 == SN_LUTHIEN) {
	  restore_level();
	  inventory[i].timeout=450;
	}
	break;
      case (65):
	if (inventory[i].name2 == SN_ULMO) {
	  msg_print("Your trident glows deep red...");
	  if (get_dir(NULL, &dir)) {
	    if (py.flags.confused > 0) {
	      msg_print("You are confused.");
	      do {dir = randint(9);} while (dir == 5);
	    }
	    teleport_monster(dir, char_row, char_col);
	    inventory[i].timeout=150;
	  }
	}
	break;
      case (123): /* Cloak *\/
      case (411):
	if (inventory[i].name2 == SN_COLLUIN) {
	  msg_print("Your cloak glows many colours...");
	  msg_print("You feel you can resist anything.");
	  py.flags.resist_heat += randint(20) + 20;
	  py.flags.resist_cold += randint(20) + 20;
	  py.flags.resist_light += randint(20) + 20;
	  py.flags.resist_poison += randint(20) + 20;
	  py.flags.resist_acid += randint(20) + 20;
	  inventory[i].timeout=111;
	} else if (inventory[i].name2 == SN_HOLCOLLETH) {
	  msg_print("You momentarily disappear...");
	  sleep_monsters1(char_row, char_col);
	  inventory[i].timeout=55;
	} else if (inventory[i].name2 == SN_THINGOL) {
	  msg_print("You hear a low humming noise...");
	  recharge(60);
	  inventory[i].timeout=70;
	} else if (inventory[i].name2 == SN_COLANNON) {
	  teleport(100);
	  inventory[i].timeout=45;
	}
	break;
      case (50): /*Flail*\/
	if (inventory[i].name2 == SN_TOTILA) {
	  msg_print("Your Flail glows in scintillating colours...");
	  if (get_dir(NULL, &dir)) {
	    if (py.flags.confused > 0) {
	      msg_print("You are confused.");
	      do {dir = randint(9);} while (dir == 5);
	    }
	    confuse_monster(dir, char_row, char_col);
	    inventory[i].timeout=15;
	  }
	}
	break;
      case (125): /*Gloves*\/
	if (inventory[i].name2 == SN_CAMMITHRIM) {
	  msg_print("Your Gloves glow extremely brightly...");
	  if (get_dir(NULL, &dir)) {
	    if (py.flags.confused > 0) {
	      msg_print("You are confused.");
	      do {dir = randint(9);} while (dir == 5);
	    }
	    fire_bolt(GF_MAGIC_MISSILE, dir, char_row, char_col, damroll(2,6),
		      "Magic Missile");
	    inventory[i].timeout=2;
	  }
	}
	break;
      case (126): /*Gauntlets*\/
	if (inventory[i].name2 == SN_PAURHACH) {
	  msg_print("Your Gauntlets are covered in fire...");
	  if (get_dir(NULL, &dir)) {
	    if (py.flags.confused > 0) {
	      msg_print("You are confused.");
	      do {dir = randint(9);} while (dir == 5);
	    }
	    fire_bolt(GF_FIRE, dir, char_row, char_col, damroll(9,8),
		      "Fire Bolt");
	    inventory[i].timeout=5+randint(10);
	  }
	}
	else if (inventory[i].name2 == SN_PAURNIMMEN) {
	  msg_print("Your Gauntlets are covered in frost...");
	  if (get_dir(NULL, &dir)) {
	    if (py.flags.confused > 0) {
	      msg_print("You are confused.");
	      do {dir = randint(9);} while (dir == 5);
	    }
	    fire_bolt(GF_FROST, dir, char_row, char_col, damroll(6,8),
		      "Frost Bolt");
	    inventory[i].timeout=4+randint(8);
	  }
	}
	else if (inventory[i].name2 == SN_PAURAEGEN) {
	  msg_print("Your Gauntlets are covered in sparks...");
	  if (get_dir(NULL, &dir)) {
	    if (py.flags.confused > 0) {
	      msg_print("You are confused.");
	      do {dir = randint(9);} while (dir == 5);
	    }
	    fire_bolt(GF_LIGHTNING, dir, char_row, char_col, damroll(4,8),
		      "Lightning Bolt");
	    inventory[i].timeout=3+randint(7);
	  }
	}
	else if (inventory[i].name2 == SN_PAURNEN) {
	  msg_print("Your Gauntlets look very acidic...");
	  if (get_dir(NULL, &dir)) {
	    if (py.flags.confused > 0) {
	      msg_print("You are confused.");
	      do {dir = randint(9);} while (dir == 5);
	    }
	    fire_bolt(GF_ACID, dir, char_row, char_col, damroll(5,8),
		      "Acid Bolt");
	    inventory[i].timeout=4+randint(7);
	  }
	}
	break;
      case (127):
	if (inventory[i].name2 == SN_FINGOLFIN) {
	  msg_print("Magical spikes appear on your Cestus...");
	  if (get_dir(NULL, &dir)) {
	    if (py.flags.confused > 0) {
	      msg_print("You are confused.");
	      do {dir = randint(9);} while (dir == 5);
	    }
	    fire_bolt(GF_MAGIC_MISSILE, dir, char_row, char_col, 150,
		      "set of spikes");
	    inventory[i].timeout=88+randint(88);
	  }
	}
	break;
      case (96):
	if (inventory[i].name2 == SN_HOLHENNETH) {
	  msg_print("You close your eyes, an image forms in your mind...");
	  detection();
	  inventory[i].timeout=55+randint(55);
	}
	break;
      case (SPECIAL_OBJ-1): /* Narya *\/
	msg_print("The ring glows deep red...");
        if (get_dir(NULL, &dir)) {
          if (py.flags.confused > 0) {
	    msg_print("You are confused.");
	    do {dir = randint(9);} while (dir == 5);
	  }
	  fire_ball(GF_FIRE, dir, char_row, char_col, 120,
		    "huge ball of Fire");
	  inventory[i].timeout=222+randint(222);
	}
	break;
      case (SPECIAL_OBJ): /* Nenya *\/
	msg_print("The ring glows bright white...");
	if (get_dir(NULL, &dir)) {
	  if (py.flags.confused > 0) {
	    msg_print("You are confused.");
	    do {dir = randint(9);} while (dir == 5);
	  }
	  fire_ball(GF_FROST, dir, char_row, char_col, 200,
		    "huge ball of Frost");
	  inventory[i].timeout=222+randint(333);
	}
	break;
      case (SPECIAL_OBJ+1): /* Vilya *\/
	msg_print("The ring glows deep blue...");
	if (get_dir(NULL, &dir)) {
	  if (py.flags.confused > 0) {
	    msg_print("You are confused.");
	    do {dir = randint(9);} while (dir == 5);
	  }
	  fire_ball(GF_LIGHTNING, dir, char_row, char_col, 250,
		    "huge ball of Lightning");
	  inventory[i].timeout=222+randint(444);
	}
	break;
      case (SPECIAL_OBJ+2): /* Power *\/
	msg_print("The ring glows intensely black...");
	switch (randint(17)+(8-py.misc.lev/10)) {
	case 5:
	  dispel_creature(0xFFFFFFFL, 1000);
	  break;
	case 6:
	case 7:
	  msg_print("You are surrounded by a malignant aura");
	  m_ptr = &py.misc;
	  m_ptr->lev--;
	  m_ptr->exp=player_exp[m_ptr->lev-2]*m_ptr->expfact/100 +
	    randint((player_exp[m_ptr->lev-1]*m_ptr->expfact/100) -
		    (player_exp[m_ptr->lev-2]*m_ptr->expfact/100));
	  m_ptr->max_exp=m_ptr->exp;
	  prt_experience();
	  ruin_stat(A_STR);
	  ruin_stat(A_APT);
	  ruin_stat(A_MCP);
	  ruin_stat(A_DEX);
	  ruin_stat(A_CON);
	  ruin_stat(A_CHR);
	  calc_hitpoints();
	  if (m_ptr->pclass ==CL_CYBORG || m_ptr->pclass== CL_ROBOT) {
								/* Class == Cyborg or Robot	*\/
	    calc_spells(A_APT);
	    calc_mana(A_APT);
	  }
	  prt_level();
	  prt_title();
	  take_hit((py.misc.chp>2)?py.misc.chp/2:0, "malignant aura");
	  break;
	case 8:
	case 9:
	case 10:
	  if (get_dir(NULL, &dir)) {
	    if (py.flags.confused > 0) {
	      msg_print("You are confused.");
	      do {dir = randint(9);} while (dir == 5);
	    }
	    fire_ball(GF_MAGIC_MISSILE, dir, char_row, char_col, 300,
		      "huge ball of Raw Mana");
	  }
	  break;
	default:
	  if (get_dir(NULL, &dir)) {
	    if (py.flags.confused > 0) {
	      msg_print("You are confused.");
	      do {dir = randint(9);} while (dir == 5);
	    }
	    fire_bolt(GF_MAGIC_MISSILE, dir, char_row, char_col, 250,
		      "bolt of Raw Mana");
	  }
	}
	inventory[i].timeout=444+randint(444);
	break;
      case (389): /* Blue *\/
	msg_print("You breathe Lightning...");
	if (get_dir(NULL, &dir)) {
	  if (py.flags.confused > 0) {
	    msg_print("You are confused.");
	    do {dir = randint(9);} while (dir == 5);
	  }
	  fire_ball(GF_LIGHTNING, dir, char_row, char_col, 100,
		    "huge ball of Lightning");
	  inventory[i].timeout=444+randint(444);
	}
	break;
      case (390): /* White *\/
	msg_print("You breathe Frost...");
	if (get_dir(NULL, &dir)) {
	  if (py.flags.confused > 0) {
	    msg_print("You are confused.");
	    do {dir = randint(9);} while (dir == 5);
	  }
	  fire_ball(GF_FROST, dir, char_row, char_col, 110,
		    "huge ball of Frost");
	  inventory[i].timeout=444+randint(444);
	}
	break;
      case (391): /* Black *\/
	msg_print("You breathe Acid...");
	if (get_dir(NULL, &dir)) {
	  if (py.flags.confused > 0) {
	    msg_print("You are confused.");
	    do {dir = randint(9);} while (dir == 5);
	  }
	  fire_ball(GF_ACID, dir, char_row, char_col, 130,
		    "huge ball of Acid");
	  inventory[i].timeout=444+randint(444);
	}
	break;
      case (392): /* Gas *\/
	msg_print("You breathe Poison Gas...");
	if (get_dir(NULL, &dir)) {
	  if (py.flags.confused > 0) {
	    msg_print("You are confused.");
	    do {dir = randint(9);} while (dir == 5);
	  }
	  fire_ball(GF_POISON_GAS, dir, char_row, char_col, 150,
		    "huge cloud of Poison Gas");
	  inventory[i].timeout=444+randint(444);
	}
	break;
      case (393): /* Fire *\/
	msg_print("You breathe Fire...");
	if (get_dir(NULL, &dir)) {
	  if (py.flags.confused > 0) {
	    msg_print("You are confused.");
	    do {dir = randint(9);} while (dir == 5);
	  }
	  fire_ball(GF_FIRE, dir, char_row, char_col, 200,
		    "huge ball of Fire");
	  inventory[i].timeout=444+randint(444);
	}
	break;
      case (394): /* Multi-hued *\/
	if (inventory[i].name2 == SN_RAZORBACK) {
	  msg_print("A storm of lightning spikes fires in all directions...");
	  starball(char_row, char_col);
	  inventory[i].timeout=1000;
        }
	else {
	  if (get_dir(NULL, &dir)) {
	    if (py.flags.confused > 0) {
	      msg_print("You are confused.");
	      do {dir = randint(9);} while (dir == 5);
	    }
	    choice=randint(5);
	    sprintf(tmp2, "You breathe %s...",
	            ((choice==1)?"Lightning":
	             ((choice==2)?"Frost":
		      ((choice==3)?"Acid":
		       ((choice==4)?"Poison Gas":"Fire")))));
	    msg_print(tmp2);
	    sprintf(tmp2, "huge %s of %s", ((choice==4)?"cloud":"ball"),
	            ((choice==1)?"Lightning":
	             ((choice==2)?"Frost":
		      ((choice==3)?"Acid":
		       ((choice==4)?"Poison Gas":"Fire")))));
	    fire_ball(((choice==1)?GF_LIGHTNING:
	               ((choice==2)?GF_FROST:
	                ((choice==3)?GF_ACID:
	                 ((choice==4)?GF_POISON_GAS:GF_FIRE)))),
		      dir, char_row, char_col, 250,
		      tmp2);
	    inventory[i].timeout=222+randint(222);
	  }
	}
	break;
      case (408): /* Bronze *\/
	msg_print("You breathe Confusion...");
	if (get_dir(NULL, &dir)) {
	  if (py.flags.confused > 0) {
	    msg_print("You are confused.");
	    do {dir = randint(9);} while (dir == 5);
	  }
	  fire_ball(GF_MAGIC_MISSILE, dir, char_row, char_col, 120,
		    "huge blast of Confusion");
	  inventory[i].timeout=444+randint(444);
	}
      break;
      case (409): /* Gold *\/
	msg_print("You breathe Sound...");
	if (get_dir(NULL, &dir)) {
	  if (py.flags.confused > 0) {
	    msg_print("You are confused.");
	    do {dir = randint(9);} while (dir == 5);
	  }
	  fire_ball(GF_MAGIC_MISSILE, dir, char_row, char_col, 130,
		    "huge blast of Sound");
	  inventory[i].timeout=444+randint(444);
	}
      break;
      case (415): /* Chaos *\/
	if (get_dir(NULL, &dir)) {
	  if (py.flags.confused > 0) {
	    msg_print("You are confused.");
	    do {dir = randint(9);} while (dir == 5);
	  }
	  choice = randint(2);
	  sprintf(tmp2, "You breathe %s...",
	          ((choice==1?"Chaos":"Disenchantment")));
	  msg_print(tmp2);
	  sprintf(tmp2, "huge ball of %s",
		  ((choice==1?"Chaos":"Disenchantment")));
	  fire_ball(GF_MAGIC_MISSILE, dir, char_row, char_col, 220, tmp2);
	  inventory[i].timeout=300+randint(300);
	}
        break;
      case (416): /* Law *\/
        if (get_dir(NULL, &dir)) {
          if (py.flags.confused > 0) {
            msg_print("You are confused.");
            do {dir = randint(9);} while (dir == 5);
          }
          choice = randint(2);
          sprintf(tmp2, "You breathe %s...",
                  ((choice==1?"Sound":"Shards")));
          msg_print(tmp2);
          sprintf(tmp2, "huge %s of %s", ((choice==1)?"blast":"ball"),
                  ((choice==1?"Sound":"Shards")));
          fire_ball(GF_MAGIC_MISSILE, dir, char_row, char_col, 230, tmp2);
          inventory[i].timeout=300+randint(300);
        }
        break;
      case (417): /* Balance *\/
	if (get_dir(NULL, &dir)) {
          if (py.flags.confused > 0) {
            msg_print("You are confused.");
            do {dir = randint(9);} while (dir == 5);
          }
          choice=randint(4);
          sprintf(tmp2, "You breathe %s...",
                  ((choice==1)?"Chaos":
                   ((choice==2)?"Disenchantment":
                    ((choice==3)?"Sound":"Shards"))));
          msg_print(tmp2);
          sprintf(tmp2, "huge %s of %s", ((choice==3)?"blast":"ball"),
                  ((choice==1)?"Chaos":
                   ((choice==2)?"Disenchantment":
                    ((choice==3)?"Sound":"Shards"))));
          fire_ball(GF_MAGIC_MISSILE, dir, char_row, char_col, 250, tmp2);
          inventory[i].timeout=300+randint(300);
        }
        break;
      case (418): /* Shining *\/
        if (get_dir(NULL, &dir)) {
          if (py.flags.confused > 0) {
            msg_print("You are confused.");
            do {dir = randint(9);} while (dir == 5);
          }
          choice = randint(2);
          sprintf(tmp2, "You breathe %s...",
                  ((choice==1?"Light":"Darkness")));
          msg_print(tmp2);
          sprintf(tmp2, "huge %s of %s", ((choice==1)?"flash":"sphere"),
                  ((choice==1?"Light":"Darkness")));
          fire_ball(GF_MAGIC_MISSILE, dir, char_row, char_col, 200, tmp2);
          inventory[i].timeout=300+randint(300);
        }
        break;
      case (419): /* Power Dragon Scale Mail *\/
	if (inventory[i].name2 == SN_BLADETURNER) {
	  msg_print("Your armour glows many colours...");
	  msg_print("You enter a berserk rage...");
	  py.flags.hero += randint(50)+50;
	  py.flags.shero += randint(50)+50;
	  bless(randint(50)+50);
	  py.flags.resist_heat += randint(50)+50;
	  py.flags.resist_cold += randint(50)+50;
	  py.flags.resist_light += randint(50)+50;
	  py.flags.resist_acid += randint(50)+50;
	  inventory[i].timeout=400;
        }
        else {
          msg_print("You breathe the Elements...");
          if (get_dir(NULL, &dir)) {
            if (py.flags.confused > 0) {
              msg_print("You are confused.");
              do {dir = randint(9);} while (dir == 5);
            }
            fire_ball(GF_MAGIC_MISSILE, dir, char_row, char_col, 300,
                      "massive ball of the Elements");
            inventory[i].timeout=300+randint(300);
          }
	}
        break;
      case (SPECIAL_OBJ+3):
	msg_print("The phial wells with clear light...");
	light_area(char_row, char_col);
	inventory[i].timeout=10+randint(10);
	break;
    /*  case (SPECIAL_OBJ+4):
	msg_print("An aura of good floods the area...");
	dispel_creature(EVIL, (int)(5*py.misc.lev));
	inventory[i].timeout=444+randint(222);
	break;
	*\/
      case (SPECIAL_OBJ+5):
        msg_print("The amulet lets out a shrill wail...");
	msg_print("You feel somewhat safer...");
	protect_evil();
	inventory[i].timeout=222+randint(222);
	break;
      case (SPECIAL_OBJ+6):
	msg_print("The star shines brightly...");
	msg_print("And you sense your surroundings...");
	map_area();
	inventory[i].timeout=50+randint(50);
        break;
      case (SPECIAL_OBJ+7):
        msg_print("The stone glows a deep green");
        wizard_light(TRUE);
        inventory[i].timeout=100+randint(100);
        break;
      case (SPECIAL_OBJ+8):
	msg_print("The ring glows brightly...");
	py.flags.fast += randint(100) + 50;
	inventory[i].timeout=200;
	break;
      default:
	(void) sprintf(tmp2, "Inventory num %d, index %d", i,
		       inventory[i].index);
	msg_print(tmp2);
      }
    }
  }
  if (redraw && !flag)
    restore_screen();
}
*/

/* Examine a Book					-RAK-	*/
static void examine_book()
{
  int32u j1;
  int32u j2;
  int i, k, item_val, flag;
  int spell_index[63];
  register inven_type *i_ptr;
  register spell_type *s_ptr;
  char tmp[100];

  if (!find_range(TV_CD_ROM, TV_CD_ROM, &i, &k))
    msg_print("You are not carrying any CD-ROMs.");
  else if (py.flags.confused > 0)
    msg_print("You are too confused.");
  else if (get_item(&item_val, "Which CD-ROM?", i, k, 0))
    {
      i_ptr = &inventory[item_val];
      if (py.misc.pclass == CL_ROBOT || py.misc.pclass==CL_CYBORG)
		flag=TRUE;
	  else flag=FALSE;
      if (!flag)
	msg_print("You do not have the equipment.");
      else
	{
	  i = 0;
	  j1 = (int32u) inventory[item_val].flags;
	  while (j1)
	    {
	      k = bit_pos(&j1);
	      s_ptr = &magic_spell[py.misc.pclass-1][k];
	      if (s_ptr->slevel < 99)
		{
		  spell_index[i] = k;
		  i++;
		}
	    }
	  j2 = (int32u) inventory[item_val].flags2;
	  while (j2)
	    {
	      k = bit_pos(&j2);
	      s_ptr = &magic_spell[py.misc.pclass-1][k+32];
	      if (s_ptr->slevel < 99)
		{
		  spell_index[i] = (k+32);
		  i++;
		}
	    }
	  save_screen();
	  print_spells(spell_index, i, TRUE, -1);
	  pause_line(0);
	  restore_screen();
	}
    }
}

/* Go up one level					-RAK-	*/
static void go_up()
{
  register cave_type *c_ptr;
  register int no_stairs = FALSE;

  c_ptr = &cave[char_row][char_col];
  if (c_ptr->tptr != 0)
    if (t_list[c_ptr->tptr].tval == TV_UP_STAIR)
      {
	if (dun_level == Q_PLANE) {
	  dun_level=0;
	  new_level_flag=TRUE;
	  msg_print("You enter an inter-dimensional staircase.");
	} else {
	  dun_level--;
	  new_level_flag = TRUE;
	  if (dun_level>0) create_down_stair = TRUE;
	  msg_print("You enter a maze of up staircases.");
	}
      }
    else
      no_stairs = TRUE;
  else
    no_stairs = TRUE;

  if (no_stairs)
    {
      msg_print("I see no up staircase here.");
      free_turn_flag = TRUE;
    }
}


/* Go down one level					-RAK-	*/
static void go_down()
{
  register cave_type *c_ptr;
  register int no_stairs = FALSE;

  c_ptr = &cave[char_row][char_col];
  if (c_ptr->tptr != 0)
    if (t_list[c_ptr->tptr].tval == TV_DOWN_STAIR)
      {
	if (dun_level == Q_PLANE) {
	  dun_level=0;
	  new_level_flag=TRUE;
	  msg_print("You enter an inter-dimensional staircase.");
	} else {
	  dun_level++;
	  new_level_flag = TRUE;
	  create_up_stair = TRUE;
	  msg_print("You enter a maze of down staircases.");
	}
      }
    else
      no_stairs = TRUE;
  else
    no_stairs = TRUE;

  if (no_stairs)
    {
      msg_print("I see no down staircase here.");
      free_turn_flag = TRUE;
    }
}


/* Jam a closed door					-RAK-	*/
static void jamdoor()
{
  int y, x, dir, i, j;
  register cave_type *c_ptr;
  register inven_type *t_ptr, *i_ptr;
  char tmp_str[80];

  free_turn_flag = TRUE;
  y = char_row;
  x = char_col;
  if (get_dir(NULL, &dir))
    {
      (void) mmove(dir, &y, &x);
      c_ptr = &cave[y][x];
      if (c_ptr->tptr != 0)
	{
	  t_ptr = &t_list[c_ptr->tptr];
	  if (t_ptr->tval == TV_CLOSED_DOOR)
	    if (c_ptr->cptr == 0)
	      {
		if (find_range(TV_SPIKE, TV_NEVER, &i, &j))
		  {
		    free_turn_flag = FALSE;
		    count_msg_print("You jam the door with a spike.");
		    if (t_ptr->p1 > 0)
		      t_ptr->p1 = -t_ptr->p1;	/* Make locked to stuck. */
		    /* Successive spikes have a progressively smaller effect.
		       Series is: 0 20 30 37 43 48 52 56 60 64 67 70 ... */
		    t_ptr->p1 -= 1 + 190 / (10 - t_ptr->p1);
		    i_ptr = &inventory[i];
		    if (i_ptr->number > 1)
		      i_ptr->number--;
		    else
		      inven_destroy(i);
		  }
		else
		  msg_print("But you have no spikes.");
	      }
	    else
	      {
		free_turn_flag = FALSE;
		(void) sprintf(tmp_str, "The %s is in your way!",
			       c_list[m_list[c_ptr->cptr].mptr].name);
		msg_print(tmp_str);
	      }
	  else if (t_ptr->tval == TV_OPEN_DOOR)
	    msg_print("The door must be closed first.");
	  else
	    msg_print("That isn't a door!");
	}
      else
	msg_print("That isn't a door!");
    }
}


/* Refill the players lamp				-RAK-	*/
static void refill_lamp()
{
  int i, j;
  register int k;
  register inven_type *i_ptr;

  free_turn_flag = TRUE;
  k = inventory[INVEN_LIGHT].subval;
  if (k != 0)
    msg_print("But you are not using a lamp.");
  else if (!find_range(TV_FLASK, TV_NEVER, &i, &j))
    msg_print("You have no oil.");
  else
    {
      free_turn_flag = FALSE;
      i_ptr = &inventory[INVEN_LIGHT];
      i_ptr->p1 += inventory[i].p1;
      if (i_ptr->p1 > OBJ_LAMP_MAX)
	{
	  i_ptr->p1 = OBJ_LAMP_MAX;
	  msg_print ("Your lamp overflows, spilling oil on the ground.");
	  msg_print("Your lamp is full.");
	}
      else if (i_ptr->p1 > OBJ_LAMP_MAX/2)
	msg_print ("Your lamp is more than half full.");
      else if (i_ptr->p1 == OBJ_LAMP_MAX/2)
	msg_print ("Your lamp is half full.");
      else
	msg_print ("Your lamp is less than half full.");
      desc_remain(i);
      inven_destroy(i);
    }
}

int is_quest(level)
  int level;
{
  if (level == Q_PLANE)
    return FALSE;
  if (quests[SAURON_QUEST] && (level==quests[SAURON_QUEST]))
    return TRUE;
  return FALSE;
}

static int regen_monsters() {
  register int i;
  int frac;

  for (i=0; i<MAX_MALLOC; i++)
    if (m_list[i].hp>0)
      if (m_list[i].hp < m_list[i].maxhp) {
	m_list[i].hp+= ((frac=2*m_list[i].maxhp/100)>0)? frac : 1;
	if (m_list[i].hp > m_list[i].maxhp)
	  m_list[i].hp=m_list[i].maxhp;
      }
}

void fcn_nothing(void) {}

command_type command_array [NUM_COMMANDS] =
	{
		{'1', southw, "southwest"},
		{'2', south, "south"},
		{'3', southe, "southeast"},
		{'4', west, "west"},
		{'6', east, "east"},
		{'7', northw, "northwest"},
		{'8', north, "north"},
		{'9', northe, "northeast"},
		{'5', stay, "stay"},
		{'\0', runsw, "runsw"},
		{'\0', runs, "runs"},
		{'\0', runse, "runse"},
		{'\0', runw, "runw"},
		{'\0', rune, "rune"},
		{'\0', runnw, "runnw"},
		{'\0', runn, "runn"},
		{'\0', runne, "runne"},
		{'.', run_dir, "run"},
		{'\0', tunsw, "tunsw"},
		{'\0', tuns, "tuns"},
		{'\0', tunse, "tunse"},
		{'\0', tunw, "tunw"},
		{'\0', tune, "tune"},
		{'\0', tunnw, "tunnw"},
		{'\0', tunn, "tunn"},
		{'\0', tunne, "tunne"},
		{'T', tunnelling, "tunnel"},
		{'<', go_up, "up"},
		{'>', go_down, "down"},
		{'?', help, "help"},
		{'/', identi, "ident"},
		{'E', eat, "eat"},
		{'F', refill_lamp, "fill"},
		{'l', examine, "look"},
		{'W', locate, "locate"},
		{'R', rest, "rest"},
		{'M', map, "map"},
		{'o', openobject, "open"},
		{'c', closeobject, "close"},
		{'-', nopickup, "nopickup"},
		{'i', invent, "inventory"},
		{'e', equip, "equipment"},
		{'w', wield, "wield"},
		{'d', drop_obj, "drop"},
		{'f', throw_object, "throw"},
		{'x', exchg, "exchange"},
		{'L', loading_gun, "load"},
		{'U', unload, "unload"},
		{CTRL('X'), exit_save, "save"},
		{CTRL('K'), Kill, "kill"},
		{CTRL('P'), prev_msg, "prevmsg"},
		{'q', inject, "inject"},
		{'S', shoot_gun, "shoot"},
		{'s', search_turn, "search"},
		{'I', search_tog, "searchmode"},
		{'C', charsheet, "charsheet"},
		{'j', jamdoor, "spike"},
		{'b', brouse, "brouse"},
		{'r', cyber_spell, "execute"},
		{'A', activate_implant, "activate"},
		{'u', use, "use"},
		{'t', Remove, "remove"},
		{'V', version, "version"},
		{'=', set_opt, "options"},
		{'{', enscribe, "inscribe"},	/* intentionally misspelled to avoid
										**	confusion with another function. */
		{'#', count_cmd, "count"},
		{'B', bash, "bash"},
		{'D', disarm_trap, "disarm"},
		{'G', gain_spells, "learn"},
		{'\0', fcn_nothing, ""},
		{'\0', fcn_nothing, ""},
		{'\0', fcn_nothing, ""},
		{CTRL('W'), enter_wiz, "wizard"},
		{CTRL('H'), wizhelp, "wizhelp"},
		{CTRL('I'), wizid, "wizid"},
		{CTRL('N'), idall, "idall"},
		{CTRL('A'), cureall, "cureall"},
		{CTRL('E'), charedit, "editchar"},
		{'!', rerate_hp, "rerate"},
		{CTRL('F'), geno, "genocide"},
		{CTRL('D'), change_level, "changelevel"},
		{CTRL('G'), make_treasure, "treasure"},
		{CTRL('V'), make_special, "special"},
		{'~', artif, "artifacts"},
		{CTRL('O'), wizmap, "maparea"},
		{CTRL('L'), wizlight, "wizlight"},
		{'+', wizexp, "experience"},
		{CTRL('U'), summon, "summon"},
		{'%', selfknow, "selfknow"},
		{CTRL('T'), wiztel, "teleport"},
		{'@', create, "create"}	,
		{CTRL('B'), disp_objects, "objects"}
		};

static char origcmds_array[NUM_COMMANDS] =
/*	{ '-', CTRL('K'), CTRL('P'), CTRL('W'), CTRL('X'), '=', '{', '!',
	'1', '#', '2', '3', '4', '6', '7', '8', '9', '.', '\0',
	'\0', '\0', '\0', '\0', '\0', '\0', '\0', '\0', '\0', '\0', '\0',
	 '\0', '\0', '\0', '\0', '/', '5', '>', '<', '?', 'B', 'A',
	'C', 'D', 'E', 'F', 'G', 'L', 'W', 'R', 'I', 'T', 'M', 'b', 'c',
	'd', 'e', 'f', 'i', 'j', 'l', 'o', 'q', 'r', 'S', 's', 't',
	'u', 'V', 'w', 'x', CTRL('A'), CTRL('E'), CTRL('F'), CTRL('G'),
	CTRL('V'), CTRL('D'), CTRL('B'), CTRL('H'), CTRL('I'), CTRL('N'),
	CTRL('L'), CTRL('O'), '~', '%', CTRL('T'), '+', CTRL('U'), '@',
	'U', '\0' }; */
	{ '1', '2', '3', '4', '6', '7', '8', '9', '5', 
	'\0', '\0',  '\0',  '\0',  '\0',  '\0',  '\0',  '\0', '.',
	'\0', '\0',  '\0',  '\0',  '\0',  '\0',  '\0',  '\0', 'T',
	'<', '>', '?', '/', 'E', 'F', 'l', 'W', 'R', 'M', 'o',
	'c', '-', 'i', 'e', 'w', 'd', 'f', 'x', 'L', 'U', CTRL('X'),
	CTRL('K'), CTRL('P'), 'q', 'S', 's', 'I', 'C', 'j', 'b', 'r', 
	'A', 'u', 't', 'V', '=', '{', '#', 'B', 'D', 'G', '\0',
	'\0', '\0', CTRL('W'),
	CTRL('H'), CTRL('I'), CTRL('N'), CTRL('A'), CTRL('E'),
	'!', CTRL('F'), CTRL('D'), CTRL('G'), CTRL('V'), '~',
	CTRL('O'), CTRL('L'), '+', CTRL('U'), '%', CTRL('T'),
	'@', CTRL('B')	};
	

static char rlikecmds_array[NUM_COMMANDS] =
/* 	{ '-', CTRL('Q'), CTRL('P'), CTRL('W'), CTRL('X'), '=', '{', '!',
	'b', '\0', 'j', 'n', 'h', 'l', 'y', 'k', 'u', '\0', 'B',
	'J', 'N', 'H', 'L', 'Y', 'K', 'U', CTRL('B'), CTRL('J'),
	CTRL('N'), CTRL('H'), CTRL('L'), CTRL('Y'), CTRL('K'), CTRL('U'),
	'/', '.', '>', '<', '?', 'f', 'a', 'C', 'D', 'E', 'O', 'G', 'A', 
	'W', 'R', '#', '\0', 'M', 'P', 'c', 'd', 'e', 't', 'i', 'S', 'x', 
	'o', 'q', 'm', 'F', 's', 'T', 'z', 'V', 'w', 'X',
	CTRL('A'), CTRL('E'), CTRL('F'), CTRL('G'), CTRL('V'), CTRL('D'),
	CTRL('O'), '\\', CTRL('I'), CTRL('Z'), '*', ';', '~', '%',
	CTRL('T'), '+', '&', '@', 'r', '\0'	}; */
	{ 'b', 'j', 'n', 'h', 'l', 'y', 'k', 'u', '.', 
	'B', 'J', 'N', 'H', 'L', 'Y', 'K', 'U', '\0',
	CTRL('B'), CTRL('J'), CTRL('N'), CTRL('H'), CTRL('L'), CTRL('Y'),
	CTRL('K'), CTRL('U'), '\0', '<', '>', '?', '/', 'E', 'O', 'x', 
	'W', 'R', 'M', 'o', 'c', '-', 'i', 'e', 'w', 'd', 't', 'X', 'A',
	'r', CTRL('X'), 'Q', CTRL('P'), 'q', 'F', 's', '#', 
	'C', 'S', 'P', 'm', 'a', 'z', 'T', 'V', '=', '{', '\0',
	'f', 'D', 'G', '\0', '\0', '\0',
	CTRL('W'), '\\', CTRL('I'), '|', CTRL('A'),
	CTRL('E'), '!', CTRL('F'), CTRL('D'), CTRL('G'), CTRL('V'), 
	'~', '%', '*', '+', '&', CTRL('O'), CTRL('T'), '@', ';'
	};

void make_orig(void)
{
	int i;
	for (i=0; i<NUM_COMMANDS; i++)
	 	command_array[i].key = origcmds_array[i];
}

void make_rlike(void)
{
	int i;
	for (i=0; i<NUM_COMMANDS; i++)
		command_array[i].key = rlikecmds_array[i];
}

/* Prompts for a direction				-RAK-	*/
/* Direction memory added, for repeated commands.  -CJS */
int get_dir(prompt, dir)
char *prompt;
int *dir;
{
  char command;
  int save, i;
  static char prev_dir;		/* Direction memory. -CJS- */

  if (default_dir)	/* used in counted commands. -CJS- */
    {
      *dir = prev_dir;
      return TRUE;
    }
  if (prompt == NULL)
    prompt = "Which direction?";
  for (;;)
    {
      save = command_count;	/* Don't end a counted command. -CJS- */
#ifdef MAC
      if (!get_comdir(prompt, &command))
#else
      if (!get_com(prompt, &command))
#endif
	{
	  free_turn_flag = TRUE;
	  return FALSE;
	}
      command_count = save;
	for (i=0; i<NUM_COMMANDS; i++)
		if (command_array[i].key == command) break;
	if (i==NUM_COMMANDS) {bell(); continue; }
	if (!strcmp(command_array[i].name, "southwest"))
		prev_dir = 1;
	else if (!strcmp(command_array[i].name, "south"))
		prev_dir = 2;
	else if (!strcmp(command_array[i].name, "southeast"))
		prev_dir = 3;
	else if (!strcmp(command_array[i].name, "west"))
		prev_dir = 4;
	else if (!strcmp(command_array[i].name, "east"))
		prev_dir = 6;
	else if (!strcmp(command_array[i].name, "northwest"))
		prev_dir = 7;
	else if (!strcmp(command_array[i].name, "north"))
		prev_dir = 8;
	else if (!strcmp(command_array[i].name, "northeast"))
		prev_dir = 9;
    else  if (command >= '1' && command <= '9' && command != '5')
	  prev_dir = command - '0';
	else { bell();	continue; }
	*dir = prev_dir;
    return TRUE;
    }
}



/* Similar to get_dir, except that no memory exists, and it is		-CJS-
   allowed to enter the null direction. */
int get_alldir(prompt, dir)
char *prompt;
int *dir;
{
  char command;
  int i;
  static int prev_dir;

  for(;;)
    {
#ifdef MAC
      if (!get_comdir(prompt, &command))
#else
      if (!get_com(prompt, &command))
#endif
	{
	  free_turn_flag = TRUE;
	  return FALSE;
	}
	for (i=0; i<NUM_COMMANDS; i++)
		if (command_array[i].key == command) break;
	if (i==NUM_COMMANDS) {bell(); continue; }
	if (!strcmp(command_array[i].name, "southwest"))
		prev_dir = 1;
	else if (!strcmp(command_array[i].name, "south"))
		prev_dir = 2;
	else if (!strcmp(command_array[i].name, "southeast"))
		prev_dir = 3;
	else if (!strcmp(command_array[i].name, "west"))
		prev_dir = 4;
	else if (!strcmp(command_array[i].name, "east"))
		prev_dir = 6;
	else if (!strcmp(command_array[i].name, "northwest"))
		prev_dir = 7;
	else if (!strcmp(command_array[i].name, "north"))
		prev_dir = 8;
	else if (!strcmp(command_array[i].name, "northeast"))
		prev_dir = 9;
	else if (!strcmp(command_array[i].name, "stay") ||
			 !strcmp(command_array[i].name, "run"))
		prev_dir = 5;
    else  if (command >= '1' && command <= '9')
	  prev_dir = command - '0';
	else { bell();	continue; }
	*dir = prev_dir;
    return TRUE;
    }
}
