
/* 	guns.c	- code for dealing with guns.
	(C) 1994	Jacob L. Mandelson, California Inst. of Tech.
	Go ahead and distribute this software, just don't sell it for profit. */

#include "constant.h"
#include "types.h"
#include "externs.h"
#include "config.h"

#ifdef DEBUGGING
#define DBG(X)	msg_print(X);
#else
#define DBG(X)
#endif

/* ammo.h -		Contains the table that determines what ammo goes in
		which gun. */

/*
**	Gun:			gun subval:		Ammo takes:		Ammo subval:
**
**  .22				0				.22				65
**	.38				1				.38				66
**	9mm pistol		2				9mm				67
**	shotgun			3				shotgun shells	68
**	Needle gun		4				needles			69
**	shredder pulse	5				needles			69
**	M-16			6				5.56 mm			71
**	Gauss rifle		7				Gauss pellet	72
**	grenade l.		8				grenade			73
**	laser baz.		9				laserpack		74
**	laser rifle		10				laserpack		74
**	flamethrower	11				fuel pack		75
**	U pellet gun	12				U238 pellet		76
**	Uzi				13				9mm or 5.56?			67 or 71?
**  Chaingun		14				7.62 mm			77
**	Rocket l.		15				rocket			78
**	Weatherby		16				.50 caliber		79
**	Anti-Tank cannon	17			25 mm			80
**	Browning HMG		18			.50 caliber		79
*/

static int ammo_table[] = {65+AMMO_CONST, 66+AMMO_CONST, 67+AMMO_CONST,
 68+AMMO_CONST, 69+AMMO_CONST, 69+AMMO_CONST, 71+AMMO_CONST, 72+AMMO_CONST,
 73+AMMO_CONST, 74+AMMO_CONST, 74+AMMO_CONST, 75+AMMO_CONST, 76+AMMO_CONST,
	67+AMMO_CONST, 77+AMMO_CONST, 78+AMMO_CONST, 79+AMMO_CONST,
 80+AMMO_CONST, 79+AMMO_CONST };

extern void shoot_missed(int, int, int);


static int direction;

int load_gun(inven_type *gun, inven_type *ammo)
{
	/* Load the ammo into the gun. */

	char string[256], string2[256];

	if (gun->tval == TV_NOTHING) {
		sprintf(string, "You must be wielding a gun to load it.");
		msg_print(string);
		return -1;
	}

	if (gun->tval != TV_HANDGUN && gun->tval != TV_HEAVYGUN) {
		objdes(string, gun, 1);
		sprintf(string2, "You cannot load %s.", string);
		msg_print(string2);
		return -1;	}
	if (ammo->tval != TV_AMMO)	{
		msg_print("You must load your gun with ammunition.");
		return -1;
	}
	if (ammo->subval != ammo_table[gun->subval]) {
		msg_print("That doesn't seem to be the correct type of ammunition"
			" for your gun." );
		return -1;
		}
	/* If we made it this far, character has a gun, and has tried to
		load it with the correct type of ammo.	*/
	
	if (gun->shots && gun->flags2 != ammo->flags2)	/* Must unload gun first. */
	{
		inven_type old_ammo;

		/* old_ammo.name = ammo->name; */
		old_ammo.tval = ammo->tval;
		old_ammo.subval = ammo->subval;
		old_ammo.flags = 0U;
		old_ammo.tchar = ammo->tchar;
		old_ammo.p1 = 0;
		old_ammo.number = gun->shots;
		old_ammo.weight = ammo->weight;
		old_ammo.damage[0] = ammo->damage[0];
		old_ammo.damage[1] = ammo->damage[1];
		old_ammo.flags2 = gun->flags2;
		old_ammo.tohit = old_ammo.todam = old_ammo.ac = old_ammo.toac
			= old_ammo.shots = old_ammo.maxshots = 0;
		objdes(string, &old_ammo, 1); 
		sprintf(string2, "Unload %s", string );
		if (!get_check(string2)) return 0;
		if (ammo->number > gun->maxshots && inven_ctr==22)
		{
			objdes(string, &old_ammo, 1);
			sprintf(string2, 
				"You don't have room in your backpack for ", string);
			msg_print(string2);
			return 0;
		}
		if (ammo->number > gun->maxshots)
		{
			gun->shots = gun->maxshots;
			ammo->number -= gun->maxshots;
			gun->flags2 = ammo->flags2;
			/* inventory[inven_ctr].name = old_ammo.name; */
			inventory[inven_ctr].tval = old_ammo.tval;
			inventory[inven_ctr].subval = old_ammo.subval;
			inventory[inven_ctr].flags = 0U;
			inventory[inven_ctr].tchar = old_ammo.tchar;
			inventory[inven_ctr].p1 = 0;
			inventory[inven_ctr].number = old_ammo.number;
			inventory[inven_ctr].weight = old_ammo.weight;
			inventory[inven_ctr].damage[0] = old_ammo.damage[0];
			inventory[inven_ctr].damage[1] = old_ammo.damage[1];
			inventory[inven_ctr].flags2 = old_ammo.flags2;
			inventory[inven_ctr].tohit = inventory[inven_ctr].todam =
					inventory[inven_ctr].ac = inventory[inven_ctr].toac
				= inventory[inven_ctr].shots = inventory[inven_ctr].maxshots
				= 0;
			inventory[inven_ctr++].cost = item_value(&old_ammo);
			objdes(string, gun, 0);
			sprintf(string2, "Your %s now has %d shots.", string, gun->shots);
			msg_print(string2);
			return 1;	/* sucessfully loaded gun. */
		}
		else {	/* Gun is unloaded, but uses up all ammo in inven slot. */
			gun->shots = ammo->number;
			gun->flags2 = ammo->flags2;
			ammo->number = old_ammo.number;
			ammo->flags2= old_ammo.flags2;
			ammo->cost = item_value(&old_ammo);
			objdes(string, gun, 0);
			sprintf(string2, "Your %s now has %d shots.", string, gun->shots);
			msg_print(string2);
			return 1;	/* success! */
		}
	}
	else { /* Don't need to unload gun. */
		if ((gun->shots+ammo->number)<=gun->maxshots)	/* use up all ammo */
		{
			gun->shots += ammo->number;
			ammo->number = 0;
			inven_destroy(ammo-inventory);
			objdes(string, gun, 0);
			sprintf(string2, "Your %s now has %d shots.", string, gun->shots);
			msg_print(string2);
			return 1;	/* success */
		}
		else {	
			ammo->number -= gun->maxshots - gun->shots;
			gun->shots = gun->maxshots;
			objdes(string, gun, 0);
			sprintf(string2, "Your %s now has %d shots.", string, gun->shots);
		 	msg_print(string2);
		/* 	sprintf(string2, "It can hold %d.", gun->maxshots );
			msg_print(string2); */
			return 1;
		}
	}
}
	
		

void fire_gun(inven_type *gun, int dir)
{
	char string[256];
	int damtype=GF_MAGIC_MISSILE;

	char *shottype="shot";
	int y=char_row,x=char_col;
	register struct misc* p_ptr=&py.misc;
	int tot_tohit, base_tohit;

	tot_tohit = p_ptr->ptohit;
	base_tohit = p_ptr->bth;

	if (dir == 5 || (dir<1 && dir > 9))
		{ free_turn_flag=TRUE; return; }
	if (gun->shots==0)
		{ free_turn_flag=FALSE;
		msg_print("You are out of ammunition."); return; }
	gun->shots--;
	free_turn_flag=FALSE;
	switch (gun->subval) {
	case 4: case 5:
		shottype = "needle burst";
		break;
	case 8:
		shottype = "grenade blast";
		break;
	case 9: case 10:
		shottype = "laser";
		break;
	case 11: 
		shottype = "flame";
		damtype = GF_FIRE;
		break;
	case 15:
		shottype = "rocket's explosion";
		break;
	}
	if ((gun->flags | gun->flags2) & FL_BALL)
		fire_ball(damtype, dir, char_row, char_col, 
			damroll(gun->damage[0], gun->damage[1])+gun->todam, shottype);
	else
DBG("Made it to locate_monster.")
	if (!locate_monster(&x, &y, dir)  || test_hit(base_tohit, (int)p_ptr->lev,
			tot_tohit, (int)c_list[cave[y][x ].cptr].ac, CLA_BTH))
	fire_bolt(damtype, dir, char_row, char_col, damroll(gun->damage[0],
			gun->damage[1])+gun->todam, shottype);
	else shoot_missed(dir, char_row, char_col);
DBG("Made it out.")
	sprintf(string, "You have %d shots left.", gun->shots);
	msg_print(string);
}

void save_dir(int dir)
{
	direction = dir;
}

int recover_dir(void)
{
	return direction;
}

int locate_monster(int *x, int* y, int dir)
		/* locates the first monster in the given directon, and returns the
			coordinates in x and y.  Returns 1 as the function value if
			it found the monster, 0 if there was no monster in range, or
			it hit a wall first. */
{
	int dist=0;
	register cave_type *c_ptr;
	int flag = FALSE;

	do {
		mmove(dir, y, x);
		dist++;
		c_ptr = &cave[*y][*x];
		if ((dist>OBJ_BOLT_RANGE) || c_ptr->fval >= MIN_CLOSED_SPACE)
			flag = TRUE;
		else if (c_ptr->cptr > 1)
			return 1;
	} while (!flag);
	return 0;
}

void shoot_missed(int dir, int y, int x)
/* Shows the animation of a bolt firing, but does no damage or anything else. */
{
	int dist, i;
	register cave_type *c_ptr;
	int flag = FALSE;
	int oldx, oldy;
	monster_type *m_ptr;
	creature_type *r_ptr;
	vtype out_val, m_name;

DBG("Missed.")
	dist=0;
	do {
		(void) mmove(dir, &y, &x);
		dist++;
		lite_spot(oldy, oldx);
		c_ptr = &cave[y][x];
		if ((dist>OBJ_BOLT_RANGE) || c_ptr->fval >=MIN_CLOSED_SPACE)
			flag=TRUE;
		else {
			if (c_ptr->cptr > 1)
			{
				flag=TRUE;
				m_ptr=&m_list[c_ptr->cptr];
				r_ptr=&c_list[m_ptr->mptr];				
				i=c_ptr->pl;
				c_ptr->pl=TRUE;
				update_mon((int) c_ptr->cptr);
				c_ptr->pl=i;
				put_qio();
				lower_monster_name(m_name, m_ptr, r_ptr);
				(void) sprintf(out_val, "Your shot misses %s.", m_name);
				msg_print(out_val);
			}
		else if (panel_contains(y,x) && (py.flags.blind <1))
		{	print(bolt_shape(dir), y, x);
			put_qio();
		}
	}
	oldy=y;
	oldx=x;
	} while (!flag);
}

inven_type unload_gun(inven_type wielded_gun)
{
	inven_type unloaded_ammo;
	int i;

	unloaded_ammo.name2 = SN_NULL;
	*unloaded_ammo.inscrip = '\0';
	unloaded_ammo.flags = 0U;
	unloaded_ammo.number = wielded_gun.shots;
	unloaded_ammo.shots = 0;
	unloaded_ammo.maxshots = 0;
	unloaded_ammo.damage[1] = 0;
	unloaded_ammo.damage[2] = 0;

	unloaded_ammo.flags2 = wielded_gun.flags2;

	for (i=0; i<MAX_DUNGEON_OBJ; i++)
		if (ammo_table[wielded_gun.subval] == object_list[i].subval
			&& object_list[i].tval == TV_AMMO)
			break;
	if (i == MAX_DUNGEON_OBJ)
	{	msg_print("Error in unload_gun().  Tell the wizard.");
		return unloaded_ammo;
	}
	unloaded_ammo.index = i;
	unloaded_ammo.tval = TV_AMMO;
	unloaded_ammo.tchar = object_list[i].tchar;
	unloaded_ammo.p1 = object_list[i].p1;
	unloaded_ammo.cost = object_list[i].cost;
	unloaded_ammo.subval = object_list[i].subval;
	unloaded_ammo.weight = object_list[i].weight;
	unloaded_ammo.tohit = unloaded_ammo.todam = 0;
	unloaded_ammo.ac = unloaded_ammo.toac = 0;
	unloaded_ammo.level = object_list[i].level;
	unloaded_ammo.ident = 0;

	return unloaded_ammo;
}

	
