/* monsters.c: monster definitions

   Copyright (c) 1989 James E. Wilson, Robert A. Koeneke

   This software may be copied and distributed for educational, research, and
   not for profit purposes provided that this copyright and statement are
   included in all such copies. */

#include "monster.h"
#include "types.h"

/*
		Attack types:
		1	Normal attack
		2	Poison Strength
		3	Confusion attack
		4	Fear attack
		5	Fire attack
		6	Acid attack
		7	Cold attack
		8	Lightning attack
		9	Corrosion attack
		10	Blindness attack
		11	Paralysis attack
		12	Steal Money
		13	Steal Object
		14	Poison
		15	Lose dexterity
		16	Lose constitution
		17	Lose intelligence
		18	Lose wisdom
		19	Lose experience
		20	Aggravation
		21	Disenchant
		22	Eats food
		23	Eat light
		24	Energy drain from pack
		25      Drain all stats
		99	Blank

		Attack descriptions:
		1	hits you.
		2	bites you.
		3	claws you.
		4	punches you.
		5	shoots you.
		6	kicks you.
		7	tramples you.
		8	gouges you.
		9	runs over you.
		10	stabs
		11	sells souvenirs   
		12	slash 
		13	hack
		14	smash against chestplate
		15	annoy
		16	strikes
		17	whips
		18	butts
		19	slices you

		20	clubs you.
		21	charges you.
		22	engulfs you.
                23      talks to you about mushrooms and dogs

		99	is repelled.

	Example:  For a creature which bites for 1d6, then stings for
		  2d4 and loss of dex you would use:
			{1,2,1,6},{15,4,2,4}

	Sleep (sleep)	:	A measure in turns of how fast creature
				will notice player (on the average).
	Area of affect (aaf) :	Max range that creature is able to "notice"
				the player.
									*/

creature_type c_list[MAX_CREATURES] = {

{"Vagarant",	(MV_ATT_NORM|MV_20|THRO_DR|PICK_UP), (NONE8), (NONE8),
	0, 10, 5, 1, 10,'c'|BLACK, {2,5}, {1,0,0,0,0,0}, {0,0,0,0}, 0, 0, AC_BLACK	},
{"Street Cat",	(MV_ATT_NORM|MV_20), (NONE8), (ANIMAL),
	0, 1, 20, 3, 10,'f'|YELLOW,  {2,2}, {3,2,2,0,0,0} ,{0,0,0,0},  0, 1, AC_YELLOW},
{"Gang Member", (MV_ATT_NORM|MV_20), (PISTOL),(GROUP),
	0, 12, 10, 2, 10, 'c'|RED, {2,6}, {4,5,0,0,0,0},{0,0,0,0},  0, 0, AC_RED	},
{"Mongrel",		(MV_ATT_NORM|MV_20), (NONE8), (ANIMAL),
	0, 3, 15, 2, 10, 'd'|YELLOW, {3,2}, {6, 7,7,0,0,0},{0,0,0,0},  0, 1, AC_YELLOW},
{"Corner Merchant",	(MV_ATT_NORM|MV_20), (NONE8), (NONE8),
	0, 100, 1, 1, 10, 'c'|GREEN, {2,5}, {8,0,0,0,0,0},{0,0,0,0},  0, 0, AC_GREEN},
{"Drunken Spacer",	(MV_ATT_NORM|MV_20|CARRY_GOLD|HAS_90|MV_40|PICK_UP),
	(NONE8), (NONE8), 0, 100, 5, 1, 10, 'c'|GREEN, {2,6}, {1,0,0,0,0,0},
	{0,0,0,0},  0,0, AC_GREEN},
{"Police officer",	(CARRY_OBJ|HAS_90|MV_ATT_NORM|THRO_DR|PICK_UP),
	(NONE8), (NONE8), 0, 1, 10, 5, 10, 'c'|BLUE,	 {2,6}, {4,0,0,0,0,0},
	{0,0,0,0},  0,1, AC_BLUE},
{"Caddis fly", (MV_ATT_NORM|MV_75), (NONE8), (ANIMAL), 
	0, 5, 5, 0, 12, 'c'|WHITE, {1,1}, {36,0,0,0,0,0}, {0,0,0,0}, 0,0, AC_WHITE},
{"New Enlist",		(CARRY_OBJ|HAS_90|MV_ATT_NORM),	(PISTOL), (NONE8),
	10, 5, 3, 5, 10, 's'|GREEN, {2,6}, {4,0,0,0,0,0},{0,0,0,0},  1, 0, AC_GREEN},
{"Catworm", (MV_ATT_NORM|MULTIPLY), (NONE8), (ANIMAL), 8, 20, 20, 
	10, 10, 'c'|RED, {2,2}, {37,0,0,0,0,0}, {0,0,0,0}, 2, 0, AC_RED},
{"Private",		(MV_ATT_NORM,CARRY_OBJ|HAS_90),	(RIFLE),	(NONE8),
	30, 1, 15, 10, 10,'s'|GREEN, {2,7}, {9,4,0,0,0,0},{0,0,0,0},  2, 0, AC_GREEN	},
{"Wild dog", (MV_ATT_NORM|MV_20), (NONE8), (ANIMAL|GROUP),
	8, 5, 5, 10, 11, 'd'|YELLOW, {2,2}, {37,0,0,0,0,0}, {0,0,0,0}, 2, 0, AC_YELLOW},
{"Green Belt",	(MV_ATT_NORM|CARRY_OBJ|HAS_60) , (NONE8), (NONE8),
	20, 4, 10, 7, 10,'n'|GREEN, {2,6}, {12,12,0,0,0,0},{0,0,0,0},  2, 0, AC_GREEN},
{"Bloodhound",		(MV_ATT_NORM), (NONE8), (ANIMAL),
	18, 1, 5, 3, 10	,'d'|YELLOW, {4,2}, {6,6,0,0,0,0},{0,0,0,0},  3, 0, AC_YELLOW	},
{"Watch dog", (MV_ATT_NORM), (NONE8), (ANIMAL), 
	80, 2, 5, 20, 11, 'd'|WHITE, {3,2}, {6,0,0,0,0,0}, {0,0,0,0}, 4, 0, AC_WHITE},
{"White Belt",	(MV_ATT_NORM|CARRY_OBJ|HAS_90),	(NONE8), (NONE8),
	80, 2, 15, 8, 10,'n'|WHITE,{2,7}, {12,12,13,0,0,0},{0,0,0,0},  4, 0	, AC_WHITE},
{"Viper", (MV_ATT_NORM|MV_20), (NONE8), (ANIMAL), 80, 20, 10, 
	20, 11, 'c'|MAGENTA, {4,4}, {6, 0,0,0,0,0}, {6,0,0,0}, 4, 0, AC_MAGENTA},
{"Yellow-uniformed Flamethrower", (MV_ATT_NORM), (FIRE_BOLT|FIRE_BALL),
	(FREQ_6|IM_FIRE), 80, 30, 4, 20, 11, 'f'|YELLOW, {4,4}, {51,0,0,0,0,0},
	 {8,0,0,0}, 4, 0, AC_YELLOW},
{"Sergeant",	(MV_ATT_NORM,CARRY_OBJ|HAS_1D2), (SHOT),	(NONE8),
	70, 2, 30, 20, 10, 's'|BLACK,	{2,8}, {10,0,0,0,0,0},{0,0,0,0},  5, 0	, AC_BLACK},
{"Bobcat",			(MV_ATT_NORM),	(NONE8),	(ANIMAL),
	75, 6, 15, 3,  10, 'f'|YELLOW,{3,3}, {6,7,7,0,0,0},{0,0,0,0},  6, 0, AC_YELLOW},
{"Orange-uniformed Flamethrower", (MV_ATT_NORM), (FIRE_BOLT|FIRE_BALL),
	(FREQ_3|IM_FIRE), 288, 15, 10, 30, 11, 'f'|MAGENTA, {5,4}, {51,51,0,0,0,0},
	{8,8,0,0}, 6, 0, AC_MAGENTA},
{"Guard dog", (MV_ATT_NORM), (NONE8), (ANIMAL),
	288, 1, 10, 30, 11, 'd'|BLACK, {4,2}, {6,6,0,0,0,0}, {0,0,0,0}, 6, 0, AC_BLACK},
{"Doberman Pincer",	(MV_ATT_NORM), (NONE8), (ANIMAL),
	50, 2, 10, 4, 10, 'd'|BLACK, {4,3}, {17,17,17,2,0,0},{0,0,0,0},  7, 0, AC_BLACK	},
{"Stinger gunnery crew", (MV_ONLY_ATT), (NONE8), (NONE8),
	441, 0, 40, 35, 11, 'g'|WHITE, {7,7}, {0,0,0,0,0,0}, {10,11,0,0}, 7, 0, AC_WHITE},
{"Red Belt",	(MV_ATT_NORM|CARRY_OBJ|HAS_1D2),	(NONE8), (NONE8),
	150, 1, 30, 9, 10,'n'|RED,{4,4}, {12,12,13,13,0,0},{0,0,0,0},  8, 0,AC_RED	},
{"Mutant Lizard",	(MV_ATT_NORM), (NONE8), (ANIMAL),
	300, 10, 5, 3, 9,'m'|RED,{2,5}, {27,27,0,0,0,0}, {0,0,0,0}, 8, 1, AC_RED},
{"Giant Crab", (MV_ATT_NORM|MV_20), (NONE8), (ANIMAL), 704, 
	20, 15, 40, 11, 'c'|RED, {8,8}, {19,19,0,0,0,0}, {0,0,0,0}, 8, 0, AC_RED},
{"Red-uniformed Flamethrower", (MV_ATT_NORM), (FIRE_BOLT|FIRE_BALL),
	(FREQ_2|IM_FIRE), 704, 5, 25, 40, 11, 'f'|RED, {6,4}, {0,0,0,0,0,0},
	{0,0,0,0}, 8, 0, AC_RED},
{"Browning Machine Gunners", (MV_ONLY_ATT), (NONE8), (NONE8),
	972, 0, 40, 45 ,11, 'g'|BLACK,  {8,7}, {0,0,0,0,0,0}, {10,10,11,11}, 9, 0, AC_BLACK},
{"Officer in Training",	(MV_ATT_NORM,CARRY_GOLD|HAS_2D2), (SUBMACH), (NONE8),
 	150	, 1, 50, 25, 10,'s'|MAGENTA, {4,4}, {11,11,0,0,0,0},{0,0,0,0},  10, 0,
	 AC_MAGENTA},
{"Mountain Lion",	(MV_ATT_NORM), (NONE8), (ANIMAL),
    100, 4, 30, 5,  10, 'f'|YELLOW,	{5,3}, {17,7,7,0,0,0},{0,0,0,0},  10, 0, AC_YELLOW},
{"Basic Cyborg", (MV_ATT_NORM|CARRY_GOLD|HAS_90), (PISTOL|CYBER), (NONE8),
	700, 5, 20, 8, 10,'r'|WHITE, {5,7}, {9,9,26,26,0,0}, {0,0,0,0}, 10, 0, AC_WHITE},
{ "Alpha android", (MV_ATT_NORM), (NONE8), (NONE8),  /* 43 */
	1400, 10, 2, 50, 11, 'a'|YELLOW, {10,10}, {31,0,0,0,0,0}, {1,0,0,0}, 10, 0, AC_YELLOW},
{"Cameleopard", (MV_ATT_NORM|MV_20), (NONE8), (ANIMAL), 1400, /* 55 */
	10, 20, 50, 11, 'c'|YELLOW, {10,10}, {41,41,41,0,0,0}, {0,0,0,0}, 10, 0, AC_YELLOW},
{"Attack dog", (MV_ATT_NORM), (NONE8), (ANIMAL|GROUP), 	/* 60 */
	1400, 2, 15, 50, 11, 'd'|RED, {6,2}, {18,38,38,0,0,0}, {0,0,0,0}, 10, 0, AC_RED},
{"Rottweiler", (MV_ATT_NORM), (NONE8), (ANIMAL),
	11200u, 0, 15, 50, 11, 'd'|MAGENTA, {6,4}, {39,39,42,42,0,0}, {0,0,0,0},
	 10, 2, AC_MAGENTA},
{"Bounty Hunter",	(MV_ATT_NORM|CARRY_GOLD|HAS_90), (SHOT|SUBMACH),
	(NONE8),	250, 1, 50, 15, 10, 'c'|BLACK, {4,4}, {10,11,0,0,0,0},{0,0,0,0},  11,0, AC_BLACK },
{"Mutant Alligator", (MV_ATT_NORM), (NONE8), (ANIMAL),
	600, 5, 7, 8, 10, 'm'|GREEN, {4,6}, {17,17,0,0,0,0}, {0,0,0,0}, 11, 0, AC_GREEN},
{"Mortar crew", (MV_ONLY_ATT), (NONE8), (NONE8), 	/* 73 */
	1815, 0, 80, 55, 11, 'g'|GREEN, {9,7}, {0,0,0,0,0,0}, {12,0,0,0}, 11, 0, AC_GREEN},
{ "Beta android", (MV_ATT_NORM), (NONE8), (NONE8),
	2304, 10, 6, 60, 11, 'a'|GREEN, {11,10}, {31,31,0,0,0,0}, {1,1,0,0}, 12, 0, AC_GREEN},
{"Cavalry",		(MV_ATT_NORM|CARRY_GOLD|HAS_1D2), (RIFLE), (NONE8),
	240, 3, 40, 11, 11, 's'|YELLOW, {10,4}, {9,16,0,0,0,0},{0,0,0,0},  13,0, AC_YELLOW},
{"Trained Wolf",	(MV_ATT_NORM),	(NONE8), (ANIMAL),
    110, 1, 35, 7,  10, 'd'|WHITE,	{7,3}, {17,17,17,7,0,0},{0,0,0,0},  13, 0, AC_WHITE},
{"Mutated Earthling", (MV_ATT_NORM), (PISTOL), (NONE8),
	1200, 3, 14, 9, 10,'m'|WHITE, {4,5}, {26, 12,1,1,0,0},{0,0,0,0},  13, 1, AC_WHITE},
{ "Gamma android", (MV_ATT_NORM), (NONE8), (NONE8),	/* 45 */
	3724, 15, 10, 70, 12, 'a'|RED, {12,10}, {31,0,0,0,0,0}, {1,0,0,0}, 14, 0, AC_RED},
{"Tailed Earth-bore", (MV_ATT_NORM), (NONE8), (BREAK_WALL), 
	3136, 20, 5, 70, 11, 'e'|YELLOW, {14,14}, {43,44,44,0,0,0}, {0,0,0,0}, 14, 0, AC_YELLOW},
{"Torquemada", (MV_ATT_NORM), (FIRE_BOLT|FIRE_BALL), (FREQ_3|IM_FIRE|UNIQUE),
	1216, 1, 45, 70, 12, 'F'|RED|A_BLINK, {10,8}, {52,52,52,52,0,0}, {9,0,0,0},
	 14, 0, AC_RED}, /* 70 */
{"Black Belt",	(MV_ATT_NORM|CARRY_OBJ|HAS_2D2),	(NONE8), (NONE8),
	220, 1, 50, 9, 11, 'n'|BLACK, {5,4}, {14,14,15,15,0,0},{0,0,0,0},  15,0, AC_BLACK	},
{"Leopard",			(MV_ATT_NORM),	(NONE8), (ANIMAL),
	200, 5, 35, 6,  10, 'f'|YELLOW,	{5,5}, {18,18,0,0,0,0},{0,0,0,0},  15,0, AC_YELLOW},
{"The Shaman", (MV_ATT_NORM), (SUMMON|EARTHQUAKE|FIREBALL|SLOW|CS_STUN),
	(UNIQUE|FREQ_5), 21000, 0, 20, 75, 11, 'C'|GREEN|A_BLINK, {20,20}, 
	{0,0,0,0,0,0}, {0,0,0,0}, 15, 0, AC_GREEN},
{"Jaguar",			(MV_ATT_NORM),	(NONE8),	(ANIMAL),
	250, 4, 45, 8, 11, 'f'|YELLOW, {6,5}, {18,19,19,0,0,0},{0,0,0,0},  16,0, AC_YELLOW},
{"Horned Earth-bore", (MV_ATT_NORM), (NONE8), (BREAK_WALL),
	5632, 30, 30, 80, 11, 'e'|BLACK, {15,14}, {43,43,47,47,47,47}, {0,0,0,0}, 16, 0, AC_BLACK},
{"Clawed Earth-bore", (MV_ATT_NORM), (NONE8), (BREAK_WALL),
	5632, 15, 30, 80, 11, 'e'|BLACK, {16,14}, {45,45,46,46,46,0}, {0,0,0,0}, 16, 0, AC_BLACK},
{"Fanged Earth-bore", (MV_ATT_NORM), (NONE8), (BREAK_WALL|IM_POISON), /* 65 */
	5632, 20, 25, 80, 11, 'e'|GREEN, {17,14}, {43,43,48,48,48,0}, {0,0,0,0}, 16, 0, AC_GREEN},
{"The Excavator", (MV_ATT_NORM), (NONE8), (BREAK_WALL|UNIQUE|IM_ACID),
	51612u, 5, 35, 85, 11, 'E'|BLUE|A_BLINK, {31,28}, {49,50,0,0,0,0}, {7,0,0,0}, 17, 0, AC_BLUE},
{"Tiger",			(MV_ATT_NORM),	(NONE8), (ANIMAL),
	300, 3, 40, 9,  11, 'f'|YELLOW,	{7,6}, {18,18,19,19,0,0},{0,0,0,0},  18,0, AC_YELLOW},
{"Delta android", (MV_ATT_NORM), (NONE8), (NONE8),
	8100, 10, 20, 80, 11, 'a'|WHITE, {13,10}, {31,31,31,0,0}, {1,1,1,0}, 18, 0, AC_WHITE},
{"Sniper",		(MV_ONLY_ATT), (SNIPER), (ALWAYS_CAST),
	1000, 1, 100, 15, 10, 's'|BLACK, {4,5}, {28,28,28,0,0,0},{0,0,0,0},  19, 0, AC_BLACK},
{"Charging Elephant", (MV_ATT_NORM), (NONE8), (ANIMAL),
	600, 7, 3, 1, 10,'e'|WHITE, {30,2}, {20,20,21,21,0,0},{0,0,0,0},  20, 1, AC_WHITE	},
{"Arnold Marc XII", (MV_ATT_NORM), (TELE), (UNIQUE|MAX_HP),
	28000, 5, 25, 100, 12, 'A'|YELLOW, {23,20}, {32,32,0,0,0,0}, {2,2,0,0}, 20, 0, AC_YELLOW},
{"Rhinoceros",		(MV_ATT_NORM),	(NONE8), (ANIMAL),
	610, 5, 10, 3, 10,'e'|WHITE, {20,3}, {20, 22,22,0,0,0},{0,0,0,0},  21, 1	, AC_WHITE},
{"Assassin",	(MV_ATT_NORM|CARRY_GOLD|HAS_2D2), (SNIPER|TELE), (ALWAYS_CAST), 
	1500, 1, 100, 20, 11,'c'|BLACK, {5,6}, {28,28,28,29,29,0}, {0,0,0,0}, 21, 0, AC_BLACK},
{"Proximity Turret",	(MV_ONLY_ATT), (SUBMACH), (ALWAYS_CAST),
		900, 1, 40, 20, 10, 'r'|MAGENTA, {20,2}, {11,11,11,0,0,0}, {0,0,0,0},
		 22,1, AC_MAGENTA},
{"Attack Droid", (MV_ATT_NORM),	(RIFLE), (NONE8),
		950, 3, 20, 20, 10,'r'|RED,	{20,3}, {9,9,26,26,0,0}, {0,0,0,0}, 24,
		 1, AC_RED},
{"Standard bioconstruct", (MV_ATT_NORM), (TELE_TO|HEAL|FIREBALL), 
	(FREQ_10), 19008, 15, 10, 120, 12, 'b'|YELLOW , {24,24}, 
	{33,33,0,0,0,0}, {3,4,4,0}, 24, 0, AC_YELLOW},
{"The Ninja Master", (MV_ATT_NORM|CARRY_OBJ|HAS_4D2|HAS_2D2), (NONE8), (UNIQUE),
	1900, 1, 45, 30, 12,'N'|BLACK|A_BLINK, {8,5}, {26,26,26,15,15,15},{0,0,0,0},  24, 0, AC_BLACK},
{"Armored Jeep",		(MV_ATT_NORM|CARRY_OBJ|HAS_1D2),
	(PISTOL|RIFLE),	(ARMORED),	1000, 4, 40, 25, 12,'a'|GREEN, 
		{29,3}, {9,4, 25,25,0,0}, {0,0,0,0}, 25, 0, AC_GREEN},
{ "Armored bioconstruct", (MV_ATT_NORM), (TELE_TO|HEAL|FIREBALL|ST_CLD),
	(FREQ_10|ARMORED), 40000, 15, 10, 130, 13, 'b'|BLACK, {25,24}, 
	{32,32,32,0,0,0}, {3,3,4,0}, 26, 0, AC_BLACK},
{"Advanced Cyborg", (MV_ATT_NORM|CARRY_GOLD|HAS_1D2), (SUBMACH|ADVCYBER), 
	(GOOD), 2000, 1, 20, 11, 11,'r'|BLUE, {10,8}, {11,11,11,26,26,0},
	{0,0,0,0},  26, 0, AC_BLUE},
{"Seek-and-Destroy Robot",(MV_ATT_NORM), (SUBMACH|TELE), (NONE8),
	1300, 1, 35, 30, 10,'r'|BLACK, {25,6}, {10,11,26,26,0,0}, {0,0,0,0},
	 27, 1, AC_BLACK},
{"Armored Personnel Carrier",	(MV_ATT_NORM|CARRY_OBJ|HAS_4D2), 
	(PISTOL|RIFLE|SUBMACH), (ARMORED), 1500, 3, 40, 50, 12,'a'|GREEN,	
		{50,2}, {10,11,9,23,23,0} ,{0,0,0,0},  30, 0, AC_GREEN},
{"Terminator",		(MV_ATT_NORM), (SUBMACH|SHOT),	(NONE8),
	2400, 1, 45, 50, 11,'r'|YELLOW, {45,3}, {10,10,10,11,26,0}, {0,0,0,0}, 30, 0, AC_YELLOW},
{"Biotech One", (MV_ATT_NORM|WINNER), (TELE_TO|HEAL|SUMMON|TELE_LEV),
	(FREQ_10|UNIQUE), 580608u, 0, 80, 155, 13, 'B'|YELLOW|A_BLINK, {49,48},
	{34,34,35,0,0,0}, {5,0,0,0}, 31, 0, AC_YELLOW},	/* 50 */
{"Tank", (MV_ATT_NORM|CARRY_OBJ|HAS_4D2), (RIFLE|SUBMACH|ROCKET), 
		(ARMORED|GOOD),
		2500, 1, 45, 75, 11,'a'|BLACK, {60,3}, {11,11,10,9,24,24},{0,0,0,0},
	  33, 0, AC_BLACK},
{"General Qonreess", (MV_ATT_NORM|CARRY_OBJ|HAS_4D2|HAS_2D2|HAS_1D2|WINNER), 
	(PISTOL|SHOT|RIFLE|SUBMACH|SNIPER|ROCKET|ADVCYBER|TELE), (UNIQUE),
	4000, 1, 55, 80, 12,'S'|MAGENTA|A_BLINK, {60,5}, {28,28,28,30,30,30},{0,0,0,0},  40, 0, AC_MAGENTA},

/*
{"Grenade-launcher crew", (MV_ONLY_ATT), (NONE8), (NONE8),
	},
*/
	

{"Player Ghost", (MV_ATT_NORM),(NONE8), (NONE8), 1, 1, 1, 1, 1, 
	'@', {1,1}, {1,1,1,1,1,1},{0,0,0,0},  1,1}
};

struct m_attack_type monster_attacks[N_MONS_ATTS] = {
/* 0 */	{0, 0, 0, 0},	{1,4,1,1}, {1,3,1,1}, {1,2,1,2}, {1,5,1,4},
/* 5 */ {1,1,1,2},	{1,2,2,2}, {1,3,2,1}, {1,11,0,0}, {1,5,2,4},
/*10 */ {1,5,4,4}, {1,5,3,4}, {1,4,2,2}, {1,6,3,2}, {1,4,3,2},
/*15 */ {1,6,4,2}, {1,17,3,4}, {1,2,3,2}, {1,2,4,4}, {1,3,3,3},
/*20 */ {1,8,4,4}, {1,17, 8,4}, {1,17,7,4}, {1,9,5,4}, {1,9,6,4},
/*25 */ {1,9,4,4}, {1,1,2,4}, {2,2,2,1}, {1,5,5,5}, {14,10,2,4},
/*30 */ {25,1,4,3}, {1,12,3,3}, {1,12,6,6}, {1,13,5,5}, {1,1,10,10},
/*35 */ {1,14,24,24}, {1,15,0,0}, {1,2,1,1}, {1,3,2,2}, {1,2,8,8},
/*40 */ {1,1,3,3}, {1,6,3,3}, {1,3,4,4}, {1,16,4,4}, {1,17,7,7},
/*45 */ {3,16,4,4}, {1,3,7,7}, {1,18,7,7}, {14,2,7,7}, {1,16,8,8},
/*50 */ {1,19,14,14}, {1,20,2,2}, {1,19,4,4}
};


monster_type m_list[MAX_MALLOC];
int16 m_level[MAX_MONS_LEVEL+1];

/* Blank monster values	*/
monster_type blank_monster = {0,0,0,0,0,0,0,FALSE,0,FALSE};
int16 mfptr;			/* Cur free monster ptr	*/
int16 mon_tot_mult;		/* # of repro's of creature	*/

/*	Ranged attack types:
		0: nothing
		1:	shoots
		2:  spits
		3: fires mortar
*/

struct m_attack_type ranged_attacks[N_RANGED_ATTACKS] = {
/* 0 */	{0,0,0,0},	{1,1,5,5},	{1,1,10,10}, {1,1,12,12}, {1,1,6,6},
/* 5 */	{1,1,24,24}, {1,2,2,2}, {6,2,14,14}, {1,1,2,2}, {1,1,4,4},
/*10 */ {1,1,3,4}, {1,1,4,3}, {1,3,11,11}
};
