/* player.c: player specific variable definitions

   Copyright (c) 1989 James E. Wilson, Robert A. Koeneke

   This software may be copied and distributed for educational, research, and
   not for profit purposes provided that this copyright and statement are
   included in all such copies. */

#include "constant.h"
#include "config.h"
#include "types.h"

/* Player record for most player related info */
player_type py;
/* player location in dungeon */
int16 char_row;
int16 char_col;
/* calculated base hp values for player at each level, store them so that
   drain life + restore life does not affect hit points */
int16u player_hp[MAX_PLAYER_LEVEL];

/* Class titles for different levels				*/
#ifdef MACGAME
char *(*player_title)[MAX_PLAYER_LEVEL];
#else
char *player_title[MAX_CLASS][MAX_PLAYER_LEVEL] = {
	/* Mercernary	 */
{"Recruit","Trainee","Private","PFC","Mercenary","Sergeant","SFC",
"Tech. Sergeant", "Veteran","Warrior","Ensign","Lieutenant JG",
"Lieutenant SG","Corporal","Lance Corporal","Hero","Superhero","Champion",
"Lt. Commander","Commander","Captain","High Captain","Commodore",
"General","Admiral", "Rear Admiral"
,"Fleet Admiral","Chief Admiral","Chief of Staff","Commander in Chief"},
	/* Cyborg		 */
{"Experiment","New Implant","Symbyote","Jr. Technician","Technician",
"Sr. Technician",
"Chief Technician","Jr. Engineer", "Engineer", "Project Engineer",
"Sr. Engineer", "Chief Engineer","Stockholder","Major Stockholder",
"Jr. Manager","Manager","Sr. Manager","Company Manager","Jr. Executive",
"Executive",
"Sr. Executive","Officer","Company Officer", "Corporate Officer"
,"Executive Officer",
"Jr. Board Member","Board Member","Sr. Board Member","Chairman",
"CEO"},
	/* Robot	 */
{"Automaton","Android","Android Adept","Android Master","Android Leader",
"Android Chief","Android Officer","Mechanical Man","Vice Robot","Robot"
,"Robot Chief","Robot Champion",
"Master Robot","Robot Officer","Robot at Large",
"Jr. Drone","Drone","Sr. Drone","Drone Master","Specialist Drone",
"Champion Drone","Drone Chief","Seek-and-Destroy","S&D Master","S&D Champion",
"S&D Chief","S&D Leader","Terminator","T-1000","Ultimate Killing Machine"
},
	/* Sharpshooter	 */
{"Gun Owner","Student","Gunslinger","Quick Draw","Bad Shot","Poor Shot"
,"Fair Shot",
"Good Shot","Excellent Shot","Dead Shot","Low Marksman","Skilled Marksman",
"Marksman","Hign Marksman","Marksman Extraordinairre","Divine Marksman",
"Low Sharpshooter","Sharpshooter",
"High Sharpshooter","Sharpshooter Master","Sr. Sharpshooter"
,"Head Sharpshooter","Sharpshooter Chief",
"Jr. Lobbyist","Lobbyist","Sr. Lobbyist","Low Assassin",
"Assassin","High Assassin","Head Assassin"},
	/* Ninja	 */
{"Trainer","Martial Artist","Fighter","Surviver","Wrestler",
"Green Belt","White Belt","Red Belt",
"Black Belt","Fight Master","Master of the Arts","Master at Arms","Samauri",
"Jr. Fencer","Fencer","Sr. Fencer","Master Fencer","Lord Fencer",
"Low Ninja","Jr. Ninja","Full Ninja",
"Sr. Ninja","High Ninja","Ninja Noble","Ninja Chief","Ninja Champion",
"Master Ninja","Ninja Leader","Head Ninja","Ninja Lord"}
};
#endif

/* Base experience levels, may be adjusted up for race and/or class*/
int32u player_exp[MAX_PLAYER_LEVEL] = {
      10,      25,	45,	 70,	  100,	    140,      200,	280,
     380,     500,     650,	850,	 1100,	   1400,     1800,     2300,
    2900,    3600,    4400,    5400,	 6800,	   8400,    10200,    12500,
   17500,   25000,  35000L,  50000L,   75000L,	100000L 
};

/*Race	STR,DEX,APT,MCP,CON,CHR,
	Ages, heights, and weights (male then female)
	Racial Bases for: dis,srh,stl,fos,bth,bthb,bsav,hitdie,
	infra, exp base, choice-classes */
#ifdef MACGAME
race_type *race;
#else
race_type race[MAX_RACES] = {
   {"Earthling",	 0,  0,	 0,  0,	 0,  0,
      14,  6, 72,  6,180, 25, 66,  4,150, 20,
      0,  0,  0,  0,  0,  0,  0, 10,  0, 100, 0x1b,
    },
   {"Robot", 3,  -3,	 0,  5, 3,  -2,
      24, 16, 66,  6,130, 15, 62,  6,100, 10,
      2,  6,  1, -1, -1,  5,  3,  9,  2, 130, 4,
    },
   {"Thorian",	+5,  2,	 -2, -1, 2,  1,
      75, 75, 60,  4,100,  6, 54,  4, 80,  6,
      5,  8,  1, -2, -5, 15,  6,  8,  3, 135, 0x13,
    },
   {"A'durran", -2,  5,	 2,  0,	 -1,  3,
      21, 12, 36,  3, 60,  3, 33,  3, 50,  3,
      15, 12,  4, -5,-10, 20, 18,  7,  4, 135, 0x0B,
    },
   {"Cerebrus",	-3,  1,	 5,  3,	 -2, -1,
      50, 40, 42,  3, 90,  6, 39,  3, 75,  3,
      10,  6,  3, -3, -8, 12, 12,  8,  4, 115, 0x0b,
    },
   {"Ironman",	 0, -2,	 0, 1,	 5, 2,
      35, 15, 48,  3,150, 10, 46,  3,120, 10,
      2,  7,  -1,  0, 15,  0,  9,  11,  5, 135, 0x13,
    },
   {"Angel",	 3, 3,	 2,  0,	 2, 5,
      11,  4, 66,  1,150,  5, 62,  1,120,  5,
      -3,  0, -1,  3, 12, -5, -3, 10,  3, 175, 0x18,
    },
   {"Rigellian",4, -3, -2, -1,	 4, -2,
      20, 10, 96, 10,255, 50, 84,  8,225, 40,
      -5, -1, -2,  5, 20,-10, -8, 12,  3, 90, 0x19,
    },
   {"Erandian",  1,  2,  4,  1,  1,  2,
      50, 20, 82, 5, 190, 20, 78,  6, 180, 15,
      4,   3,  2, -3, 15, 10,  5, 10,  0, 130, 0x1b,
    },
   {"Centaurian",  -2,  2, 4,  3,  -1,  3,
     100, 30, 90,10, 190, 20, 82, 10, 180, 15,
      4,   3,  3, -4, 15, 25, 20, 10,  4, 120, 0x1a,
    },
	{"Andorian",   -3, 4, 1, 2, -2, 4,
	30, 20, 35, 3, 70, 4, 35, 3, 60, 3,
	5, 6, 2, -2, -8, 15, 13, 7, 4, 120, 0xa,
	}
 };
#endif

/* 5 char race for printing scores. */
char *dsp_race[MAX_RACES] = {
  "Earth",
  "Robot",
  "Thor ",
  "A'drr",
  "Cereb",
  "Iron ",
  "Angel",
  "Rigel",
  "Eran ",
	"Cent ",
  "Andor"
};

/* Background information					*/
#ifdef MACGAME
background_type *background;
#else
background_type background[MAX_BACKGROUND] = {
{"You were given to an orphanage by ",		 10, 1, 2, 25},
{"You are the adopted child of ",		 20, 1, 2, 35},
{"You are one of several children of ",				 95, 1, 2, 45},
{"You are the only child of ",					100, 1, 2, 50},
{"a manual laborer.  ",						 40, 2, 3, 65},
{"a technician.  ",						 65, 2, 3, 80},
{"a regional manager.  ",						 80, 2, 3, 90},
{"a successful entrepreneur.  ",						 90, 2, 3,105},
{"a military officer.  ",					 96, 2, 3,120},
{"a powerful politician.  ",					 99, 2, 3,130},
{"a Head of State.  ",					100, 2, 3,140},
{"You are considered a mistake.  ",			 20, 3,50, 20},
{"You are a credit to your family.  ",				 80, 3,50, 55},
{"You are a well liked, but sometimes forgotten, child.  ",				100, 3,50, 60},
{"Your mother was a Laborer.  ",				 40, 4, 1, 50},
{"Your father was a Laborer.  ",				 75, 4, 1, 55},
{"Your mother was a Manager.  ",				 90, 4, 1, 55},
{"Your father was a Manager.  ",				 95, 4, 1, 60},
{"Your mother was an Elite.  ",				 98, 4, 1, 65},
{"Your father was an Elite.  ",				100, 4, 1, 70},
{"You are one of several children ",				 60, 7, 8, 50},
{"You are the only child ",					100, 7, 8, 55},
{"of a Laborer ",						 75, 8, 9, 50},
{"of a Manager ",						 95, 8, 9, 55},
{"of a Elite ",						100, 8, 9, 60},
{"farmer.  ",							 40, 9,54, 80},
{"industrialist.  ",							 70, 9,54, 90},
{"soldier.  ",							 87, 9,54,110},
{"courtier.  ",							 95, 9,54,125},
{"ambassador.  ",							 99, 9,54,140},
{"ruler.  ",							100, 9,54,145},
{"You are one of several children of a Durranese ",		 85,10,11, 45},
{"You are the only child of a Durranese ",			        100,10,11, 55},
{"criminal.  ",							 20,11, 3, 55},
{"police officer.  ",						 30,11, 3, 80},
{"cook.  ",							 40,11, 3, 90},
{"land owner.  ",						 50,11, 3,100},
{"lawyer.  ",							 80,11, 3,110},
{"military officer.  ",							 95,11, 3,115},
{"scientist.  ",							 99,11, 3,125},
{"politician.  ",						100,11, 3,140},
{"You are one of several children of a Cerebrus ",			 85,13,14, 45},
{"You are the only child of a Cerebrus ",				100,13,14, 55},
{"vagrant.  ",							 20,14, 3, 55},
{"con artist.  ",							 50,14, 3, 70},
{"engineer.  ",						 75,14, 3, 85},
{"soldier.  ",							 95,14, 3,100},
{"Imperial Bodyguard.  ",							100,14, 3,125},
{"You are one of two children of an Ironman ",			 25,16,17, 40},
{"You are the only child of an Ironman",			100,16,17, 50},
{"mugger.  ",							 10,17,18, 60},
{"prison guard.  ",						 25,17,18, 75},
{"miner.  ",							 75,17,18, 90},
{"warrior.  ",							 90,17,18,110},
{"cleric.  ",							 99,17,18,130},
{"Lord.  ",							100,17,18,150},
{"You are a discrase to the family.  ",			 15,18,57, 10},
{"You are the center of attention.  ",				 85,18,57, 50},
{"You had a comfortable childhood.  ",				100,18,57, 55},
{"Your mother was an Angel Ambassador.  ",		 25,19,20, 25},
{"Your father was an Angel Ambassador.  ",		100,19,20, 25},
{"You are the adopted child ",					100,20, 2, 50},
{"Your mother was a Rigellian ",				 30,22,23, 20},
{"Your father was a Rigellian ",				 60,22,23, 25},
{"Your mother was a Rigellian ",				 75,22,23, 30},
{"Your father was a Rigellian ",				 90,22,23, 35},
{"Your mother was a Rigellian ",				 95,22,23, 40},
{"Your father was a Rigellian ",				100,22,23, 45},
{"dancer.  ",							  5,23,62, 60},
{"cleric.  ",							 95,23,62, 55},
{"ceremonial guard.  ",							 99,23,62, 65},
{"Master of Protocol.  ",						100,23,62, 80},
{"You have black eyes, ",					 20,50,51, 50},
{"You have brown eyes, ",					 60,50,51, 50},
{"You have red eyes, ",					 70,50,51, 50},
{"You have green eyes, ",					 80,50,51, 50},
{"You have blue eyes, ",					 90,50,51, 50},
{"You have gray eyes, ",					100,50,51, 50},
{"straight ",							 70,51,52, 50},
{"wavy ",							 90,51,52, 50},
{"curly ",							100,51,52, 50},
{"black hair, ",						 30,52,53, 50},
{"brown hair, ",						 70,52,53, 50},
{"auburn hair, ",						 80,52,53, 50},
{"red hair, ",							 90,52,53, 50},
{"blonde hair, ",						100,52,53, 50},
{"and a very dark complexion.",					 10,53, 0, 50},
{"and a dark complexion.",					 30,53, 0, 50},
{"and an average complexion.",					 80,53, 0, 50},
{"and a fair complexion.",					 90,53, 0, 50},
{"and a very fair complexion.",					100,53, 0, 50},
{"You have white eyes, ",					 85,54,55, 50},
{"You have yellow eyes, ",					 95,54,55, 50},
{"You have violet eyes, ",					100,54,55, 50},
{"straight ",							 75,55,56, 50},
{"wavy ",							100,55,56, 50},
{"black hair, and a fair complexion.",				 75,56, 0, 50},
{"brown hair, and a fair complexion.",				 85,56, 0, 50},
{"blond hair, and a fair complexion.",				 95,56, 0, 50},
{"silver hair, and a fair complexion.",				100,56, 0, 50},
{"You have green and brown eyes, ",					 99,57,58, 50},
{"You have azure eyes, ",					100,57,58, 60},
{"straight ",							 90,58,59, 50},
{"wavy ",							100,58,59, 50},
{"black hair, ",						 75,59,60, 50},
{"brown hair, ",						100,59,60, 50},
{"a one foot beard, ",						 25,60,61, 50},
{"a two foot beard, ",						 60,60,61, 51},
{"a three foot beard, ",					 90,60,61, 53},
{"a four foot beard, ",						100,60,61, 55},
{"and a dark complexion.",					100,61, 0, 50},
{"You have orange eyes, ",					 60,62,63, 50},
{"You have magenta eyes, ",					 85,62,63, 50},
{"You have sea-blue eyes, ",				 99,62,63, 50},
{"You have flame-red eyes, ",					100,62,63, 55},
{"rough ",							 33,63,64, 50},
{"greasy ",							 66,63,64, 50},
{"braided ",							100,63,64, 50},
{"green hair, ",					 33,64,65, 50},
{"red hair, ",						 66,64,65, 50},
{"purple hair, ",						100,64,65, 50},
{"and green ",							 25,65,66, 50},
{"and blue ",							 50,65,66, 50},
{"and white ",							 75,65,66, 50},
{"and black ",							100,65,66, 50},
{"skin.",						 33,66, 0, 50},
{"skin.",						 66,66, 0, 50},
{"skin.",						100,66, 0, 50}
};
#endif

/* Classes.							*/
class_type class[MAX_CLASS] = {
/*  	        HP Dis Src Stl Fos bth btb sve S  D  A  M Co Ch Exp  spl */
{"Mercernary"   ,9, 30, 14, 2, 24, 60, 62, 18, 2, 1,-2,-1, 2, 0, 0, 0},
{"Cyborg",       0, 25, 16, 2, 36, 48, 35, 30,-1,-1, 2, 3,-2,-2, 20, 3},
{"Robot",        2, 25, 16, 1, 38, 34, 20, 36, 3,-3, 3, 5, 3,-2, 50, 1},
{"Marksman", 6, 30, 24, 3, 20, 50, 72, 18,-2, 3,-1, 1, 1, 1, 20, 0},
{"Ninja",        4, 45, 32, 5, 16, 70, 52, 18, 2, 3,-1,-2, 5, 1, 40, 0}
};

/* making it 16 bits wastes a little space, but saves much signed/unsigned
   headaches in its use */
/* CLA_MISC_HIT is identical to CLA_SAVE, which takes advantage of
   the fact that the save values are independent of the class */
int16 class_level_adj[MAX_CLASS][MAX_LEV_ADJ] = {
/*	      	 bth    bthb   device  disarm   save/misc hit  */
/* Merc */ {  4,	4,    	2,		2,			3 },
/* Cyborg*/ { 2,	2,		3,		2,			4 },
/* Robot  */ { 2,	2,		4,		2,			5 },
/* Sharp   */ { 3,	5,		2,		3,			3 },
/* Ninja  */ { 5,	3,		2,		4,			3 }
};

int32u spell_learned = 0;	/* bit mask of spells learned */
int32u spell_learned2 = 0;	/* bit mask of spells learned */
int32u spell_worked = 0;	/* bit mask of spells tried and worked */
int32u spell_worked2 = 0;	/* bit mask of spells tried and worked */
int32u spell_forgotten = 0;	/* bit mask of spells learned but forgotten */
int32u spell_forgotten2 = 0;	/* bit mask of spells learned but forgotten */
int8u spell_order[64];		/* order spells learned/remembered/forgotten */

/* Warriors don't have spells, so there is no entry for them.  Note that
   this means you must always subtract one from the py.misc.pclass before
   indexing into magic_spell[]. */
#ifdef MACGAME
spell_type (*magic_spell)[63];
#else
spell_type magic_spell[NUM_SPELLCAST_CL][42] = {
  {		  /* Cyborg	   */
	/*lev	mna	fail	exp */
	{	2,	2,	24,		2	},
	{	1,	1,	22,		1	},
	{	2,	1,	23,		1	},
	{	1,	2,	22,		1	},
	{	2,	2,	26,		2	},
	{	2,	2,	27,		2	},
	{	3,	2,	30,		1	},
	{	3,	3,	30,		1	},
	{	4,	5,	35,		3	},
	{	4,	5,	35,		3	},
	{	5,	6,	30,		6	},
	{	7,	6,	40,		9	}, /* Neural sear */
	{	7,	5,	30,		5	},
	{	9,	7,	45,		10	},
	{	9,	7,	40,		10	},
	{	9,	7,	45,		12	},
	{	11,	8,	45,		19	},
	{	99, 99, 0,		0	},	/* Electric bolt */
	{	15,	12,	55,		25	},
	{	15,	10,	40,		31	},
	{	19,	15,	66,		31	},
	{	99, 99, 0, 		0	},
	{	21,	18,	65	,	38	},
	{	99, 99, 0,		0 	},
	{	25,	18,	80,		60	},	/* Phase other */
	{	25,	18,	70,		50	},
	{	99, 99, 0,		0	},
	{	99, 99, 0,		0	},
	{	99,	99,	0,		0	},
	{	29,	21,	70,		50	},
	{	99, 99, 0,		0	},
	{	30,	25,	75,		54	},
	{	28,	25,	65,		60	},
	{	99, 99, 0,		0	},	/* Electric shielding */
	{	26,	20,	50,		45	},
	{	28,	30,	60,		55	},
	{	27,	25,	51,		46	}	/* Seismic mapping	*/
,	{	99,99,0,0}				/* blanks	*/
,	{	99,99,0,0}
,	{	99,99,0,0}
,	{	99,99,0,0}
,	{	99,99,0,0}
   },
   {		  /* Robot	   */
	{	1,	1,	10,		1	},	/* Neural Shock	*/
	{	1,	1,	10,		2	},	
	{	1,	2,	15,		3	},
	{	1,	2,	15,		3	},
	{	2,	3,	20,		5	},
	{	2,	3,	20,		7	},	/* Minor Repairs	*/
	{	3,	5,	25,		10	},
	{	3,	6,	30,		15	},
	{	99,99,0,0},				/* Adrenaline rush*/
	{	4,	7,	25,		30	},
	{	4,	6,	30,		40	},
	{	5,	8,	30,		50	},
	{	4,	6,	20,		40	},
	{	5,	8,	30,		50	},
	{	6,	9,	40,		60	},	/* Filter blood */
	{	6,	9,	40,		60	},
	{	9,	10,	35,		80	},	
	{	10,	10,	38,		100	},
	{	10,	9,	40,		120	},
	{	9,	9,	30,		130	},
	{	10,	11,	50,		140	},	/* Encyclopedic Ref. */
	{	13,	11,	50,		150	},	
	{	13,	11,	45,		150	},
	{	15,	14,	50,		200	},	
	{	15,	13,	35,		180	},
	{	17,	16,	50,		200	},
	{	17,	16,	50,		210	},
	{	19,	20,	60,		240	},
	{	21,	24,	60,		250	},	/* Exterminate */
	{	21,	20,	40,		210	},
	{	25,	30,	70,		280	},
	{	22,	23,	40,		200	},
	{	99,99,0,0},				/* Karmic shield	*/
	{	25,	25,	30,		230	},
	{	22,	23,	40,		200	},
	{	25,	24,	40,		230	},
	{	22,	20,	30,		205	},	/* Seismic Mapping	*/
{99,99,0,0},					/* blanks */
{99,99,0,0},
{99,99,0,0},
{99,99,0,0},
{99,99,0,0}
   }
 };
#endif

char *spell_names[42] = {
	"Neural Shock",
	"Detect Sounds",
	"Phase Self",
	"Light Area",
	"Metal and Gem Imaging",
	"Minor Repairs",
	"Inorganic Detection",
	"Sonic Analysis",
	"Adrenaline Rush",
	"Neural Blast",
	"Thought Blender",
	"Neural Sear",
	"Disarm",
	"Knockout",
	"Filter Blood",
	"Transport Self",
	"Laser Beam",
	"Electric Bolt",
	"Recharging",
	"Instant Mutation",
	"Encyclopedic Reference",
	"Energy Bolt",
	"Atrophy Muscles",
	"Ball Lightning",
	"Phase Other",
	"Haste",
	"Energy Ball",
	"Destruction",
	"Exterminate",
	"Long-Range Transportation",
	"Mass Extermination",
	"Subspace Sweep",
	"Karmic Shield",
	"Electric Shielding",
	"Concentration",
	"Earthquake",
	"Seismic Mapping",

  "blank",
  "blank",
  "blank",
  "blank",
  "blank"
};

#define MDO MAX_DUNGEON_OBJ
/* Each type of character starts out with a few provisions.	*/
/* Note that the entries refer to elements of the object_list[] array*/
/* 356 = Food Ration, 365 = Wooden Torch, 123 = Cloak, 30 = Stiletto,
   103 = Soft Leather Armor, 318 = Beginners-Magic, 322 = Beginners Handbook */
int16u player_init[MAX_CLASS][5] = {
		{ 0, 2, MDO+5, 8, 10} ,	/* Merc	 */
		{ 0, 2, MDO+5, 8, 10}, 	/* Cyborg		 */
		{ 0, 2, MDO+5, 8, 10}, 	/* Robot	 */
			{ 0, 2, MDO+5, 8, 10}, /* Sharp	 */
		{ 0, 2, MDO+5, 8, 10} 	/* Ninja	 */
        /* Last array object added for one extra useful object per class */
};
