/* recall.c: print out monster memory info			-CJS-

   Copyright (c) 1989 James E. Wilson, Christopher J. Stuart

   This software may be copied and distributed for educational, research, and
   not for profit purposes provided that this copyright and statement are
   included in all such copies. */

#include "constant.h"
#include "config.h"
#include "monster.h"
#include "types.h"
#include "externs.h"

static void roff();

static char *desc_atype[] = {
  "do something undefined",
  "attack",
  "weaken",
  "confuse",
  "terrify",
  "shoot flames",
  "shoot acid",
  "freeze",
  "shoot lightning",
  "corrode",
  "blind",
  "paralyse",
  "steal money",
  "steal things",
  "poison",
  "reduce dexterity",
  "reduce constitution",
  "drain intelligence",
  "drain wisdom",
  "lower experience",
  "call for help",
  "disenchant",
  "eat your food",
  "absorb light",
  "absorb charges",
  "reduce all stats" };
static char *desc_amethod[] = {
  "make an undefined advance",
  "hit",
  "bite",
  "claw",
  "punch",
  "shoot",
  "kick",
	"trample",
  "gouge",
  "run over",
  "stab",
  "sell souvenirs",
  "crawl on you",
  "release spores",
  "beg",
  "slime you",
  "crush",
  "trample",
  "drool",
  "insult",
  "butt",
  "charge",
  "engulf",
  "moan"
};
static char *desc_howmuch[] = {
  " not at all",
  " a bit",
  "",
  " quite",
  " very",
  " most",
  " highly",
  " extremely" };

static char *desc_move[] = {
  "move invisibly",
  "open doors",
  "pass through walls",
  "kill weaker creatures",
  "pick up objects",
  "breed explosively" };
static char *desc_spell[] = {
  "a pistol",
  "a rifle",
  "a shotgun",
  "a submachine gun",
  "a rocket launcher",
  "a sniper rifle",
  "a teleporter at its command",
  "cybernetic enhancements",
  "advanced cybernetics",
  "the power to create traps",
  "psionic razors",
  "something undefinied",
  "something undefinied",
  "something undefinied",
  "something undefinied",
  "something undefinied",
  "something undefinied",
  "reinforcements available",
  "something undefinied",
  "something undefinied",
  "something undefinied",     
  "mind-blanking psionics",
  "the psychic ability to smash minds",
  "something undefinied",
  "a long range teleporter",             /* end */
  "the ability to cause storms to psychic energy",
  "something undefinied",
  "something undefinied",
  "something undefinied",
  "something undefinied",
  "something undefinied",
  "something undefinied"
};
static char *desc_weakness[] = {
  "bright light",
  "rock remover"
};
static char *desc_immune[] = {
  "frost",
  "fire",
  "lightning",
  "poison",
  "acid"
};

static vtype roffbuf;		/* Line buffer. */
static char *roffp;		/* Pointer into line buffer. */
static int roffpline;		/* Place to print line now being loaded. */

#define plural(c, ss, sp)	((c) == 1 ? ss : sp)

/* Number of kills needed for information. */

/* the higher the level of the monster, the fewer the kills you need */
#define knowarmor(l,d)		((d) > 304 / (4 + (l)))
/* the higher the level of the monster, the fewer the attacks you need,
   the more damage an attack does, the more attacks you need */
#define knowdamage(l,a,d)	((4 + (l))*(a) > 80 * (d))

/* use slightly different tests for unique monsters to hasten learning -CFT */
#define knowuniqarmor(l,d)    ((d) > 304 / (38 + (5*(l))/4))
#define knowuniqdamage(l,a,d) ((4 +(long)(l))*(2*(long)(a)) > 80 * (d))

/* Do we know anything about this monster? */
int bool_roff_recall(mon_num)
int mon_num;
{
  register recall_type *mp;
  register int i;

  if (wizard)
    return TRUE;
  mp = &c_recall[mon_num];
  if (mp->r_cmove || mp->r_flags1 || mp->r_kills || mp->r_flags2 ||
      mp->r_deaths)
    return TRUE;
  for (i = 0; i < 6; i++)
    if (mp->r_attacks[i])
      return TRUE;
  return FALSE;
}

/* Print out what we have discovered about this monster. */
int roff_recall(mon_num)
int mon_num;
{
  char *p, *q;
  attid *pu;
  vtype temp;
  register recall_type *mp;
  register creature_type *cp;
  register int i, k;
  int32u j;
  int mspeed;
  int32u rcmove, rflags1, rflags2;
  recall_type save_mem;
  int breath=FALSE, magic=FALSE;

  cp = &c_list[mon_num];

  mp = &c_recall[mon_num];
  if (wizard) {
    save_mem = *mp;
    mp->r_kills = MAX_SHORT;
    mp->r_wake = mp->r_ignore = MAX_UCHAR;
    j = (((cp->cmove & CM_4D2_OBJ) != 0) * 8) +
      (((cp->cmove & CM_2D2_OBJ) != 0) * 4) +
	(((cp->cmove & CM_1D2_OBJ) != 0) * 2) +
	  ((cp->cmove & CM_90_RANDOM) != 0) +
	    ((cp->cmove & CM_60_RANDOM) != 0);
    mp->r_cmove = (cp->cmove & ~CM_TREASURE) | (j << CM_TR_SHIFT);
    mp->r_flags1 = cp->flags1;
    mp->r_flags2 = cp->flags2 ;
    j = 0;
    pu = cp->damage;
    while (*pu != 0 && j < 6) {
      mp->r_attacks[j] = MAX_UCHAR;
      j++;
      pu++;
    }
  }
  roffpline = 0;
  roffp = roffbuf;
  rflags1 = mp->r_flags1 & cp->flags1 ;
  /* the CM_WIN property is always known, set it if a win monster */
  rcmove = mp->r_cmove | (CM_WIN & cp->cmove);
  rflags2 = mp->r_flags2 & cp->flags2;
  if (cp->flags2 & UNIQUE)
    (void) sprintf(temp, "%s:\n", cp->name);
  else
    (void) sprintf(temp, "The %s:\n", cp->name);
  roff(temp);
  /* Conflict history. */
  /* changed to act better for unique monsters -CFT */
    if (cp->flags2 & UNIQUE) { /* treat unique differently... -CFT */
      if (mp->r_deaths) { /* We've been killed... */
       (void) sprintf(temp, "It has slain %d of your ancestors", mp->r_deaths);
        roff(temp);
        if (u_list[mon_num].dead) { /* but we've also killed it */
        roff(", but you have avenged them!");
          }
        }        
      else if (u_list[mon_num].dead) { /* we killed it w/o dying... yet! */
        roff("You have slain this foe.");
        }
      }      
    else if (mp->r_deaths) { /* not unique.... */
    (void) sprintf(temp,
                   "%d of ancestors %s",
                   mp->r_deaths, plural(mp->r_deaths, "has", "have") );
      roff(temp);
      roff(" been killed by this creature, and ");
      if (mp->r_kills == 0)
        roff("it is not ever known to have been defeated.");
      else {
        (void) sprintf(temp,
                       "at least %d of the beasts %s been exterminated.",
                       mp->r_kills, plural(mp->r_kills, "has", "have") );
        roff(temp);
      }
    }
    else if (mp->r_kills) {
      (void) sprintf(temp, "At least %d of these creatures %s",
                     mp->r_kills, plural(mp->r_kills, "has", "have") );
      roff(temp);
      roff(" been killed by your ancestors.");
    }
     else roff("No battles to the death are recalled.");
  /* Immediately obvious. */
  for (k=0; k<MAX_CREATURES; k++) {
    if (!strcmp(desc_list[k].name, cp->name)) {
      if (strlen(desc_list[k].desc) != 0) {
	roff(" ");
	roff(desc_list[k].desc);
	roff(".");
      }
      break;
    }
  }
  if (k==MAX_CREATURES)
    roff(" You feel you know it, and it knows you. This can only mean trouble.");
  k = FALSE;
  if (cp->level == 0)
    {
      roff(" It lives in the town");
      k = TRUE;
    }
  else if (mp->r_kills)
    {
      (void) sprintf(temp, " It is normally found at depths of %d feet",
		     cp->level*50);
      roff(temp);
      k = TRUE;
    }
  /* the c_list speed value is 10 greater, so that it can be a int8u */
  mspeed = cp->speed - 10;
  if ((rcmove & CM_ALL_MV_FLAGS) || (rcmove & CM_RANDOM_MOVE))
    {
      if (k)
	roff(", and");
      else
	{
	  roff(" It");
	  k = TRUE;
	}
      roff(" moves");
      if (rcmove & CM_RANDOM_MOVE)
	{
	  roff(desc_howmuch[(rcmove & CM_RANDOM_MOVE) >> 3]);
	  roff(" erratically");
	}
      if (mspeed == 1)
	roff(" at normal speed");
      else
	{
	  if (rcmove & CM_RANDOM_MOVE)
	    roff(", and");
	  if (mspeed <= 0)
	    {
	      if (mspeed == -1)
		roff(" very");
	      else if (mspeed < -1)
		roff(" incredibly");
	      roff(" slowly");
	    }
	  else
	    {
	      if (mspeed == 3)
		roff(" very");
	      else if (mspeed > 3)
		roff(" unbelievably");
	      roff(" quickly");
	    }
	}
    }
  if (rcmove & CM_ATTACK_ONLY)
    {
      if(k)
	roff(", but");
      else
	{
	  roff(" It");
	  k = TRUE;
	}
      roff(" does not deign to chase intruders");
    }
  if(k)
    roff(".");
  /* Kill it once to know experience, and quality (evil, undead, monsterous).
     The quality of being a dragon is obvious. */
  if (mp->r_kills)
    {
      roff(" A kill of this"); /* elses removed because animal/evil/undead
				        aren't mutually exclusive -CFT */
      if (cp->flags2 & ANIMAL)
	roff(" natural");
      if (cp->flags2 & ARMORED)
	roff(" armored vehicle");
      else roff(" creature");

      /* calculate the integer exp part */
      i = (long)cp->mexp * cp->level / py.misc.lev;
      /* calculate the fractional exp part scaled by 100,
	 must use long arithmetic to avoid overflow */
      j = (((long)cp->mexp * cp->level % py.misc.lev) * (long)1000 /
	   py.misc.lev+5) / 10;

      (void) sprintf(temp, " is worth %d.%02ld point%s", i,
		     j, (i == 1 && j == 0 ? "" : "s"));
      roff(temp);

      if ((py.misc.lev / 10) == 1) p = "th";
      else
	{
	  i = py.misc.lev % 10;
	  if (i == 1)		p = "st";
	  else if (i == 2)	p = "nd";
	  else if (i == 3)      p = "rd";
	  else			p = "th";
	}
      i = py.misc.lev;
      if ((i == 8) || (i == 11) || (i == 18)) q = "n";
      else				q = "";
      (void) sprintf(temp, " for a%s %d%s level character.", q, i, p);
      roff(temp);
      if (cp->flags2 & GROUP)
	roff(" It usually appears in groups.");
    }
  /* Spells known, if have been used against us. */
  k = TRUE;
  i = 0;
  if (rflags1 ) {
    magic=TRUE;
    j=rflags1;
    while ((i=bit_pos(&j)) != -1) {
      if (k) {
	  roff(" It has");
/*	if (mp->r_cdefense & INTELLIGENT)
	  roff(" magical, casting spells intelligently which ");
	else
	  roff(" magical, casting spells which ");
*/
	k = FALSE;
      }
      else if (j)
	roff(", ");
      else
	roff(" and ");
      roff(desc_spell[i]);
    }
  }
  if (magic) {
    if (mp->r_flags2 & ALWAYS_CAST) { /* Could offset by level */
      roff(". It never stops firing");
    }
    roff(".");
  }
  /* Do we know how hard they are to kill? Armor class, hit die. */
  if (knowarmor(cp->level, mp->r_kills) || /* hasten learning  of uniques -CFT */
      ((cp->flags2 & UNIQUE) && knowuniqarmor(cp->level, mp->r_kills)))
    {
      (void) sprintf(temp, " It has an armor rating of %d", cp->ac);
      roff(temp);
      (void) sprintf(temp, " and a%s life rating of %dd%d.",
		     ((cp->flags2 & MAX_HP) ? " maximized" : ""),
		     cp->hd[0], cp->hd[1]);
      roff(temp);
    }
  /* Do we know how clever they are? Special abilities. */
  k = TRUE;
  j = rcmove;
    if (rflags2 & BREAK_WALL) { /* I wonder why this wasn't here before? -CFT */
      roff(" It can bore through rock");
      k = FALSE;
      }
  for (i = 0; j & CM_SPECIAL; i++) {
    if (j & (CM_INVISIBLE << i)) {
      j &= ~(CM_INVISIBLE << i);
      if (k) {
	roff(" It can ");
	k = FALSE;
      }
      else if (j & CM_SPECIAL)
	roff(", ");
      else
	roff(" and ");
      roff(desc_move[i]);
    }
  }
  if (!k)
    roff(".");
  /* Do we know its immunities? They are in flags2. */
  j = rflags2;
  k = TRUE;
  for (i = 0; j & (IM_FROST|IM_FIRE|IM_ACID|IM_POISON|IM_LIGHTNING); i++)
    {
      if (j & (IM_FROST << i))
	{
	  j &= ~(IM_FROST << i);
	  if (k)
	    {
	      roff(" It resists ");
	      k = FALSE;
	    }
	  else if (j & (IM_FROST|IM_FIRE|IM_ACID|IM_POISON|IM_LIGHTNING))
	    roff(", ");
	  else
	    roff(" and ");
	  roff(desc_immune[i]);
	}
    }
  if (!k)
    roff(".");
  if (rflags2 & CHARM_SLEEP)
    {
      roff(" It cannot be charmed or slept.");
    }
  /* Do we know how aware it is? */
  if (((mp->r_wake * mp->r_wake) > cp->sleep) || mp->r_ignore == MAX_UCHAR ||
      (cp->sleep == 0 && mp->r_kills >= 10))
    {
      roff(" It ");
      if(cp->sleep > 200)
	roff("prefers to ignore");
      else if(cp->sleep > 95)
	roff("pays very little attention to");
      else if(cp->sleep > 75)
	roff("pays little attention to");
      else if(cp->sleep > 45)
	roff("tends to overlook");
      else if(cp->sleep > 25)
	roff("takes quite a while to see");
      else if(cp->sleep > 10)
	roff("takes a while to see");
      else if(cp->sleep > 5)
	roff("is fairly observant of");
      else if(cp->sleep > 3)
	roff("is observant of");
      else if(cp->sleep > 1)
	roff("is very observant of");
      else if(cp->sleep != 0)
	roff("is vigilant for");
      else
	roff("is ever vigilant for");
      (void) sprintf(temp, " intruders, which it may notice from %d meters.",
		     10 * cp->aaf);
      roff(temp);
    }
  /* Do we know what it might carry? */
  if (rcmove & (CM_CARRY_OBJ|CM_CARRY_GOLD))
    {
      roff(" It may");
      j = (rcmove & CM_TREASURE) >> CM_TR_SHIFT;
      if (j == 1)
	{
	  if ((cp->cmove & CM_TREASURE) == CM_60_RANDOM)
	    roff(" sometimes");
	  else
	    roff(" often");
	}
      else if ((j == 2) && ((cp->cmove & CM_TREASURE) ==
			    (CM_60_RANDOM|CM_90_RANDOM)))
	roff (" often");
      roff(" carry");
      p = " objects";
      if (j == 1)
	p = " an object";
      else if (j == 2)
	roff(" one or two");
      else
	{
	  (void) sprintf(temp, " up to %ld", j);
	  roff(temp);
	}
      if (rcmove & CM_CARRY_OBJ)
	{
	  roff(p);
	  if (rcmove & CM_CARRY_GOLD)
	    {
	      roff(" or treasure");
	      if (j > 1)
		roff("s");
	    }
	  roff(".");
	}
      else if (j != 1)
	roff(" treasures.");
      else
	roff(" treasure.");
    }

  /* We know about attacks it has used on us, and maybe the damage they do. */
  /* k is the total number of known attacks, used for punctuation */
  k = 0;
  for (j = 0; j < 6; j++)
    if (mp->r_attacks[j])
      k++;
  pu = cp->damage;
  /* j counts the attacks as printed, used for punctuation */
  j = 0;
  for (i = 0; *pu != 0 && i < 4; pu++, i++)
    {
      int att_type, att_how, d1, d2;

      /* don't print out unknown attacks */
      if (!mp->r_attacks[i])
	continue;

      att_type = monster_attacks[*pu].attack_type;
      att_how = monster_attacks[*pu].attack_desc;
      d1 = monster_attacks[*pu].attack_dice;
      d2 = monster_attacks[*pu].attack_sides;

      j++;
      if (j == 1)
	roff(" It can ");
      else if (j == k)
	roff(", and ");
      else
	roff(", ");

      if (att_how > 23)
	att_how = 0;
      roff(desc_amethod[att_how]);
      if (att_type != 1 || d1 > 0 && d2 > 0)
	{
	  roff(" to ");
	  if (att_type > 25)
	    att_type = 0;
	  roff(desc_atype[att_type]);
	  if (d1 && d2)
	    {
              if (knowdamage(cp->level, mp->r_attacks[i], (int)d1*(int)d2) ||
                 ((cp->flags2 & UNIQUE) && /* learn uniques faster -CFT */
                 knowuniqdamage(cp->level, mp->r_attacks[i], (int)d1*(int)d2)))
		{
		  if (att_type == 19)	/* Loss of experience */
		    roff(" by");
		  else
		    roff(" with damage");
		  (void) sprintf(temp, " %dd%d", d1, d2 );
		  roff(temp);
		}
	    }
	}
    }
  if (j)
    roff(".");
  else if (k > 0 && mp->r_attacks[0] >= 10)
    roff(" It has no physical attacks.");
  else
    roff(" Nothing is known about its attack.");
  /* Always know the win creature. */
  if (cp->cmove & CM_WIN)
    roff(" Killing him wins the game!");
  roff("\n");
  prt("--pause--", roffpline, 0);
  if (wizard)
    *mp = save_mem;
  return inkey();
}

/* Print out strings, filling up lines as we go. */
static void roff(p)
register char *p;
{
  register char *q, *r;
  register int linesize;

  linesize=sizeof(roffbuf);
  if (linesize>80) linesize=80;

  while (*p)
    {
      *roffp = *p;
      if (*p == '\n' || roffp >= roffbuf + linesize - 1)
	{
	  q = roffp;
	  if (*p != '\n')
	    while (*q != ' ')
	      q--;
	  *q = 0;
	  prt(roffbuf, roffpline, 0);
	  roffpline++;
	  r = roffbuf;
	  while (q < roffp)
	    {
	      q++;
	      *r = *q;
	      r++;
	    }
	  roffp = r;
	}
      else
	roffp++;
      p++;
    }
}
