/* treasure.c: dungeon object definitions

   Copyright (c) 1989 James E. Wilson, Robert A. Koeneke

   This software may be copied and distributed for educational, research, and
   not for profit purposes provided that this copyright and statement are
   included in all such copies. */

#include "constant.h"
#include "config.h"
#include "types.h"


#define ARMOR_PIERCE TR_ARMOR_PIERCE

/* Following are treasure arrays	and variables			*/

/* Object description:	Objects are defined here.  Each object has
  the following attributes:
	Descriptor : Name of item and formats.
		& is replaced with 'a', 'an', or a number.
		~ is replaced with null or 's'.
	Flags: Flags accosiated with weapon.
	Type value : Value representing the type of object.
	Character: How object is represented on screen.
	P1: Field that contain misc. stuff
	Cost: How much it is worth
	Sub value  : separate value for each item of a type.
		0 - 63: object can not stack
		64 - 127: dungeon object, can stack with other D object
		128 - 191: unused, previously for store items
		192: stack with other iff have same p1 value, always
			treated as individual objects
		193 - 255: object can stack with others iff they have
			the same p1 value, usually considered one group
		Objects which have two type values, e.g. potions and
		scrolls, need to have distinct subvals for
		each item regardless of its tval
	Number:		how many of them there are
	Weight:		in tenths of a pound
	tohit:		magical plusses to hit
	todam: 		magical plusses to dam 
	AC:		armor class of the object
	toac:	magic addition to AC
	shots:	How many shots are left in the weapon
	maxshots:	How many shots the weapon can hold
	Damage	   : amount of damage item can cause.
	Level	   : Minimum level on which item can be found.
	Rarity     : Normally 0, but is 1/rare chance of it being found
	Flags2     : Flags associated with the ammo

	Special Abilities can be added to item by magic_init(),
	found in misc.c.

	Scrolls, Potions, and Food:
	Flags is used to define a function which reading/quaffing
	will cause.  Most scrolls and potions have only one bit
	set.  Potions will generally have some food value, found
	in p1.

	Wands and Staffs:
	Flags defines a function, p1 contains number of charges
	for item.  p1 is set in magic_init() in misc.c.

	Chests:
	Traps are added randomly by magic_init() in misc.c.	*/
/* Note: magic_init() seems to have been moved to desc.c  
	It initalizes potions, wands, etc...
	magic_treasure() in misc1.c adds the plusses to items. */

/* Object list (All objects must be defined here)		 */
#ifdef MACGAME
treasure_type *object_list;
#else              0
treasure_type object_list[MAX_OBJECTS] = {
/* Dungeon items from 0 to MAX_DUNGEON_OBJ */

{ "& Blowgun", 0U, TV_BLOWGUN, '\\', 2, 20, 0, 1, 1, 
	0, 0, 0, 0, 0, 0, {0, 0}, 1, 1, 0U },   /* #0 */
{ "& blowgun dart~", 0U, TV_AMMO, '{', 0, 1, 64+AMMO_CONST, 10, 
	1, 0, 0, 0, 0, 0, 0, {1,1}, 1, 1, 0U }, 
{ "& .22 Caliber Pistol", 0U, TV_HANDGUN, '\\', 0, 50, 0, 1, 20, 
	0, 0, 0, 0, 0, 12, {1,3}, 2, 2, 0U },
{ "& .38 Caliber Pistol", 0U, TV_HANDGUN, '\\', 0, 100, 1, 1, 23, 
	0, 0, 0, 0, 0, 12, {1,6}, 3, 2, 0U },
{ "& 9 mm Pistol", 0U, TV_HANDGUN, '\\', 0, 150, 2, 1, 25, 
	0, 0, 0, 0, 0, 18, {1,10}, 4, 3, 0U },
{ "& .22 caliber shot~", 0U, TV_AMMO, '{', 0, 1, 65+AMMO_CONST, 10, 1, 
	0, 0, 0, 0, 0, 0, {0,0}, 2, 1, 0U },
{ "& .38 caliber shot~", 0U, TV_AMMO, '{', 0, 1, 66+AMMO_CONST, 10, 1,
	0, 0, 0, 0, 0, 0, {0,0}, 3, 1, 0U },
{ "& 9 mm shot~", 0U, TV_AMMO, '{', 0, 2, 67+AMMO_CONST, 10, 1,
	0, 0, 0, 0, 0, 0, {0,0}, 4, 1, 0U },
{ "& Shotgun", 0U, TV_HANDGUN, '\\', 0, 250, 3, 1, 70,
	0, 0, 0, 0, 0, 6, {1,15}, 4, 2, 0U },
{ "& shotgun shell~", 0U, TV_AMMO, '{', 0, 3, 68+AMMO_CONST, 10, 2,  
	0, 0, 0, 0, 0, 0, {0,0}, 4, 1, 0U },
{ "& Needle Gun", 0U, TV_HANDGUN, '\\', 0, 750, 4, 1, 60,
	0, 0, 0, 0, 0, 100, {1,21}, 5, 2, 0U },
{ "& needle burst~", 0U, TV_AMMO, '{', 0, 5, 69+AMMO_CONST, 10, 1,
	0, 0, 0, 0, 0, 0, {0,0}, 5, 1, 0U },
{ "& Shredder Pulse Gun", 0U, TV_HEAVYGUN, '/', 0, 2000, 5, 1, 80,
	0, 0, 0, 0, 0, 500, {1,28}, 6, 3, 0U },
{ "& Shuriken~", 0U, TV_AMMO, '{', 0, 2, 70+AMMO_CONST, 10, 1,
	0, 0, 0, 0, 0, 0, {2,1}, 2, 1, 0U },
{ "& Two-Handed Sword", 0U, TV_SWORD, '|', 0, 75, 0, 1, 100,
	0, 0, 0, 0, 0, 0, {2,3}, 2, 2, 0U },
{ "& M-16 Rifle", 0U, TV_HEAVYGUN, '/', 0, 250, 6, 1, 65,
	0, 0, 0, 0, 0, 50, {2,6}, 7, 2, 0U },
{ "& 5.56 mm burst~", 0U, TV_AMMO, '}', 0, 3, 71+AMMO_CONST, 10, 3,
	0, 0, 0, 0, 0, 0, {0,0}, 7, 1, 0U },
{ "& Gauss Rifle", 0U, TV_HEAVYGUN, '/', 0, 1000, 7, 1, 72,
	0, 0, 0, 0, 0, 100, {2,10}, 8, 2, 0U },
{ "& Gauss pellet~", 0U, TV_AMMO, '}', 0, 7, 72+AMMO_CONST, 10, 3,
	0, 0, 0, 0, 0, 0, {0,0}, 8, 1, 0U },
{ "& Grenade Launcher", FL_BALL, TV_HEAVYGUN, '/', 0, 1500, 8, 1, 82,
	0, 0, 0, 0, 0, 8, {2,15}, 9, 2, 0U },
{ "& Grenade~", 0U, TV_AMMO, '}', 0, 10, 73+AMMO_CONST, 10, 5,
	0, 0, 0, 0, 0, 0, {1,10}, 9, 1, FL_BALL },
{ "& Laser Bazooka", 0U, TV_HEAVYGUN, '/', 0, 3000, 9, 1, 87, 
	0, 0, 0, 0, 0, 30, {2,21}, 10, 2, 0U },
{ "& Laserpack~", 0U, TV_AMMO, '}', 0, 3, 74+AMMO_CONST, 10, 3,
	0, 0, 0, 0, 0, 0, {0,0}, 10, 1, 0U },
{ "& Dagger", 0U, TV_SWORD, '|', 0, 30, 1, 1, 5,
	0, 0, 0, 0, 0, 0, {3,1}, 3, 1, 0U },
{ "& Axe", 0U, TV_SWORD, '|', 0, 125, 2, 1, 45,
	0, 0, 0, 0, 0, 0, {3,3}, 5, 2, 0U },
{ "& Laser Rifle", 0U, TV_HEAVYGUN, '/', 0, 1100, 10, 1, 70,
	0, 0, 0, 0, 0, 20, {3,6}, 5, 2, 0U },
{ "& Flamethrower", FL_BALL, TV_HEAVYGUN, '/', 0, 1300, 11, 1, 100,
	0, 0, 0, 0, 0, 12, {3,10}, 7, 2, 0U },
{ "& flame fuel pack~", 0U, TV_AMMO, '}', 0, 10, 75+AMMO_CONST, 10, 15,
	0, 0, 0, 0, 0, 0, {0, 0}, 7, 1, 0U },
{ "& Uranium Pellet Gun", FL_BALL, TV_HEAVYGUN, '/', 0, 3500, 12, 1, 140,
	0, 0, 0, 0, 0, 20, {3,15}, 12, 2, 0U },
{ "& U-238 pellet~", 0U, TV_AMMO, '}', 0, 30, 76+AMMO_CONST, 10, 2,
	0, 0, 0, 0, 0, 0, {0,0}, 12, 1, 0U },
{ "& Whip", 0U, TV_SWORD, '|', 0, 40, 3, 1, 2,
	0, 0, 0, 0, 0, 0, {4,1}, 4, 1, 0U },
{ "& Uzi", 0U, TV_HANDGUN, '\\', 0, 150, 13, 1, 67, 
	0, 0, 0, 0, 0, 50, {4,3}, 8, 2, 0U },
{ "& Chaingun", 0U, TV_HANDGUN, '\\',  0, 1300, 14, 1, 90,
	0, 0, 0, 0, 0, 200, {4,6}, 15, 2, 0U },
{ "& 7.62 mm burst~", 0U, TV_AMMO, '{', 0, 4, 77+AMMO_CONST, 10, 3,
	0, 0, 0, 0, 0, 0, {0,0}, 15, 1, 0U },
{ "& Rocket Launcher", FL_BALL, TV_HEAVYGUN, '/', 0, 2000, 15, 1, 110,
	0, 0, 0, 0, 0, 5, {4,10}, 20, 2, 0U },
{ "& rocket~", 0U, TV_AMMO, '}', 0, 15,  78+AMMO_CONST, 10, 10, 
	0, 0, 0, 0, 0, 0, {0,0}, 20, 1, 0U},
{ "Nunchaku", 0U, TV_SWORD, '|', 0, 45, 4, 1, 12,
	0, 0, 0, 0, 0, 0, {5,1}, 10, 2, 0U },
{ "& Sniper Rifle", 0U, TV_HANDGUN, '\\', 0, 350, 16, 1, 30,
	5, 0, 0, 0, 0, 10, {5,3}, 10, 2, 0U },
{ "& Anti-Tank Cannon", FL_BALL|ARMOR_PIERCE, TV_HEAVYGUN, '/', 0, 1750, 17,
	 1, 500, 0, 0, 0, 0, 0, 8, {5,6}, 15, 2, 0U },
{ "& .50 caliber shell~", 0U, TV_AMMO, '}', 0, 5, 79+AMMO_CONST, 10, 4,
	0, 0, 0, 0, 0, 0, {0,0}, 10, 1, 0U },
{ "& 25 mm shell~", 0U, TV_AMMO, '}', 0, 20, 80+AMMO_CONST, 10, 30,
	0, 0, 0, 0, 0, 0, {0,0}, 15, 1, 0U },
{ "& Sword", 0U, TV_SWORD, '|', 0, 55, 5, 1, 27,
	0, 0, 0, 0, 0, 0, {6,1}, 4, 2, 0U },
{ "& Browning HMG", 0U, TV_HEAVYGUN, '/', 0, 1150, 18, 1, 700,
	0, 0, 0, 0, 0, 200, {6,3}, 20, 2, 0U },
{ "& Chainsaw", 0U, TV_SWORD, '|', 0, 150, 6, 1, 85,
	0, 0, 0, 0, 0, 0, {7,1}, 11, 1, 0U },   /* #43 */
{ "& Sword-Hilt", 0U, TV_SWORD, '|', 0, 0, 7, 1, 5, 
	0, 0, 0, 0, 0, 0, {1,1}, 1, 0, 0U },
{ "[Intro. to Programming]", 0xA000012FUL , TV_CD_ROM, '?',
	0, 25, 64, 1, 30, 0, 0, 0, 0, 0, 0, {1,1}, 5, 0, 4U },
{ "[Control Routines]", 0x2100F7F0UL, TV_CD_ROM, '?', 
	0, 100, 65, 1, 30, 0,0, 0, 0, 0, 0, {1,1}, 10, 0, 0x18U },
{ "[Advanced Algorithms]", 0x525f0C00UL, TV_CD_ROM, '?',
	0, 400, 66, 1, 30, 0,0, 0, 0, 0, 0, {1,1}, 20, 0, 3U },
{ "[Skynet\'s Guide to Power]", 0x0CA20000UL, TV_CD_ROM, '?',
	0, 800, 67, 1, 30, 0,0, 0, 0, 0, 0, {1,1}, 25, 0, 0U },
{ "& small wooden chest", 0x0F000000UL, TV_CHEST, '~',
	0, 20, 1, 1, 250, 0, 0, 0, 0, 0, 0, {2,3}, 7, 0, 0U },
{ "& large wooden chest", 0x15000000UL, TV_CHEST, '~',
	0, 60, 4, 1, 500, 0, 0, 0, 0, 0, 0, {2,5}, 15, 0, 0U },
{ "& small iron chest", 0x0F000000UL, TV_CHEST, '~',
	0, 100, 7, 1, 300, 0, 0, 0, 0, 0, 0, {2,4}, 25, 0, 0U },
{ "& large iron chest", 0x1F000000UL, TV_CHEST, '~',
	0, 150, 10, 1, 1000, 0, 0, 0, 0, 0, 0, {2,6}, 35, 0, 0U },
{ "& small steel chest", 0x0F000000UL, TV_CHEST, '~',
	0, 200, 13, 1, 500, 0,0, 0, 0, 0, 0, {2,4}, 45, 0, 0U },
{ "& large steel chest", 0x23000000UL, TV_CHEST, '~',
	0, 250, 16, 1, 1000, 0, 0, 0, 0, 0, 0, {2,6}, 50, 0, 0U },
{ "& rat skeleton", 0U, TV_MISC, 's', 
	0, 0, 1, 1, 10, 0, 0, 0, 0, 0, 0, {1,1}, 1, 0, 0U },
{ "& Earthling Skeleton", 0U, TV_MISC, 's',
	0, 0, 7, 1, 50, 0, 0, 0, 0, 0, 0, {1,1}, 1, 0, 0U },
{ "& Deactivated Robot", 0U, TV_MISC, 's',
	0, 0, 8, 1, 160, 0, 0, 0, 0, 0, 0, {1,1}, 1, 0, 0U },
{ "& Thorian Skeleton", 0U, TV_MISC, 's',
	0, 0, 9, 1, 50, 0, 0, 0, 0, 0, 0, {1,1}, 1, 0, 0U },
{ "& A\'durran Skeleton", 0U, TV_MISC, 's',
	0, 0, 10, 1, 45, 0, 0, 0, 0, 0, 0, {1,1}, 1, 0, 0U },
{ "& Cerebrus Skeleton", 0U, TV_MISC, 's', 
	0, 0, 11, 1, 50, 0, 0, 0, 0, 0, 0, {1,1}, 1, 0, 0U },
{ "& Ironman Skeleton", 0U, TV_MISC, 's',
	0, 0, 12, 1, 55, 0, 0, 0, 0, 0, 0, {1,1}, 1, 0, 0U },
{ "& Rigellian Skeleton", 0U, TV_MISC, 's',
	0, 0, 13, 1, 40, 0, 0, 0, 0, 0, 0, {1,1}, 1, 0, 0U },
{ "& Erandian Skeleton", 0U, TV_MISC, 's',
	0,0, 14, 1, 50, 0, 0, 0, 0, 0, 0, {1,1}, 1, 0, 0U },
{ "& Centaurian Skeleton", 0U, TV_MISC, 's',
	0, 0, 15, 1, 45, 0, 0, 0, 0, 0, 0, {1,1}, 1, 0, 0U },
{ "& Andorian Skeleton", 0U, TV_MISC, 's',  /* #65 */
	0, 0, 16, 1, 40, 0, 0, 0, 0, 0, 0, {1,1}, 1, 0, 0U },
{ "& Coleman Lantern", 0U, TV_LIGHT, '~', 5000, 50, 0, 1 , 10,
	0, 0, 0, 0, 0, 0, {0,0}, 1, 2, 0U },
{ "& flashlight", 0U, TV_LIGHT, '~', 1000, 1, 192, 1, 2, 
	0, 0, 0, 0, 0, 0, {0,0}, 1, 1, 0U },
{ "& flask~ of oil", FL_FIRE, TV_FLASK, '!', 5000, 3, 64, 1, 1,
	0, 0, 0, 0, 0, 0, {2,6}, 1, 0, 0U },	/* #68 */
{ "combat rations", 0U, TV_FOOD, ',', 10000u,10, 65, 1, 1,
	0, 0, 0, 0, 0, 0, {0,0},1 , 1, 0U },
{ "Light Riot Gear", 0U, TV_SOFT_ARMOR, '(',	/* #70 */
	0, 18, 2, 1, 80, 0, 0, 4, 0, 0 , 0,{0,0}, 2, 0, 0U },
{ "Bulletproof Vest", 0U, TV_SOFT_ARMOR, '(',
	0, 55, 4, 1, 100, 0, 0, 6, 0, 0, 0, {1,1}, 5, 0, 0U },
{ "Flak Jacket", 0U, TV_SOFT_ARMOR , '(',
	0, 430, 1, 1, 250, -1, 0, 13, 0, 0, 0, {1,4}, 24, 0, 0U },
{ "Full Combat Armor", 0U, TV_SOFT_ARMOR, '(',
	0, 720, 6, 1, 280, -2, 0, 18, 0, 0, 0, {1,4}, 34, 0, 0U },
{ "Inertial Shielding Suit", 0U, TV_SOFT_ARMOR, '(',	/* #74 */
	0, 15000, 14, 1, 300, 0, 0, 35, 0, 0, 0, {2,4}, 60, 5 , 0U},
{ "Force Shield", 0U, TV_SHIELD, '[',
	0, 10000, 17, 1, 100, 0, 0, 10, 10, 0, 0, {1,1}, 70, 6, 0U },
{ "& Cape", 0U, TV_CLOAK, ')',
	0, 3, 1, 1, 10, 0, 0, 1, 0,0, 0, {0,0}, 1, 0, 0U },
{ "& Cloak", 0U, TV_CLOAK, ')', 
	0, 40, 2, 1, 5, 0, 0, 3, 0, 0, 0, {0,0}, 20, 0, 0U },
{ "Latex Gloves", 0U, TV_GLOVES, ']',
	0, 3, 1, 1, 5, 0, 0, 1, 0, 0, 0, {0,0}, 1, 0, 0U },
{ "Leather Gloves", 0U, TV_GLOVES, ']',
	0, 35, 2, 1, 25, 0, 0, 2, 0, 0, 0, {0,0}, 12, 0, 0U },
{ "& Shield", 0U, TV_SHIELD, '[', 			/* #80 */
	0, 120, 3, 1, 100, 0, 0, 4, 0, 0, 0, {1,2}, 15, 0, 0U},

{ "Saline Solution", 0U, TV_POTION1, '!',
	20, 1, 66, 1, 4, 0, 0, 0, 0, 0, 0, {1,1}, 0, 0, 0U },
{ "Muscle Enhancement", 1U, TV_POTION1, '!',
	0, 8000, 67, 1, 4, 0, 0, 0, 0, 0, 0, {1,1}, 30, 0, 0U },
{ "Atrophy Muscles", 2U, TV_POTION1, '!',
	0, 0, 68, 1, 4, 0, 0, 0, 0, 0, 0 , {1,1}, 3, 0 , 0U },
{ "Restore Muscle Mass", 3, TV_POTION1, '!',
	0, 300, 69, 1, 4 ,0,0, 0, 0, 0, 0, {1,1}, 25, 0, 0U },
{ "Smart Drugs", 4, TV_POTION1, '!',
	0, 8000, 70, 1, 4, 0, 0, 0, 0, 0, 0, {1,1}, 32, 0, 0U },
{ "Dumb Drugs", 5, TV_POTION1, '!',
	0, 0, 71, 1, 4, 0, 0, 0, 0, 0, 0, {1,1}, 20, 0, 0U },
{ "Neural Recovery", 6, TV_POTION1, '!',
	0, 300, 72, 1, 4, 0, 0, 0, 0, 0, 0, {1,1}, 26, 0, 0U },
{ "Nanites", 7, TV_POTION1, '!',
	0, 8000, 73, 1, 4, 0, 0, 0, 0, 0 , 0, {1,1}, 32, 0, 0U },
{ "Microrobotic Parasites", 8, TV_POTION1, '!',
	0, 0, 74, 1, 4, 0, 0, 0, 0, 0, 0, {1,1}, 20, 0, 0U },
{ "Antimicrorobotics", 9, TV_POTION1, '!',
	0, 300, 75, 1, 4, 0, 0, 0, 0, 0, 0, {1,1}, 26, 0, 0U },
{ "Beauty Enhancement", 10, TV_POTION1, '!',
	0, 1000, 76, 1, 4, 0, 0, 0, 0, 0, 0, {1,1}, 20, 0, 0U },
{ "Acne", 11, TV_POTION1, '!',
	0, 0, 77, 1, 4, 0, 0, 0, 0, 0, 0, {1,1}, 20,0, 0U },
{ "Cure Acne", 12, TV_POTION1, '!',
	0, 300, 78, 1, 4, 0, 0, 0, 0, 0, 0, {1,1}, 20, 0, 0U },
{ "Clotting Agent", 13, TV_POTION1, '!',
	50, 15, 79, 1, 4, 0, 0, 0, 0, 0, 0, {1,1}, 0, 0, 0U },

{ "Clotting Agent", 13, TV_POTION1, '!',		/* #95 */
	50, 15, 79, 1, 4, 0, 0, 0, 0, 0, 0, {1,1}, 1, 0, 0U },
{ "Clotting Agent", 13, TV_POTION1, '!',
	50, 15, 79, 1, 4, 0, 0, 0, 0, 0, 0, {1,1}, 2, 0, 0U },
{ "Plasma", 14, TV_POTION1, '!',
	100, 40, 80, 1, 4, 0, 0, 0, 0, 0, 0, {1,1}, 3, 0, 0U },
{ "Emergency Blood", 15, TV_POTION1, '!',
	100, 100, 81, 1, 4, 0, 0, 0, 0, 0, 0, {1,1}, 5, 0, 0U },
{ "Endurance Builder", 16, TV_POTION1, '!',
	0, 8000, 83, 1, 4, 0, 0, 0, 0 ,0, 0, {1,1}, 30, 0, 0U },
{ "Training Drugs", 17, TV_POTION1, '!',
	0, 25000, 84, 1, 4, 0, 0, 0, 0, 0, 0, {1,1}, 65, 0, 0U },
{ "Anaesthetic", 18, TV_POTION1, '!',
	100, 0, 85, 1, 4, 0, 0, 0, 0, 0, 0, {1,1}, 0, 0, 0U},
{ "Glaucoma", 19, TV_POTION1, '!',
	0, 0, 86, 1, 4, 0, 0, 0, 0 ,0, 0, {1,1}, 0, 0, 0U },
{ "Poisons", 20, TV_POTION1, '!',
	0, 0, 88, 1, 4, 0, 0, 0, 0 ,0 ,0 ,{1,1}, 3, 0, 0U },
{ "Confusion", 21, TV_POTION1, '!',
	50, 0, 87, 1, 4, 0, 0, 0, 0, 0, 0, {1,1}, 0, 0 ,0U },
{ "Speed", 22, TV_POTION1, '!',
	0, 75, 89, 1, 4, 0, 0, 0, 0, 0, 0, {1,1}, 1, 0, 0U },
{ "Sluggishness", 23, TV_POTION1, '!',
	50, 0, 90, 1, 4, 0, 0, 0, 0, 0, 0, {1,1}, 1, 0, 0U },
{ "Dexterity Enhancement", 24, TV_POTION1, '!',
	0, 8000, 91, 1, 4, 0, 0, 0,  0, 0, 0, {1,1}, 30, 0,  0u},
{ "Dexterity Restoration", 25, TV_POTION1, '!',
	0, 300, 92, 1, 4, 0, 0, 0, 0, 0, 0, {1,1}, 25, 0, 0U },
{ "Endureance Recovery", 26, TV_POTION1, '!',
	0, 300, 93, 1, 4, 0, 0, 0, 0, 0, 0, {1,1}, 25 ,0, 0U },
{ "Amnesia", 27, TV_POTION1, '!',
	0, 0, 95, 1, 4, 0, 0, 0, 0, 0, 0, {1,1}, 10, 0, 0U },
{ "Ipecac" , 28, TV_POTION1, '!',
	0, 4, 96, 1, 4, 0, 0, 0, 0, 0, 0, {1,1}, 0, 0, 0U },
{ "Speed", 22, TV_POTION1, '!',
	0, 75, 89, 1, 4, 0, 0, 0, 0, 0, 0, {1,1}, 40, 0, 0U },
{ "Battle Stimulant", 29, TV_POTION1, '!',
	0, 35, 98, 1, 4, 0, 0, 0 ,0, 0, 0 ,{1,1}, 1, 0, 0U },
{ "Adrenaline", 30, TV_POTION1, '!',
	0, 100, 99, 1, 4, 0, 0, 0, 0, 0, 0, {1,1}, 3, 0, 0U },
{ "Courage", 31, TV_POTION1, '!',
	0, 10, 100, 1, 4, 0, 0, 0, 0, 0, 0, {1,1}, 2, 0, 0U },
{ "Restore Memories", 32, TV_POTION1, '!',
	0, 400, 101, 1, 4, 0, 0, 0, 0, 0, 0, {1,1}, 40, 0, 0U },
{ "Flame Tolerance", 33, TV_POTION1, '!',
	0, 30, 102, 1, 4, 0, 0, 0, 0, 0, 0, {1,1}, 1, 0, 0U },
{ "Cold Tolerance", 34, TV_POTION1, '!',
	0, 30, 103, 1, 4, 0, 0, 0 ,0, 0, 0, {1,1}, 1, 0, 0U },
{ "Ultravision", 35, TV_POTION1, '!',
	0, 50, 104, 1, 4, 0, 0, 0, 0 ,0, 0 ,{1,1}, 3, 0, 0U },
{ "Broad-Spectrum Antidote", 36, TV_POTION1, '!',
	0, 25, 105, 1, 4, 0, 0, 0, 0, 0, 0, {1,1}, 1, 0, 0U },
{ "Poison Cure", 37, TV_POTION1, '!', 
	0, 75, 106, 1, 4, 0, 0, 0, 0, 0, 0, {1,1}, 5, 0, 0U },
{ "Power", 38, TV_POTION1, '!',
	0, 350, 107, 1,4, 0, 0, 0, 0, 0, 0, {1,1}, 25, 0, 0U },
{ "Night Vision", 39, TV_POTION1, '!', 				/* #123 */
	0, 20, 108, 1, 4, 0, 0, 0, 0, 0, 0, {1,1}, 3, 0, 0U },
{ "Uranium Hexafluoride", 40, TV_POTION1, '!',
	0, 400, 109, 1, 4, 0, 0, 0, 0, 0, 0, {40,40}, 25, 0, 0U },

/* Below are cybernetic implants.  Shots and maxshots should contain
	charging times.	  P1 field contains difficulty.	*/
{ "Laser", WD_LT, TV_IMPLANT, '-', 					/* #125 */
	5, 200, 0, 1, 10, 0, 0, 0, 0, 0, 10, {1,1}, 2, 0, 0U },
{ "Electric Shock", WD_LT_BLTS, TV_IMPLANT, '-',
	10, 600, 1, 1, 10, 0, 0, 0, 0, 0, 10, {1,1} , 15, 0, 0U },
{ "Cold Blast", WD_FT_BLTS, TV_IMPLANT, '-',
	 15, 800, 2, 1, 10, 0, 0, 0, 0, 0, 20, {1,1}, 20, 0, 0U },
{ "Fire Blast", WD_FR_BLTS, TV_IMPLANT, '-',
	20, 1000, 3, 1, 10, 0, 0, 0, 0, 0, 20, {1,1}, 30, 0, 0U },
{ "Sonic Disruption", WD_ST_MUD, TV_IMPLANT, '-',
	15, 300, 4, 1, 10, 0, 0, 0, 0, 0, 25, {1,1}, 12, 0, 0U },
{ "Mutation", WD_POLY, TV_IMPLANT, '-',
	15, 400, 5, 1, 10, 0, 0, 0, 0, 0, 25, {1,1}, 20, 0, 0U },
{ "Monster Healing", WD_HEAL_MN, TV_IMPLANT, '-',
	1, 0, 6, 1, 10, 0, 0, 0, 0, 0, 1, {1,1}, 2, 0, 0U },
{ "Confusion", WD_CONF_MN, TV_IMPLANT, '-',
	10, 400, 9, 1, 10, 0, 0, 0, 0, 0, 10, {1,1}, 2, 0, 0U },
{ "Sleep", WD_SLEE_MN, TV_IMPLANT, '-',
	20, 500, 10, 1, 10, 0, 0, 0, 0, 0, 20, {1,1}, 2, 0, 0U },
{ "Life Drain", WD_DRAIN, TV_IMPLANT , '-',
	50, 1200, 11, 1, 10, 0, 0, 0, 0, 0 , 100, {1,1}, 2, 0, 0U },
{ "Neural Shock", WD_MAG_MIS, TV_IMPLANT, '-',
	10, 200, 13, 1, 10, 0, 0, 0, 0, 0, 10, {1,1}, 2, 0, 0U },
{ "Transport Away", WD_TELE, TV_IMPLANT, '-',
	15, 350, 16, 1, 10, 0, 0, 0, 0, 0, 20, {1,1}, 20, 0, 0U },
{ "Disarming", WD_DISARM, TV_IMPLANT, '-',
	10, 700, 17, 1, 10, 0, 0, 0, 0, 0, 25, {1,1}, 20, 0, 0U },
{ "Electric Storm", WD_LT_BALL, TV_IMPLANT, '-',
	40, 1200, 18, 1, 10, 0, 0, 0, 0, 0, 50, {1,1}, 35, 0, 0U },	/* #138 */

/*   Devices.	*/
{ "Light", ST_LIGHT, TV_STAFF, '_',
	0, 250, 0, 1, 50, 0, 0, 0, 0, 0, 0, {1,2}, 5, 0, 0U },
{ "Sonar Analysis", ST_DR_TRP_LC, TV_STAFF, '_',
	0, 350, 2, 1, 50, 0, 0, 0, 0, 0, 0, {1,2}, 10, 0, 0U },
{ "Treasure Detection", ST_TRE_LC, TV_STAFF, '_',
	0, 200, 3, 1, 50, 0, 0, 0, 0, 0, 0, {1,2}, 5, 0, 0U },
{ "Object Location", ST_OBJ_LC, TV_STAFF, '_',
	0, 200, 4, 1, 50, 0, 0, 0, 0, 0, 0, {1,2}, 5, 0, 0U },
{ "Teleport", ST_TELE, TV_STAFF, '_', 
	0, 2000, 5, 1, 50, 0, 0, 0, 0, 0, 0, {1,2}, 20, 0, 0U },
{ "Earthquake", ST_EARTH, TV_STAFF, '_',
	0, 350, 6, 1, 50, 0, 0, 0, 0, 0, 0, {1,2}, 40, 0, 0U },
{ "Signal Monsters", ST_SUMMON, TV_STAFF, '_',
	0, 0, 7, 1, 50, 0, 0, 0, 0, 0, 0, {1,2}, 10, 0, 0U },
{ "Signal Monsters", ST_SUMMON, TV_STAFF, '_',
	0, 0, 7, 1, 50, 0, 0, 0, 0, 0, 0, {1,2}, 50, 0, 0U },
{ "Massive Destruction", ST_DEST, TV_STAFF, '_',
	0, 2500, 8, 1, 50, 0, 0, 0, 0, 0, 0, {1,2}, 50, 0, 0U },
{ "Laser Rosette", ST_STAR, TV_STAFF, '_',
	0, 800, 9, 1, 50, 0, 0, 0, 0, 0, 0, {1,2}, 20, 0, 0U },
{ "Hypnosis", ST_SLEE_MN, TV_STAFF, '_',
	0, 700, 12, 1, 50, 0, 0, 0, 0, 0, 0, {1,2}, 10, 0, 0U },
{ "Minor Healing", ST_CURE_LT, TV_STAFF, '_',
	0, 350, 13, 1, 50, 0, 0, 0, 0, 0, 0, {1,2}, 5, 0, 0U },		/* #150 */
{ "Speed", ST_SPEED, TV_STAFF, '_',
	0, 1000, 15, 1, 50, 0, 0, 0, 0, 0, 0, {1,2}, 40, 0, 0U },
{ "Slowness", ST_SLOW, TV_STAFF, '_',
	0, 0, 16, 1, 50, 0, 0, 0, 0, 0, 0, {1,2}, 40, 0, 0U },
/* Remove Curse? */
{ "Blackness", ST_DARK, TV_STAFF, '_', 
	0, 0, 22, 1, 50, 0, 0, 0, 0, 0, 0, {1,2}, 50, 0, 0U },

{ "Blackness", ST_DARK, TV_STAFF, '_', 
	0, 0, 22, 1, 50, 0, 0, 0, 0, 0, 0, {1,2}, 5, 0, 0U },
{ "Exterminate", ST_GENOCIDE, TV_STAFF, '_',
	0, 3500, 23, 1, 50, 0, 0, 0, 0, 0, 0, {1,2}, 35, 4, 0U },
{ "Identification", ST_IDENTIFY, TV_STAFF, '_', 
	0, 400, 26, 1, 50, 0, 0, 0, 0, 0, 0, {1,2}, 10, 0, 0U },
{ "Major Healing", ST_HEALING, TV_STAFF, '_',
	0, 5000, 29, 1, 50, 0, 0, 0, 0, 0, 0, {1,2}, 35, 2, 0U },	/* #157 */

/* Store items go here. */
{ "& K-Ration~", 0U, TV_FOOD, ',', 10000u, 5, 64, 1, 1, 
	0, 0, 0, 0, 0, 0, {0,0}, 1, 1, 0U },	/* MDO+1 */
{ "& Coleman Lantern", 0U, TV_LIGHT, '~', 5000, 50, 0, 1, 10,
	0, 0, 0, 0, 0, 0, {0,0}, 1, 2, 0U }, 	/* MDO+2 */
{ "& flashlight", 0U, TV_LIGHT, '~', 1000, 1, 192, 10, 2,
	0, 0, 0, 0, 0, 0, {0,0}, 1, 1, 0U }, 	/* MDO+3 */
{ "& sonic disruptor", TR_TUNNEL, TV_DIGGING, '~', 3, 3, 0,  1, 10, 
	0, 0, 0, 0, 0, 0, {1,1}, 1, 2, 0U }, 	/* MDO+4 */
{ "& .22 caliber shot~", 0U, TV_AMMO, '{', 0, 1, 65+AMMO_CONST, 10, 1,
	0, 0, 0, 0, 0, 0, {0,0}, 2, 1, 0U },	/* MDO+5 */
{ "& flask~ of oil", FL_FIRE, TV_FLASK, '!', 5000, 3, 64, 10, 1, 
	0, 0, 0, 0, 0, 0, {2,6}, 1, 0, 0U },    /* MDO+6 */
{ "Implant cybernetics in body", 0U, TV_SERVICE, ' ', 0, 1000, 
	0, 1, 0, 0, 0, 0, 0, 0, 0, {0,0}, 0, 0, 0U },	/* MDO+7 */
{ "Remove cyberware", 0U, TV_SERVICE, ' ', 0, 500, 1,
	1, 0, 0, 0, 0, 0, 0, 0, {0,0}, 0, 0, 0U },
{ "Lift curse", 0U, TV_SERVICE, ' ', 0, 5000, 2,
	1, 0, 0, 0, 0, 0, 0, 0, {0,0}, 0, 0, 0U },

/* start doors */
/* Secret door must have same subval as closed door in	*/
/* TRAP_LISTB.	See CHANGE_TRAP. Must use & because of stone_to_mud. */
{"& open door"			,0x00000000L, TV_OPEN_DOOR, '\'',
    0,	   0,	1,   1,   0,	0,   0,	 0,   0, 0, 0, {1,1}	,  0, 0},
{"& closed door"		,0x00000000L, TV_CLOSED_DOOR, '+',
    0,	   0,	19,   1,   0,	0,   0,	 0,   0, 0, 0, {1,1}	,  0, 0},
{"& secret door"		,0x00000000L, TV_SECRET_DOOR, '#',
    0,	   0,	19,   1,   0,	0,   0,	 0,   0, 0, 0, {1,1}	,  0, 0},
/* end doors */
/* stairs */
{"an up staircase "		,0x00000000L, TV_UP_STAIR, '<',
    0,	   0,	1,   1,   0,	0,   0,	 0,   0, 0, 0, {1,1}	,  0, 0},
{"a down staircase"		,0x00000000L, TV_DOWN_STAIR, '>',
    0,	   0,	1,   1,   0,	0,   0,	 0,   0, 0, 0, {1,1}	,  0, 0},
/* store door */
/* Stores are just special traps		*/
{"The General Store"		,0x00000000L, TV_STORE_DOOR, '1',
    0,	   0, 101,   1,   0,	0,   0,	 0,   0, 0, 0, {0,0}	,  0, 0},
{"The Armoury"			,0x00000000L, TV_STORE_DOOR, '2',
    0,	   0, 102,   1,   0,	0,   0,	 0,   0, 0, 0, {0,0}	,  0, 0},
{"The Weapon Smiths"		,0x00000000L, TV_STORE_DOOR, '3', /* +30 */
    0,	   0, 103,   1,   0,	0,   0,	 0,   0, 0, 0, {0,0}	,  0, 0},
{"The Temple"			,0x00000000L, TV_STORE_DOOR, '4',
    0,	   0, 104,   1,   0,	0,   0,	 0,   0, 0, 0, {0,0}	,  0, 0},
{"The Alchemy Shop"			,0x00000000L, TV_STORE_DOOR, '5',
    0,	   0, 105,   1,   0,	0,   0,	 0,   0, 0, 0, {0,0}	,  0, 0},
{"The Magic Shop"			,0x00000000L, TV_STORE_DOOR, '6',
    0,	   0, 106,   1,   0,	0,   0,	 0,   0, 0, 0, {0,0}	,  0, 0},
{"The Black Market"			,0x00000000L, TV_STORE_DOOR, '7',
    0,	   0, 107,   1,   0,	0,   0,	 0,   0, 0, 0, {0,0}	,  0, 0},
{"Your Home"  			,0x00000000L, TV_STORE_DOOR, '8',
    0,	   0, 108,   1,   0,	0,   0,	 0,   0, 0, 0, {0,0}	,  0, 0},
/* end store door */
/* Traps are just Nasty treasures.				*/
/* Traps: Level represents the relative difficulty of disarming;	*/
/* and P1 represents the experienced gained when disarmed*/
{"an open pit"			,0x00000000L, TV_VIS_TRAP, ' ',	/* +36 */
    1,	   0,	1,   1,   0,	0,   0,	 0,   0, 0, 0, {2,6}	,50, 0},
{"an arrow trap"		,0x00000000L, TV_INVIS_TRAP, '^',
    3,	   0,	2,   1,   0,	0,   0,	 0,   0, 0, 0, {1,8}	,90, 0},
{"a covered pit"		,0x00000000L, TV_INVIS_TRAP, '^',
    2,	   0,	3,   1,   0,	0,   0,	 0,   0, 0, 0, {2,6}	,60, 0},
{"a trap door"			,0x00000000L, TV_INVIS_TRAP, '^',
    5,	   0,	4,   1,   0,	0,   0,	 0,   0, 0, 0, {2,8}	,75, 0},
{"a gas trap"			,0x00000000L, TV_INVIS_TRAP, '^',
    3,	   0,	5,   1,   0,	0,   0,	 0,   0, 0, 0, {1,4}	,95, 0},
{"a loose rock"			,0x00000000L, TV_INVIS_TRAP, ';',
    0,	   0,	6,   1,   0,	0,   0,	 0,   0, 0, 0, {0,0}	,10, 0},
{"a dart trap"			,0x00000000L, TV_INVIS_TRAP, '^',
    5,	   0,	7,   1,   0,	0,   0,	 0,   0, 0, 0, {1,4}	,110, 0},
{"a strange rune"		,0x00000000L, TV_INVIS_TRAP, '^',
    5,	   0,	8,   1,   0,	0,   0,	 0,   0, 0, 0, {0,0}	,90, 0},
{"some loose rock"		,0x00000000L, TV_INVIS_TRAP, '^',
    5,	   0,	9,   1,   0,	0,   0,	 0,   0, 0, 0, {2,6}	,90, 0},
{"a gas trap"			,0x00000000L, TV_INVIS_TRAP, '^',
   10,	   0,	10,   1,   0,	0,   0,	 0,   0, 0, 0, {1,4}	,105, 0},
{"a strange rune"		,0x00000000L, TV_INVIS_TRAP, '^', /* +46 */
    5,	   0,	11,   1,   0,	0,   0,	 0,   0, 0, 0, {0,0}	,90, 0},
{"a blackened spot"		,0x00000000L, TV_INVIS_TRAP, '^',
   10,	   0,	12,   1,   0,	0,   0,	 0,   0, 0, 0, {4,6}	,110, 0},
{"some corroded rock"		,0x00000000L, TV_INVIS_TRAP, '^',
   10,	   0,	13,   1,   0,	0,   0,	 0,   0, 0, 0, {4,6}	,110, 0},
{"a gas trap"			,0x00000000L, TV_INVIS_TRAP, '^',
    5,	   0,	14,   1,   0,	0,   0,	 0,   0, 0, 0, {2,6}	,105, 0},
{"a gas trap"			,0x00000000L, TV_INVIS_TRAP, '^',
    5,	   0,	15,   1,   0,	0,   0,	 0,   0, 0, 0, {1,4}	,110, 0},
{"a gas trap"			,0x00000000L, TV_INVIS_TRAP, '^',
    5,	   0,	16,   1,   0,	0,   0,	 0,   0, 0, 0, {1,8}	,105, 0},
{"a dart trap"			,0x00000000L, TV_INVIS_TRAP, '^',
    5,	   0,	17,   1,   0,	0,   0,	 0,   0, 0, 0, {1,8}	,110, 0},
{"a dart trap"			,0x00000000L, TV_INVIS_TRAP, '^',/* +53 */
    5,	   0,	18,   1,   0,	0,   0,	 0,   0, 0, 0, {1,8}	,110, 0},
/* rubble */
{"some rubble"			,0x00000000L, TV_RUBBLE, ':',	/* +54 */
    0,	   0,	1,   1,   0,	0,   0,	 0,   0, 0, 0, {0,0}	,  0, 0},
/* mush */
{"& pint~ of fine grade mush"	,0x00000000L, TV_FOOD, ',',	/* +55 */
 3500,	   0,  97,   1,   1,   0,   0,   0,   0, 0, 0, {1,1}  ,  1, 0},
/* Special trap	*/
{"a strange rune"		,0x00000000L, TV_VIS_TRAP, '^',	/* +56 */
    0,	   0,	99,   1,   0,	0,   0,	 0,   0, 0, 0, {0,0}	, 10, 0},
/* Gold list (All types of gold and gems are defined here)	*/
{"silver"			,0x00000000L, TV_GOLD, '$',	/* +57 */
    0,	   3,	1,   1,   0,	0,   0,	 0,   0, 0, 0, {0,0}	  ,  1, 0},
{"silver"			,0x00000000L, TV_GOLD, '$',
    0,	   4,	2,   1,   0,	0,   0,	 0,   0, 0, 0, {0,0}	  ,  1, 0},
{"silver"			,0x00000000L, TV_GOLD, '$',
    0,	   5,	3,   1,   0,	0,   0,	 0,   0, 0, 0, {0,0}	  ,  1, 0},
{"gold"			,0x00000000L, TV_GOLD, '$',
    0,	   6,	4,   1,   0,	0,   0,	 0,   0, 0, 0, {0,0}	  ,  1, 0},
{"gold"			,0x00000000L, TV_GOLD, '$',
    0,	   7,	5,   1,   0,	0,   0,	 0,   0, 0, 0, {0,0}	  ,  1, 0},
{"gold"			,0x00000000L, TV_GOLD, '$',
    0,	   8,	6,   1,   0,	0,   0,	 0,   0, 0, 0, {0,0}	  ,  1, 0},
{"garnets"			,0x00000000L, TV_GOLD, '*',
    0,	   9,	7,   1,   0,	0,   0,	 0,   0, 0, 0, {0,0}	  ,  1, 0},
{"garnets"			,0x00000000L, TV_GOLD, '*',
    0,	  10,	8,   1,   0,	0,   0,	 0,   0, 0, 0, {0,0}	  ,  1, 0},
{"platinum"				,0x00000000L, TV_GOLD, '$',
    0,	  12,	9,   1,   0,	0,   0,	 0,   0, 0, 0, {0,0}	  ,  1, 0},
{"platinum"				,0x00000000L, TV_GOLD, '$',      /* +66*/
    0,	  14,	10,   1,   0,	0,   0,	 0,   0, 0, 0, {0,0}	  ,  1, 0},
{"platinum"				,0x00000000L, TV_GOLD, '$',
    0,	  16,	11,   1,   0,	0,   0,	 0,   0, 0, 0, {0,0}	  ,  1, 0},
{"opals"			,0x00000000L, TV_GOLD, '*',
    0,	  18,	12,   1,   0,	0,   0,	 0,   0, 0, 0, {0,0}	  ,  1, 0},
{"sapphires"			,0x00000000L, TV_GOLD, '*',
    0,	  20,	13,   1,   0,	0,   0,	 0,   0, 0, 0, {0,0}	  ,  1, 0},
{"platinum"				,0x00000000L, TV_GOLD, '$',
    0,	  24,	14,   1,   0,	0,   0,	 0,   0, 0, 0, {0,0}	  ,  1, 0},
{"rubies"			,0x00000000L, TV_GOLD, '*',
    0,	  28,	15,   1,   0,	0,   0,	 0,   0, 0, 0, {0,0}	  ,  1, 0},
{"diamonds"			,0x00000000L, TV_GOLD, '*',
    0,	  32,	16,   1,   0,	0,   0,	 0,   0, 0, 0, {0,0}	  ,  1, 0},
{"emeralds"			,0x00000000L, TV_GOLD, '*',
    0,	  40,	17,   1,   0,	0,   0,	 0,   0,0, 0,  {0,0}	  ,  1, 0},
{"iridium"			,0x00000000L, TV_GOLD, '$',	/* +74 */
    0,	  80,	18,   1,   0,	0,   0,	 0,   0, 0, 0, {0,0}	  ,  1, 0},
/* nothing, used as inventory place holder */
/* must be stackable, so that can be picked up by inven_carry */
{"nothing"			,0x00000000L,  TV_NOTHING, ' ',	/* +75 */
   0,       0,  64,   0,   0,   0,   0,   0,   0, 0, 0, {0,0}    ,  0, 0},
/* these next two are needed only for the names */
{"& ruined chest"		,0x00000000L,   TV_CHEST, '~',	/* +76 */
   0,	   0,	0,   1, 250,	0,   0,	 0,   0, 0, 0, {0,0}	,  0, 0},
{""				,0x00000000L,  TV_NOTHING, ' ',	/* +77*/
   0,       0,   0,   0,   0,   0,   0,   0,   0, 0, 0, {0,0}  ,  0, 0},
{"Power named Narya"		,0x01C008BFL,	TV_RING, '=',	/* +78 */
    1,	100000,	31,   1,   2,	0,   0,	 0,   0, 0, 0, {0,0}	, 80, 0, 0x02000848L},

/* For some reason THIS is where SPECIAL_OBJ should point */

{"Power named Nenya"		,0x05C008BFL,	TV_RING, '=',	/*467*/
    2,	200000,	32,   1,   2,	0,   0,	 0,   0, 0, 0, {0,0}	, 90, 0, 0x02000888L},
{"Power named Vilya"		,0x01C008BFL,	TV_RING, '=',	/*468*/
    3,	300000,	33,   1,   2,	0,   0,	 0,   0, 0, 0, {0,0}	, 100, 0, 0x02000A08L},
{"Power"			,0x83A80A3FL,	TV_RING, '=',	/*469*/
    5,5000000,	34,   1,   2,	0,   0,	 0,   0, 0, 0, {0,0}	, 110, 0, 0x02000BC0L},
{"The Phial of Galadriel"        ,0x00000000L,	TV_LIGHT, '~',	   /*470*/
    0,	10000,	4,    1,  10,	0,   0,	 0,   0, 0, 0, {1,1}	,  1, 0, 0x02000C00L},
{"Ingwe"                         ,0x43380024L,  TV_AMULET, '\"',    /*471*/
    3,  90000,  10,   1,   3,   0,   0,  0,   0, 0, 0, {0,0}  ,  65, 0,0x02000800L},
{"Carlammas"                     ,0x00080010L,  TV_AMULET, '\"',    /*472*/
    2,  60000,  11,   1,   3,   0,   0,  0,   0, 0, 0, {0,0}  ,  50, 0,0x02000800L},
{"The Star of Elendil"		 ,0x00000000L,  TV_LIGHT,   '~',   /*473*/
    0,  25000,  5,    1,   5,   0,   0,  0,   0, 0, 0, {1,1}  ,  25, 0,0x02000C00L},
{"The Arkenstone of Thrain"	 ,0x0100000CL,  TV_LIGHT,   '~',   /*474*/
    2,  60000,  6,    1,   5,   0,   0,  0,   0, 0, 0, {1,1}  ,  60, 0,0x02000C08L},
{"Tulkas"			 ,0x00000019L,  TV_RING, '=',     /*475*/
    4,  150000, 35,   1,   2,   0,   0,  0,   0, 0, 0, {0,0}  ,  90, 0,0x02000800L},
{"the Dwarves"			 ,0x41800811L,  TV_AMULET,  '\"',  /*476*/
    3,  75000,  12,   1,   3,   0,   0,  0,   0, 0, 0, {0,0},    70, 0,0x02000000L},
{"Barahir"			 ,0x1000003FL,  TV_RING,   '=',   /*477*/
    1,  65000,  36,   1,   2,   0,   0,  0,   0, 0, 0, {0,0},    50, 0,0x02000000L}
};
#endif

char *special_names[SN_ARRAY_SIZE] = {
	NULL,			"of Resistance",	"of Resist Acid",
	"of Resist Fire",	"of Resist Cold",	"of Resist Lightning",
	"(Federation)",	"(Syndicate)",		"(Armor Piercing)",
	"Adamantium",	"of Accuracy",		"of Wounding",
	"of Flame",	        "of Frost",	        "of Free Action",
	"of Slaying",		"of Death","of Clumsiness",	"of Weakness",
	"of Slow Descent",	"of Speed",		"of Stealth",
	"of Slowness",		"of Noise",		"of Great Mass",
	"of Intelligence",	"Mechanical",		"of Vision",
	"of Might",		"of Lordliness",	"Betelgeusean",
	"of Beauty",		"of Seeing",		"of Regeneration",
	"of Stupidity",		"of Dullness",		"of Blindness",
	"of Timidness",		"of Teleportation",	"of Ugliness",
	"of Protection",	"of Irritation",	"of Vulnerability",
	"of Enveloping",	"of Fire",	"fluff 1",	
	"Black Talon",	"Diamond Tip", 		"Uranium Tipped",
	"High Density", "Monofilament",		"of Vulcan",
	"(Empty)",		"(Locked)",		"fluff 2"
	"(Poison Needle)",	"(Gas Trap)",		"(Explosion Device)",
	"(Signalling Device)",	"(Multiple Traps)",	"(Disarmed)",
	"(Unlocked)",	"fluff 3", "fluff 4",
	"of DOOM", "Virtural Sword",	"fluff 5",
	"named Grond",	"fluff 6",		"fluff 7" 
};

int16 sorted_objects[MAX_DUNGEON_OBJ];

/* Identified objects flags					*/
int8u object_ident[OBJECT_IDENT_SIZE];
int16 t_level[MAX_OBJ_LEVEL+1];
inven_type t_list[MAX_TALLOC];
inven_type inventory[INVEN_ARRAY_SIZE];

/* Treasure related values					*/
int16 inven_ctr = 0;		/* Total different obj's	*/
int16 inven_weight = 0;		/* Cur carried weight	*/
int16 equip_ctr = 0;		/* Cur equipment ctr	*/
int16 tcptr;			/* Cur treasure heap ptr	*/
